const http = require('http');
const https = require('https');
const config = require('config');
const AWS = require('aws-sdk');
const HttpsProxyAgent = require('https-proxy-agent');
const ms = require('ms');
const { first } = require('lodash');
const createError = require('http-errors');
const consts = require('../../utils/consts');
const getCredentials = require('../cloud-manager/credentials-operations');
const getLogger = require('../../utils/logger');
const { getProxy } = require('../cloud-manager/config-operations');

const logger = getLogger();

async function createHttpOptions(provider, host) {
    const useProxy = config.get(`protocols.${provider}.use-proxy`);
    const rejectInvalidCertificates = config.get(`protocols.${provider}.reject-invalid-certificates`);
    const maxVersion = config.get(`protocols.${provider}.max-tls-version`) ? config.get(`protocols.${provider}.max-tls-version`) : undefined;
    const proxy = await getProxy();
    let agent;
    if (proxy && useProxy) {
        agent = new HttpsProxyAgent(proxy.toString());
    } else if (host && host.startsWith('http://')) {
        agent = new http.Agent();
    } else {
        agent = new https.Agent({ rejectUnauthorized: rejectInvalidCertificates, maxVersion });
    }
    return {
        agent,
        timeout: ms(config.get(`protocols.${provider}.timeout`)),
        connectTimeout: ms(config.get(`protocols.${provider}.connectTimeout`))
    };
}

async function getClient(provider, options) {
    logger.debug('Getting S3', { provider, options });
    switch (provider) {
        case consts.S3: {
            const { region } = options;
            const credentials = await getCredentials(consts.AWS);
            const httpOptions = await createHttpOptions(consts.S3);

            return new AWS.S3({
                maxRetries: config.get('protocols.S3.retries'),
                credentials,
                region: region || consts.DEFAULT_AWS_REGION,
                httpOptions
            });
        }
        case consts.SGWS: {
            const {
                host, port, accessKeyId, secretAccessKey, isS3EndPoint
            } = options;
            const httpOptions = await createHttpOptions(consts.SGWS, host);

            return new AWS.S3({
                endpoint: (port === '443') ? `${host}` : `${host}:${port}`,
                accessKeyId,
                secretAccessKey,
                httpOptions,
                s3BucketEndpoint: isS3EndPoint,
                s3ForcePathStyle: true,
                correctClockSkew: true
            });
        }
        default: {
            throw createError('Received unknown provider', provider);
        }
    }
}

async function headBucket(s3, bucket) {
    logger.debug('Heading bucket');

    return new Promise((resolve, reject) => {
        s3.headBucket({ Bucket: bucket })
            .on('complete', res => {
                if (res.httpResponse.headers['x-amz-bucket-region']) {
                    resolve(res.httpResponse.headers['x-amz-bucket-region']);
                } else {
                    const { statusCode } = res.httpResponse;
                    reject(new Error(`Got unexpected status code ${statusCode} - for headBucket operation on bucket ${bucket}`));
                }
            }).send();
    });
}

async function createBucket(s3, params) {
    return s3.createBucket(params).promise();
}

async function getBucketLifecycleConfiguration(bucket) {
    logger.info('Getting bucket lifecycle configuration:', bucket);
    const s3 = await getClient(consts.S3, { region: consts.DEFAULT_AWS_REGION });

    try {
        const data = await s3.getBucketLifecycleConfiguration({ Bucket: bucket }).promise();
        const transitionRules = data.Rules.filter(rule => rule.Status === 'Enabled' && rule.Transitions.length);
        const rule = transitionRules.find(ruleObj => ruleObj.ID.startsWith('CT')) || first(transitionRules);
        const storageClass = rule ? rule.Transitions[0].StorageClass : consts.STANDARD_AWS;
        return { storageClass };
    } catch (err) {
        if (err.code === 'NoSuchLifecycleConfiguration') {
            return { storageClass: consts.STANDARD_AWS };
        }
        if (err.code === 'NoSuchBucket') {
            throw createError(403, `${bucket} does not exists`);
        }
        logger.error('Failed to receive storage class', err);
        return { storageClass: consts.STANDARD_AWS };
    }
}

// Relevant only for standard_ia, ia_single and intelligent
async function putBucketLifecycleConfiguration(bucket, storageClass) {
    logger.info('Put bucket lifecycle configuration:', { bucket, storageClass });
    const s3 = await getClient(consts.S3, { region: consts.DEFAULT_AWS_REGION });

    return s3.putBucketLifecycleConfiguration({
        Bucket: bucket,
        LifecycleConfiguration: {
            Rules: [{
                Filter: {
                    Prefix: ''
                },
                Status: 'Enabled',
                ID: `CT-${storageClass}`,
                Transitions: [{
                    Days: 30,
                    StorageClass: storageClass
                }]
            }]
        }
    }).promise();
}

async function listBuckets(s3) {
    return s3.listBuckets().promise();
}

async function putObject(s3, bucket, key, body) {
    return s3.putObject({ Bucket: bucket, Key: key, Body: body }).promise();
}

async function getObject(s3, bucket, key) {
    const data = await s3.getObject({ Bucket: bucket, Key: key }).promise();
    return data.Body;
}

async function deleteObject(s3, bucket, key) {
    return s3.deleteObject({ Bucket: bucket, Key: key }).promise();
}

module.exports = {
    putObject,
    getObject,
    deleteObject,
    headBucket,
    getClient,
    createBucket,
    getBucketLifecycleConfiguration,
    putBucketLifecycleConfiguration,
    listBuckets
};
