const config = require('config');
const got = require('got');
const ms = require('ms');
const { get } = require('lodash');
const createError = require('http-errors');
const Agent = require('agentkeepalive').HttpsAgent;
const { parseXMLToJSON, parseZapiXmlResponseToJSON } = require('../../utils/utils');
const getLogger = require('../../utils/logger');

const logger = getLogger('got');
const agent = new Agent({ rejectUnauthorized: false });

const gotInstance = got.extend({
    retry: {
        limit: config.get('got.retries'),
        methods: ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS', 'TRACE']
    },
    timeout: {
        lookup: ms(config.get('got.lookup-timeout')),
        connect: ms(config.get('got.connect-timeout')),
        response: ms(config.get('got.response-timeout'))
    },
    headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
        Accept: 'application/xml'
    },
    resolveBodyOnly: true,
    responseType: 'text',
    hooks: {
        beforeRequest: [
            request => logger.info(`${request.method} request to: ${request.url.origin}${request.url.pathname}  body:`, request.body)
        ],
        afterResponse: [
            response => {
                logger.info(`${response.req.method} response status: ${response.statusCode} from: ${get(response, 'request.options.url.origin')}${get(response, 'request.options.url.pathname')} body:`, response.body, 'took', get(response, 'timings.phases.total'));
                return response;
            }
        ],
        beforeError: [
            error => {
                logger.error('Received error status:', get(error, 'response.status', get(error, 'response.statusCode', error.code || 500)), '\nmessage', get(error, 'response.body'), '\nRaw error:', error, '\ntook', get(error, 'timings.phases.total'));
                if (error.options.url.pathname.includes('/servlets/netapp.servlets.admin.XMLrequest_filer') && error.response) {
                    // error is on error.response.body
                    const errorBody = error.response.body;
                    const jsonBody = parseXMLToJSON(errorBody);

                    return createError(error.response.statusCode, jsonBody.html.body.p._text);
                }
                if (error.response) {
                    // applicable for ParseError, HTTPError, got.MaxRedirectsError
                    return createError(error.response.statusCode, error.response.body);
                }

                return createError(error.code, error.message);
            }
        ],
        beforeRetry: [
            (options, error, retryCount) => logger.info(`Before retry number: ${retryCount} on request to: ${options.url.origin}${options.url.pathname}, received Error: ${error.code}, ${error.name}`)
        ]
    }
});

async function zapiRequest(username, password, ip, body) {
    const url = `https://${ip}/servlets/netapp.servlets.admin.XMLrequest_filer`;

    // basic authorization parameters should be passed as parameters to got options
    // currently it does not work because of the following Node issues:
    // https://github.com/nodejs/node/pull/31450
    // https://github.com/nodejs/node/issues/31439
    //
    // got issue awaiting above issues:
    // https://github.com/sindresorhus/got/issues/1169
    //
    // once these issues are resolved we can remove the header and use username and password parameters again
    const data = await gotInstance.post(url, {
        headers: {
            Authorization: `Basic ${Buffer.from(`${username}:${password}`).toString('base64')}`
        },
        body,
        agent: {
            https: agent
        }
    });

    return parseZapiXmlResponseToJSON(data);
}

module.exports = zapiRequest;
