const Promise = require('bluebird');
const { compact } = require('lodash');
const { DEFAULT_AWS_REGION, S3, SGWS } = require('../utils/consts');
const {
    headBucket, getClient, listBuckets, createBucket,
    deleteObject, putObject, getObject
} = require('../lib/aws/s3');
const getLogger = require('../utils/logger');

const logger = getLogger();

async function listS3Buckets() {
    logger.info('Listing S3 buckets');
    const s3 = await getClient(S3, { regoin: DEFAULT_AWS_REGION });
    const response = await listBuckets(s3);

    const buckets = await Promise.map(response.Buckets, async bucket => {
        const { Name: name } = bucket;
        try {
            const region = await headBucket(s3, name);
            return { name, region };
        } catch (err) {
            logger.warn(`Failed to get bucket location/tagging, bucket name: ${name}`);
        }
    }, { concurrency: 50 });

    return compact(buckets);
}

async function listSgwsBuckets(options) {
    logger.info('Listing SGWS buckets', options);
    const sgws = await getClient(SGWS, options);
    const response = await listBuckets(sgws);
    return response.Buckets.map(bucket => ({ name: bucket.Name }));
}

async function createS3Bucket(name, region) {
    logger.info('Creating S3 bucket:', { name, region });
    const s3 = await getClient(S3, { region });
    const params = {
        Bucket: name
    };

    if (region !== DEFAULT_AWS_REGION) {
        params.CreateBucketConfiguration = {
            LocationConstraint: region
        };
    }

    return createBucket(s3, params);
}

async function createSgwsBucket(name, options) {
    logger.info('Creating SGWS bucket:', { name, options });
    const sgws = await getClient(SGWS, options);
    const params = {
        Bucket: name
    };
    return createBucket(sgws, params);
}

async function deleteSgwsObject(bucket, key, options) {
    logger.info('Delete sgws object', { key, bucket, options });
    const sgws = await getClient(SGWS, options);
    return deleteObject(sgws, bucket, key);
}

async function putSgwsObject(bucket, key, body, options) {
    logger.info('Put sgws object', { key, bucket, options });
    const sgws = await getClient(SGWS, options);
    return putObject(sgws, bucket, key, body);
}

async function getSgwsObject(bucket, key, options) {
    logger.info('Get sgws object', { key, bucket, options });
    const sgws = await getClient(SGWS, options);
    return getObject(sgws, bucket, key);
}

module.exports = {
    getSgwsObject,
    putSgwsObject,
    deleteSgwsObject,
    listSgwsBuckets,
    listS3Buckets,
    createS3Bucket,
    createSgwsBucket
};
