const log4js = require('log4js');
const config = require('config');
const stackTrace = require('stack-trace');
const path = require('path');
const { readFileSync } = require('fs');
const { format } = require('util');
const chokidar = require('chokidar');
const {
    isError, cloneDeep, isString
} = require('lodash');
const { getRequestId, getServiceRequestId } = require('./async-local-storage');
const hideSecretsHelper = require('./hide-secrets');

const LOG4JS_CONFIG_FILE = config.get('paths.log4js-config-file');

function hideSecrets(logEventData) {
    try {
        const jsonObj = JSON.parse(logEventData);
        hideSecretsHelper(jsonObj);

        return JSON.stringify(jsonObj);
    } catch (err) {
        if (!isString(logEventData)) {
            hideSecretsHelper(logEventData);
        }

        return logEventData;
    }
}

function configureLayoutWithToken() {
    const configFileContent = JSON.parse(readFileSync(config.get('paths.log4js-config-file')));
    Object.keys(configFileContent.appenders).forEach(appenderType => {
        const { layout } = configFileContent.appenders[appenderType];

        if (layout) {
            const { pattern } = layout;
            if (pattern) {
                layout.tokens = {
                    serviceRequestId: () => {
                        const requestId = getServiceRequestId();
                        return requestId || 'system';
                    },
                    requestId: () => {
                        const requestId = getRequestId();
                        return requestId || 'system';
                    },
                    message: logEvent => format(...logEvent.data.map(obj => {
                        if (isError(obj)) {
                            return hideSecrets(obj);
                        }
                        // used to return hideSecrets(obj) without cloneDeep.
                        const objClone = cloneDeep(obj);
                        return hideSecrets(objClone);
                    }))
                };
            }
        }
    });
    log4js.configure(configFileContent);
}

configureLayoutWithToken();

function getLogger(category = 'server') {
    const logger = log4js.getLogger(category);
    const trace = stackTrace.parse(new Error('something went wrong'));
    logger.addContext('module', trace[1].getFileName().replace(/(^.*[\\/]src[\\/])|(^.*[\\/]cloudtiering[\\/])/, '').split(path.sep).join('.'));
    return logger;
}

chokidar.watch(LOG4JS_CONFIG_FILE, { usePolling: { interval: 3000 } })
    .on('change', () => {
        try {
            getLogger().fatal('Got LOG4JS configuration change identified, LOG4JS is shutting down...');
            log4js.shutdown(() => {
                getLogger().fatal('Reloading LOG4JS configuration');
                configureLayoutWithToken();
                getLogger().fatal('LOG4JS configuration reloaded!');
            });
        } catch (err) {
            getLogger().fatal('Failed to change LOG4JS configuration', err);
        }
    });

module.exports = getLogger;
