const {
    AGENT_LIST_SGWS_BUCKETS, AGENT_CREATE_SGWS_BUCKET, AGENT_SGWS_PATH

} = require('../../utils/consts');
const {
    createSgwsBucketRequestSchema, sgwsBucketsResponse, sgwsBucket, putSgwsObjectRequestSchema
} = require('../../joi-schemas/agent-schemas');
const { gotInstanceForAgent } = require('../../utils/got-utils');
const getLogger = require('../../utils/logger/logger');
const { validateResponse, validateRequest } = require('../../utils/schema-validation');
const { createAgentPath, createHeadersForAgent } = require('../../utils/utils');

const logger = getLogger();

async function getSgwsBuckets(agentId, host, port, accessKey, secretKey) {
    logger.info('Get AWS buckets', { agentId });
    const data = await gotInstanceForAgent.get(`${createAgentPath(AGENT_LIST_SGWS_BUCKETS)}?host=${host}&port=${port}`, createHeadersForAgent(agentId,
        { accessKey, secretKey }));
    validateResponse(data, sgwsBucketsResponse);
    return data;
}

async function createSgwsBucket(agentId, name, host, port, accessKey, secretKey) {
    try {
        logger.info('Create SGWS bucket', {
            agentId, name, host, port, accessKey, secretKey
        });
        const requestData = {
            name, host, port
        };
        validateRequest(requestData, createSgwsBucketRequestSchema);
        const data = await gotInstanceForAgent.post(createAgentPath(AGENT_CREATE_SGWS_BUCKET), {
            json: requestData,
            ...createHeadersForAgent(agentId,
                { accessKey, secretKey })
        });
        validateResponse(data, sgwsBucket);
        return data;
    } catch (err) {
        logger.error('Failed to create bucket on AWS', err);
        throw err;
    }
}

async function putSgwsObject(agentId, bucket, key, keyBody, accessKey, secretKey, host, port) {
    logger.info('Put SGWS object', {
        agentId, host, port, bucket, accessKey, secretKey
    });
    const requestData = {
        host, port, keyBody
    };
    validateRequest(requestData, putSgwsObjectRequestSchema);
    await gotInstanceForAgent.put(createAgentPath(`${AGENT_SGWS_PATH}${bucket}/${key}`), {
        json: requestData,
        ...createHeadersForAgent(agentId,
            { accessKey, secretKey })
    });
}

async function getSgwsObject(agentId, bucket, key, accessKey, secretKey, host, port) {
    logger.info('Get SGWS object', {
        agentId, host, port, bucket, accessKey, secretKey
    });
    await gotInstanceForAgent.get(`${createAgentPath(`${AGENT_SGWS_PATH}${bucket}/${key}`)}?host=${host}&port=${port}`,
        createHeadersForAgent(agentId,
            { accessKey, secretKey }));
}

async function deleteSgwsObject(agentId, bucket, key, accessKey, secretKey, host, port) {
    logger.info('Delete SGWS object', {
        agentId, host, port, bucket, accessKey, secretKey
    });
    await gotInstanceForAgent.delete(`${createAgentPath(`${AGENT_SGWS_PATH}${bucket}/${key}?host=${host}&port=${port}`)}`,
        createHeadersForAgent(agentId,
            { accessKey, secretKey }));
}

module.exports = {
    getSgwsBuckets,
    createSgwsBucket,
    putSgwsObject,
    deleteSgwsObject,
    getSgwsObject
};
