const randomize = require('randomatic');
const {
    AGENT_MODIFY_TIERING_FULLNESS_THRESHOLD_PATH,
    AGENT_DESCRIBE_LIFS_PATH, AGENT_DESCRIBE_ONTAP_VERSION_PATH, AGENT_DESCRIBE_OBJECTSTORE_CONFIG_PATH,
    AGENT_DESCRIBE_SVM_VOLUMES_PATH, AGENT_DESCRIBE_NODES_PATH, AGENT_CLUSTER_INFO_PATH,
    AGENT_ZAPI_PATH, AGENT_DESCRIBE_AGGREGATES_PATH, AGENT_DESCRIBE_SYSTEM_IMAGES_PATH, AGENT_DESCRIBE_CLUSTER_VOLUME_FOOTPRINT_PATH, AGENT_DESCRIBE_VOLUME_MINIMUM_COOLING_DAYS_PATH,
    AGENT_DESCRIBE_OBJECT_STORES_PATH, AGENT_DESCRIBE_SVMS_PATH, OBJECT_STORAGE_CONFIG_NAME_PREFIX,
    AGENT_ENABLE_COLD_DATA_REPORTING_PATH, AGENT_MODIFY_VOLUME_MINIMUM_COOLING_DAYS_PATH,
    AGENT_LICENSES_PATH
} = require('../../utils/consts');
const {
    modifyTieringFullnessThresholdRequestSchema, describeLifsResponse, describeOntapVersionResponse, describeObjectStoreConfigResponse,
    volumeMinimumCoolingDaysResponse, volumesResponseSchema, clusterVolumeFootprintResponse,
    svmsResponse, clusterInfoResponse, nodesResponse,
    systemImagesResponse, attachObjectStoreSchema, objectStoresResponse, aggregatesResponse,
    createObjectStoreConfigRequestSchema,
    modifyVolumeRequestSchema, enableColdDataReportingRequestSchema, modifyVolumeMinimumCoolingDaysRequestSchema, applyLicenseRequestSchema, licenseStatusResponseSchema
} = require('../../joi-schemas/agent-schemas');
const { createAgentPath, createHeadersForAgent } = require('../../utils/utils');
const { validateResponse, validateRequest } = require('../../utils/schema-validation');
const { gotInstanceForAgent } = require('../../utils/got-utils');
const getLogger = require('../../utils/logger/logger');

const logger = getLogger();

async function describeAggregates(agentId, clusterId, accountId) {
    logger.info('Describing aggregates:', {
        agentId, clusterId, accountId
    });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_AGGREGATES_PATH}`, accountId), createHeadersForAgent(agentId));
    validateResponse(data, aggregatesResponse);
    return data;
}

async function describeObjectStores(agentId, clusterId, accountId) {
    logger.info('Describing object stores:', {
        agentId, clusterId, accountId
    });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_OBJECT_STORES_PATH}`, accountId), createHeadersForAgent(agentId));
    validateResponse(data, objectStoresResponse);
    return data;
}

async function describeSystemImages(agentId, clusterId) {
    logger.info('Describing system images:', { agentId, clusterId });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_SYSTEM_IMAGES_PATH}`), createHeadersForAgent(agentId));
    validateResponse(data, systemImagesResponse);

    return data;
}

async function attachObjectStore(agentId, clusterId, aggregateName, objectStoreName, checkOnly, allowFlexgroup, accountId) {
    logger.info('Attaching object store:', {
        agentId, clusterId, aggregateName, objectStoreName, checkOnly, allowFlexgroup, accountId
    });
    const body = {
        aggregateName, objectStoreName, checkOnly, allowFlexgroup
    };
    validateRequest(body, attachObjectStoreSchema);
    return gotInstanceForAgent.post(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_OBJECT_STORES_PATH}`, accountId), {
        json: body,
        ...createHeadersForAgent(agentId)
    });
}

async function describeSvms(agentId, clusterId) {
    logger.info('Describing svms:', { agentId, clusterId });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_SVMS_PATH}`), createHeadersForAgent(agentId));
    validateResponse(data, svmsResponse);
    return data;
}

async function describeClusterInfo(agentId, clusterId, accountId) {
    logger.info('Describing cluster info:', {
        agentId, clusterId, accountId
    });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_CLUSTER_INFO_PATH}`, accountId), createHeadersForAgent(agentId));
    validateResponse(data, clusterInfoResponse);
    return data;
}

async function describeNodes(agentId, clusterId, accountId) {
    logger.info('Describing nodes:', {
        agentId, clusterId, accountId
    });

    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_NODES_PATH}`, accountId), createHeadersForAgent(agentId));
    validateResponse(data, nodesResponse);
    return data;
}

async function describeSvmVolumes(agentId, clusterId, svmName, volumeName) {
    logger.info('Describing svm volumes:', {
        agentId, clusterId, svmName, volumeName
    });
    let url = createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}/${svmName}${AGENT_DESCRIBE_SVM_VOLUMES_PATH}`);
    if (volumeName) {
        url = `${url}?volumeName=${volumeName}`;
    }
    const data = await gotInstanceForAgent.get(url, createHeadersForAgent(agentId));
    validateResponse(data, volumesResponseSchema);
    return data;
}

async function describeClusterVolumesFootprint(agentId, clusterId) {
    logger.debug('Describing cluster volumes footprint:', { agentId, clusterId });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_CLUSTER_VOLUME_FOOTPRINT_PATH}`), createHeadersForAgent(agentId));
    validateResponse(data, clusterVolumeFootprintResponse);
    return data;
}

async function describeVolumesMinimumCoolingDays(agentId, clusterId) {
    logger.debug('Describing volumes minimum cooling days:', { agentId, clusterId });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_VOLUME_MINIMUM_COOLING_DAYS_PATH}`), createHeadersForAgent(agentId));
    validateResponse(data, volumeMinimumCoolingDaysResponse);
    return data;
}

async function describeLifs(agentId, clusterId, accountId, svmName) {
    logger.debug('Describing lifs:', {
        agentId, clusterId, accountId, svmName
    });
    const url = svmName ? `${createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_LIFS_PATH}`, accountId)}?svmName=${svmName}` : `${createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_LIFS_PATH}`, accountId)}`;
    const data = await gotInstanceForAgent.get(url, createHeadersForAgent(agentId));
    validateResponse(data, describeLifsResponse);
    return data;
}

async function describeOntapVersion(agentId, clusterId, accountId) {
    logger.debug('Describing Ontap version:', {
        agentId, clusterId, accountId
    });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_ONTAP_VERSION_PATH}`, accountId), createHeadersForAgent(agentId));
    validateResponse(data, describeOntapVersionResponse);
    return data;
}

async function describeObjectStoreConfig(agentId, clusterId, accountId) {
    logger.debug('Describing object store config:', {
        agentId, clusterId, accountId
    });
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_OBJECTSTORE_CONFIG_PATH}`, accountId),
        createHeadersForAgent(agentId));
    validateResponse(data, describeObjectStoreConfigResponse);
    return data;
}

async function createObjectStoreConfig(cloudProvider, agentId, clusterId, serverEndPoint, bucketName, ipSpace, providerProperties) {
    logger.info(`About to create object store config on clusterId  ${clusterId} to AWS with server endpoint ${serverEndPoint}`);
    const objectStoreName = `${OBJECT_STORAGE_CONFIG_NAME_PREFIX}-${randomize('0', 3)}`;
    const requestData = {
        objectStoreName,
        s3Name: bucketName,
        objectStoreProviderType: cloudProvider,
        server: serverEndPoint,
        ipSpace,
        ...providerProperties,
        checkOnly: false
    };
    validateRequest(requestData, createObjectStoreConfigRequestSchema);
    await gotInstanceForAgent.post(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_OBJECTSTORE_CONFIG_PATH}`), {
        json: requestData,
        ...createHeadersForAgent(agentId)
    });
    return objectStoreName;
}

async function enableColdDataReporting(agentId, clusterId, aggregateName, enable = true) {
    logger.info('Enable cold data reporting', {
        agentId, clusterId, aggregateName
    });
    const requestData = { enable };
    validateRequest(requestData, enableColdDataReportingRequestSchema);
    return gotInstanceForAgent.put(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_DESCRIBE_AGGREGATES_PATH}/${aggregateName}${AGENT_ENABLE_COLD_DATA_REPORTING_PATH}`),
        { json: requestData, ...createHeadersForAgent(agentId) });
}

async function modifyVolume(agentId, clusterId, svmName, volumeName, tieringPolicy) {
    logger.info(`Modifying volume tiering policy ${volumeName} of clusterId ${clusterId} and svmName ${svmName} to ${tieringPolicy}`);
    const body = { tieringPolicy };
    validateRequest(body, modifyVolumeRequestSchema);
    return gotInstanceForAgent.put(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}/${svmName}${AGENT_DESCRIBE_SVM_VOLUMES_PATH}/${volumeName}`), {
        json: body,
        ...createHeadersForAgent(agentId)
    });
}

async function modifyVolumeMinimumCoolingDays(agentId, clusterId, volumeName, svmName, minimumCoolingDays) {
    logger.info(`Modifying volume ${volumeName}  of ${svmName} of clusterId ${clusterId} minimum cooling days to ${minimumCoolingDays}`);
    const body = { volumeName, days: minimumCoolingDays, svmName };
    validateRequest(body, modifyVolumeMinimumCoolingDaysRequestSchema);
    return gotInstanceForAgent.put(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_MODIFY_VOLUME_MINIMUM_COOLING_DAYS_PATH}`),
        {
            json: body,
            ...createHeadersForAgent(agentId)
        });
}

async function modifyTieringFullnessThreshold(agentId, clusterId, aggregateName, objectStoreName, tieringFullnessThreshold) {
    logger.info('modify tiering fullness thershold', {
        agentId, clusterId, aggregateName, objectStoreName, tieringFullnessThreshold
    });
    const body = { aggregateName, objectStoreName, tieringFullnessThreshold };
    validateRequest(body, modifyTieringFullnessThresholdRequestSchema);
    return gotInstanceForAgent.put(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_MODIFY_TIERING_FULLNESS_THRESHOLD_PATH}`), {
        json: body,
        ...createHeadersForAgent(agentId)
    });
}

async function applyLicense(agentId, clusterId, license) {
    logger.info(`Applying Fabric Pool license on cluster ${clusterId} and agent ${agentId}`);
    validateRequest(license, applyLicenseRequestSchema);
    return gotInstanceForAgent.put(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_LICENSES_PATH}`), {
        json: license,
        ...createHeadersForAgent(agentId)
    });
}

async function licenseStatus(agentId, clusterId, accountId) {
    const data = await gotInstanceForAgent.get(createAgentPath(`${AGENT_ZAPI_PATH}${clusterId}${AGENT_LICENSES_PATH}`, accountId), createHeadersForAgent(agentId));
    validateResponse(data, licenseStatusResponseSchema);
    return data;
}

module.exports = {
    describeAggregates,
    describeObjectStores,
    describeSystemImages,
    attachObjectStore,
    describeSvms,
    describeClusterInfo,
    describeNodes,
    describeSvmVolumes,
    describeClusterVolumesFootprint,
    describeVolumesMinimumCoolingDays,
    describeLifs,
    describeOntapVersion,
    describeObjectStoreConfig,
    enableColdDataReporting,
    modifyVolume,
    modifyVolumeMinimumCoolingDays,
    applyLicense,
    licenseStatus,
    createObjectStoreConfig,
    modifyTieringFullnessThreshold
};
