const {
    AGENTS_MGMT_ENDPOINT, API_PATH_AGENTS, API_PATH_AGENTS_INTERNAL, CLOUD_PROVIDERS
} = require('../../utils/consts');
const { getAgentsSchema, getAgentSchema, getAgentsInternalSchema } = require('../../joi-schemas/agents-schemas');
const { validateResponse } = require('../../utils/schema-validation');
const getLogger = require('../../utils/logger/logger');
const { getServiceToken } = require('../auth0');
const { getUserToken } = require('../../utils/async-local-storage');
const { gotInstanceForServices } = require('../../utils/got-utils');
const { createHeaders } = require('../../utils/utils');

const logger = getLogger();

function adjustAgentResponse(agent) {
    const { provider } = agent;
    agent.provider = provider.toUpperCase();
    return agent;
}

async function getAgentsInternal(accountId) {
    logger.info(`Get all agents internal for account ${accountId}`);
    const url = accountId ? `${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS_INTERNAL}?account_id=${accountId}` : `${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS_INTERNAL}`;
    const data = await gotInstanceForServices.get(url,
        createHeaders(
            {
                'X-User-Token': getServiceToken(),
                authorization: getServiceToken()
            }
        ));
    validateResponse(data, getAgentsInternalSchema);
    const agents = data.agents.filter(agent => [...Object.values(CLOUD_PROVIDERS), 'onprem'].includes(agent.agent.provider.toLowerCase())).map(agent => ({ ...adjustAgentResponse(agent.agent), clientInfraStatus: agent.clientInfraStatus }));
    return { agents };
}

async function getAgents(accountId) {
    logger.info(`Get agents of accountId ${accountId}`);
    const data = await gotInstanceForServices.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${accountId}`,
        createHeaders({
            'X-User-Token': getUserToken(),
            authorization: getServiceToken()
        }));
    validateResponse(data, getAgentsSchema);
    const agents = data.agents.filter(agent => [...Object.values(CLOUD_PROVIDERS), 'onprem'].includes(agent.provider.toLowerCase())).map(agent => adjustAgentResponse(agent));
    return { agents };
}

async function getAgent(agentId, accountId) {
    logger.debug('Get agent', { agentId, accountId });
    const queryParams = `?account_id=${accountId}`;
    const data = await gotInstanceForServices.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agentId}${queryParams}`,
        createHeaders({
            'X-User-Token': getUserToken(),
            authorization: getServiceToken()
        }));
    validateResponse(data, getAgentSchema);
    if ([...Object.values(CLOUD_PROVIDERS), 'onprem'].includes(data.agent.provider.toLowerCase())) {
        const agent = adjustAgentResponse(data.agent);
        return { agent };
    }
    return { agent: undefined };
}

async function deleteAgent(agentId, accountId) {
    logger.info('remove agent', { agentId, accountId });
    return gotInstanceForServices.delete(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agentId}`,
        createHeaders({
            'X-User-Token': getUserToken(),
            authorization: getServiceToken(),
            'X-Tenancy-Account-Id': accountId
        }));
}

module.exports = {
    getAgents, getAgentsInternal, getAgent, deleteAgent
};
