const {
    getAccountsSchema, getWorkspaceSchema, createAccountResponseSchema,
    workspaceSchema, resourcesSchema, tenancyGetAccountUsersResponseSchema
} = require('../../joi-schemas/multi-tenancy-schemas');
const { validateResponse } = require('../../utils/schema-validation');
const {
    MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS, API_PATH_RESOURCES, API_PATH_RESOURCES_INTERNAL,
    API_PATH_USERS_OF_ACCOUNT_INTERNAL, VSA, ONPREM_DISCOVER_LOCATION
} = require('../../utils/consts');
const { getUserToken } = require('../../utils/async-local-storage');
const getLogger = require('../../utils/logger/logger');
const { getServiceToken } = require('../auth0');
const { gotInstanceForServices } = require('../../utils/got-utils');
const { createHeaders } = require('../../utils/utils');

const logger = getLogger();

async function getAccounts(accountId) {
    const data = await gotInstanceForServices.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, createHeaders({
        'X-User-Token': getUserToken(),
        authorization: getServiceToken(),
        'X-Account-Id': process.env.IS_SIMULATOR ? accountId : undefined
    }));
    validateResponse(data, getAccountsSchema);
    return data;
}

async function createAccount(accountName) {
    const data = await gotInstanceForServices.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountName}`,
        createHeaders({
            'X-User-Token': getUserToken(),
            authorization: getServiceToken()
        }));
    validateResponse(data, createAccountResponseSchema);
    return data;
}

async function getAccountUsers(accountId) {
    const data = await gotInstanceForServices.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_USERS_OF_ACCOUNT_INTERNAL}?account_id=${accountId}&only_admins=false`,
        createHeaders({
            'X-User-Token': getServiceToken(),
            authorization: getServiceToken()
        }));

    validateResponse(data, tenancyGetAccountUsersResponseSchema);
    return data;
}

async function getWorkspaces(accountId) {
    logger.info(`get workspaces of account ${accountId}`);
    const data = await gotInstanceForServices.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountId}/workspace`, createHeaders({
        'X-User-Token': getUserToken(),
        authorization: getServiceToken()
    }));
    validateResponse(data, getWorkspaceSchema);
    return data;
}

async function getResources(accountId, workspaceId, isInternalAPI) {
    const tenancyResourcesPath = isInternalAPI ? API_PATH_RESOURCES_INTERNAL : API_PATH_RESOURCES;
    const workspaceParam = workspaceId ? `&workspace=${workspaceId}` : '';
    const xUserToken = isInternalAPI ? getServiceToken() : getUserToken();
    const data = await gotInstanceForServices.get(`${MULTI_TENANCY_ENDPOINT}${tenancyResourcesPath}?account=${accountId}${workspaceParam}`,
        createHeaders(
            {
                'X-User-Token': xUserToken,
                authorization: getServiceToken()
            }
        ));
    validateResponse(data, resourcesSchema);
    // since resource might have empty agentId we need to filter it out
    return data.filter(resource => resource.agentId !== '' && [VSA, ONPREM_DISCOVER_LOCATION].includes(resource.resourceClass));
}

async function getResourcesByAccount(accountId, isInternal) {
    logger.info(`get resources by  account ${accountId}, isInternal ${isInternal}`);
    return getResources(accountId, undefined, isInternal);
}

async function createWorkspace(accountId, workspaceName) {
    logger.info(`create workspace with name ${workspaceName} for account ${accountId}`);
    const data = await gotInstanceForServices.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountId}/workspace/${workspaceName}`, createHeaders({
        'X-User-Token': getUserToken(),
        authorization: getServiceToken()
    }));
    validateResponse(data, workspaceSchema);
    return data;
}

async function deleteAccount(accountId) {
    logger.info(`delete account ${accountId}`);
    return gotInstanceForServices.delete(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountId}`,
        {
            ...createHeaders({
                'X-User-Token': getUserToken(),
                authorization: getServiceToken()
            })
        });
}

async function deleteClusterFromAccount(clusterId) {
    return gotInstanceForServices.delete(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}/${clusterId}`,
        {
            ...createHeaders({
                'X-User-Token': getUserToken(),
                authorization: getServiceToken()
            }),
            responseType: 'text'
        });
}

module.exports = {
    getAccounts,
    getWorkspaces,
    createWorkspace,
    createAccount,
    getResourcesByAccount,
    deleteAccount,
    deleteClusterFromAccount,
    getAccountUsers,
    getResources
};
