const Hubspot = require('hubspot');
const config = require('config');
const Promise = require('bluebird');
const { getAccountUsers } = require('./cloud-manager/multi-tenancy');
const { jwtGetUserInfo } = require('./auth0');
const { getUserToken } = require('../utils/async-local-storage');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function hasApiKey() {
    try {
        return config.get('hubspot.key');
    } catch (err) {
        return false;
    }
}

async function createOrUpdateContact(hubspot, userMetadata, property, propertyValue) {
    const { name, company } = userMetadata;
    const email = userMetadata.email.trim().toLowerCase();
    const nameParts = name.split(' ');
    const [firstname] = nameParts;
    const lastname = nameParts.slice(1).join(' ');
    const data = {
        properties: [
            {
                property: 'email',
                value: email
            },
            {
                property: 'company',
                value: company
            },
            {
                property: 'firstname',
                value: firstname
            },
            {
                property: 'lastname',
                value: lastname
            },
            {
                property,
                value: propertyValue
            }
        ]
    };
    await hubspot.contacts.createOrUpdate(email, data);
    logger.info('contact with email', email, 'was updated');
}

// only for subscribed
async function updateHubspotContactByAccount(account, property, propertyValue) {
    if (hasApiKey() && !process.env.IS_SIMULATOR) {
        try {
            const hubspot = new Hubspot({ apiKey: config.get('hubspot.key') });
            const usersOfAnAccount = await getAccountUsers(account);
            await Promise.map(usersOfAnAccount, async user => {
                try {
                    await createOrUpdateContact(hubspot, user, property, propertyValue);
                } catch (err) {
                    logger.error('Failed to update contact of user', user, err);
                }
            }, { concurrency: 10 });
        } catch (err) {
            logger.error(`Failed to update hubspot users of account ${account} with property ${property}`, err);
        }
    }
}

async function updateHubspotContactByUserToken(property, propertyValue) {
    if (hasApiKey() && !process.env.IS_SIMULATOR) {
        const userToken = getUserToken();
        try {
            const userMetadata = await jwtGetUserInfo(userToken);
            const hubspot = new Hubspot({ apiKey: config.get('hubspot.key') });
            await createOrUpdateContact(hubspot, userMetadata, property, propertyValue);
        } catch (err) {
            logger.error(`Failed to update hubspot contact for user with token ${userToken} with property ${property}`, err);
        }
    }
}

module.exports = { updateHubspotContactByUserToken, updateHubspotContactByAccount };
