const config = require('config');
const { get } = require('lodash');
const { ApolloError } = require('apollo-server');
const {
    IPA_BASE_URL, IPA_SIGN_PATH, SIGNED_SERVICE_USER_NAME,
    SIGNED_SERVICE_PASSWORD,
    IPA_LICENSE_REGISTRATION_URL, IPA_LOOKUP_TOKEN_URL, IPA_LOOKUP_URL
} = require('../utils/consts');
const { validateResponse, validateRequest } = require('../utils/schema-validation');
const {
    licenseSchema, signLicenseRequestSchema,
    sendLicenseRequestToIPASchema
} = require('../joi-schemas/ipa-schemas');
const { gotInstanceForOuterServices } = require('../utils/got-utils');
const { getUserToken } = require('../utils/async-local-storage');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

const username = config.get('ipa.credentials.username');
const password = config.get('ipa.credentials.password');
const scopeHeader = config.get('ipa.license-lookup-token-headers.scope');

async function signLicense(license) {
    validateRequest(license, signLicenseRequestSchema);

    const data = await gotInstanceForOuterServices.post(`${IPA_BASE_URL}${IPA_SIGN_PATH}`,
        // basic authorization parameters should be passed as parameters to got options
        // currently it does not work because of the following Node issues:
        // https://github.com/nodejs/node/pull/31450
        // https://github.com/nodejs/node/issues/31439
        //
        // got issue awaiting above issues:
        // https://github.com/sindresorhus/got/issues/1169
        //
        // once these issues are resolved we can remove the header and use username and password parameters again
        {
            json: license,
            headers: {
                Authorization: `Basic ${Buffer.from(`${SIGNED_SERVICE_USER_NAME}:${SIGNED_SERVICE_PASSWORD}`).toString('base64')}`

            }
        });
    logger.info('Got signed license:', data);
    validateResponse(data, licenseSchema);
    return data;
}

function generateLicenseRequestToIPA(serialNumber, packageName, requestType, clusterUuid) {
    return {
        version: '1',
        source: 'CloudTiering',
        serialNumber,
        op: 'get',
        product: 'FabricPool',
        package: packageName,
        requestType,
        HostID: clusterUuid || undefined
    };
}

async function sendLicenseRequestToIPA(nssUserName, nssPassword, serialNumber, packageName, requestType, clusterUuid) {
    logger.info('Register license to IPA', {
        nssUserName, nssPassword, serialNumber, packageName, requestType, clusterUuid
    });
    // here we can check if license already applied
    const licenseRequest = generateLicenseRequestToIPA(serialNumber, packageName, requestType, clusterUuid);

    validateRequest(licenseRequest, sendLicenseRequestToIPASchema);
    return gotInstanceForOuterServices.post(IPA_LICENSE_REGISTRATION_URL, {
        json: { licenseRequest },
        headers: {
            Authorization: `Basic ${Buffer.from(`${nssUserName}:${nssPassword}`).toString('base64')}`,
            'x-user-id': (process.env.IS_SIMULATOR) ? getUserToken() : undefined
        }
    });
}

async function getIPAAccessToken(nssUserName, nssPassword) {
    logger.info('Getting access token from IPA', { nssUserName });
    try {
        const data = await gotInstanceForOuterServices.post(IPA_LOOKUP_TOKEN_URL, {
            form: {
                username: nssUserName,
                password: nssPassword,
                grant_type: 'password',
                scope: scopeHeader
            },
            headers: {
                Authorization: `Basic ${Buffer.from(`${username}:${password}`).toString('base64')}`,
                nssUserName: (process.env.IS_SIMULATOR) ? nssUserName : undefined
            }
        });
        return data.access_token;
    } catch (err) {
        if (err.status === 401 || get(err, 'message.error') === 'invalid_grant') {
            throw new ApolloError('Couldn\'t get access token. Please make sure your credentials are correct.', 401);
        }
        throw new ApolloError(`Couldn't get access token. unknown error ${err.message}`, 401);
    }
}
async function sendLookupRequestToIPA(nssUserName, nssPassword, serialNumber) {
    logger.info('Lookup request for license to IPA', {
        nssUserName, nssPassword, serialNumber
    });

    const accessToken = await getIPAAccessToken(nssUserName, nssPassword);
    // here we can check if license already applied
    const headers = {
        Authorization: `Bearer ${accessToken}`,
        'x-user-id': getUserToken()
    };
    const queryParams = new URLSearchParams({ serialNumber, licenses_package: 'FabricPool' }).toString();
    return gotInstanceForOuterServices.get(`${IPA_LOOKUP_URL}${queryParams}`, {
        headers
    });
}

module.exports = {
    signLicense, sendLicenseRequestToIPA, sendLookupRequestToIPA
};
