const {
    last, get, uniq, groupBy, first, mapValues
} = require('lodash');
const {
    OCCM_PROVIDER_TO_CLOUD_PROVIDER, ONTAP_VERSIONS_FEATURES_SUPPORT, HOT,
    STANDARD, AWS, AZURE, GCP, STORAGE_CLASSES_NEW_AGENT, ON_STATUS, FAILED_STATUS
} = require('../utils/consts');
const { capacityToBytes } = require('../utils/utils');
const { Cluster } = require('../models/cluster');
const { getAccountId } = require('../utils/async-local-storage');
const {
    describeAggregates, describeObjectStores, describeSystemImages, attachObjectStore, describeSvms, describeClusterInfo, describeVolumesMinimumCoolingDays,
    describeNodes, describeSvmVolumes, describeClusterVolumesFootprint, describeLifs, describeOntapVersion, describeObjectStoreConfig,
    enableColdDataReporting,
    licenseStatus
} = require('../lib/agent/zapi');
const { getAwsTierLevel } = require('../lib/agent/aws');
const { getAzureStorageAccounts } = require('../lib/agent/azure');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function parseOntapVersion(ontapVersion) {
    if (ontapVersion) {
        const ontapVersionRegex = /(\d+\.\d*|\d+)/;
        const version = ontapVersionRegex.exec(ontapVersion)[1];
        return parseFloat(version);
    }
}

async function getOntapVersion(agentId, clusterId, accountId) {
    const versionObj = await describeOntapVersion(agentId, clusterId, accountId);
    const ontapVersion = `${versionObj.generation}.${versionObj.major}.${versionObj.minor}`;
    return parseOntapVersion(ontapVersion);
}

async function getFullOntapVersion(agentId, clusterId) {
    const images = await describeSystemImages(agentId, clusterId);
    const currentImage = images.find(image => image.isCurrent);
    const version = currentImage ? currentImage.version : last(images).version;
    return parseOntapVersion(version);
}

async function checkTieringRestrictions(agentId, clusterId, aggregate, ontapVersion, accountId) {
    try {
        if (aggregate.isComposite) {
            return { restrictionReason: `Aggregate ${aggregate.name} is already a FabricPool`, isTieringRestricted: true };
        }
        const allowFlexgroup = ontapVersion >= ONTAP_VERSIONS_FEATURES_SUPPORT.FLEX_GROUP;
        const attachResponse = await attachObjectStore(agentId, clusterId, aggregate.name, '', true, allowFlexgroup, accountId);
        if (attachResponse.isAggregateRestricted) {
            logger.info(`Restricted aggregate ${aggregate.name} on cluster id: ${clusterId} detected. Reason: ${attachResponse.reason}`);
            return { restrictionReason: `Aggregate can’t be configured for tiering :${attachResponse.reason}`, isTieringRestricted: true };
        }
        return { isTieringRestricted: false };
    } catch (err) {
        logger.error(`Unexpected error occurred when trying to attach object store to aggregate: ${aggregate.name} on cluster id: ${clusterId} with check-only true`, err);
        return { restrictionReason: err.message, isTieringRestricted: true };
    }
}

// if ontapversion is lower than 9.2- no aggr-object-store-get-iter api and we fail when trying to get cluster
async function getObjectStores(agentId, clusterId, accountId, ontapVersion) {
    if (ontapVersion >= ONTAP_VERSIONS_FEATURES_SUPPORT.OBJECT_STORES) {
        return describeObjectStores(agentId, clusterId, accountId);
    }
    return [];
}

async function getAggregatesByAggregateNames(agentId, clusterId, aggregateNames, accountId) {
    logger.debug('get aggregates by aggregate names', {
        agentId, clusterId, aggregateNames, accountId
    });
    const ontapVersion = await getOntapVersion(agentId, clusterId, accountId);
    const [aggregates, objectStores] = await Promise.all([
        describeAggregates(agentId, clusterId, accountId),
        getObjectStores(agentId, clusterId, accountId, ontapVersion)]);
    return Promise.all(aggregates.filter(aggregate => aggregateNames.includes(aggregate.name) && !aggregate.isRoot).map(async aggregate => {
        const { name } = aggregate;
        aggregate.tieringRestriction = await checkTieringRestrictions(agentId, clusterId, aggregate, ontapVersion, accountId);
        const objectStore = objectStores.find(objectStoreObj => objectStoreObj.aggregateName === name);
        aggregate.objectStoreName = get(objectStore, 'name');
        aggregate.availabilityState = get(objectStore, 'availabilityState');
        aggregate.tieringFullnessThreshold = get(objectStore, 'tieringFullnessThreshold');
        return aggregate;
    }));
}

async function getAggregates(agentId, clusterId, accountId) {
    logger.debug('Get aggregates:', {
        agentId, clusterId, accountId
    });
    const ontapVersion = await getOntapVersion(agentId, clusterId, accountId);
    const [aggregates, objectStores] = await Promise.all([
        describeAggregates(agentId, clusterId, accountId),
        getObjectStores(agentId, clusterId, accountId, ontapVersion)
    ]);
    return Promise.all(aggregates.filter(aggregate => !aggregate.isRoot).map(async aggregate => {
        const { name } = aggregate;
        aggregate.tieringRestriction = await checkTieringRestrictions(agentId, clusterId, aggregate, ontapVersion, accountId);
        const objectStore = objectStores.find(objectStoreObj => objectStoreObj.aggregateName === name);
        aggregate.objectStoreName = get(objectStore, 'name');
        aggregate.availabilityState = get(objectStore, 'availabilityState');
        aggregate.tieringFullnessThreshold = objectStore?.tieringFullnessThreshold;
        aggregate.tieredCapacity = objectStore?.usedSpace ?? 0;
        return aggregate;
    }));
}

function flexgroupAggregates(volumes) {
    logger.debug('Flexgroup aggregates:', volumes);
    const allConstituents = volumes.filter(volume => volume.flexgroupConstituentIndex);
    const msIdToVolumes = groupBy(allConstituents, 'flexgroupMSid');
    return mapValues(msIdToVolumes, (volumesArr => uniq(volumesArr.map(vol => vol.aggregateName))));
}

async function findVolumesMinimumCoolingDays(agentId, clusterId, volumes) {
    logger.debug('Finding volumes minimum cooling days:', {
        agentId, clusterId, volumes
    });
    const minimumCoolingDaysResponse = await describeVolumesMinimumCoolingDays(agentId, clusterId);
    return volumes.map(volume => {
        const { name } = volume;
        const minimumColligDaysPattern = new RegExp(`\\s*\\w+\\s*${name}\\s*(\\d+)`);
        const regextResult = minimumColligDaysPattern.exec(minimumCoolingDaysResponse.cliOutput);
        const minimumCoolingDays = { minimumCoolingDays: regextResult ? regextResult[1] : regextResult };
        return { ...volume, ...minimumCoolingDays };
    });
}

async function getSvmVolumes(agentId, clusterId) {
    const svms = await describeSvms(agentId, clusterId);
    return (await Promise.all(svms.filter(svm => svm.svmType === 'data' && svm.svmState === 'running')
        .map(async svm => {
            try {
                return await describeSvmVolumes(agentId, clusterId, svm.svmName);
            } catch (err) {
                logger.error(`Failed to describe svm volumes for cluster id: ${clusterId} and svm name: ${svm.svmName}`, err);
                return [];
            }
        }))).flat();
}

async function getVolumesExtended(agentId, clusterId, volumeProperties) {
    const [svmVolumes, volumesFootprint] = await Promise.all([getSvmVolumes(agentId, clusterId), describeClusterVolumesFootprint(agentId, clusterId)]);
    const requestedVolumes = volumeProperties ? svmVolumes.filter(svmVol => volumeProperties.find(vol => vol.name === svmVol.name && vol.svmName === svmVol.svmName)) : svmVolumes;
    const volumes = requestedVolumes.filter(volume => !(volume.isVserverRoot || volume.flexgroupConstituentIndex));
    const msIdToAggregateNames = flexgroupAggregates(svmVolumes);
    const filteredAggregateNames = uniq(requestedVolumes.map(vol => (vol.aggregateName ? [vol.aggregateName] : get(msIdToAggregateNames, `${vol.flexgroupMSid}`))).flat());
    const aggregates = await getAggregatesByAggregateNames(agentId, clusterId, filteredAggregateNames, getAccountId());
    const volumesWithMinimumCoolingDays = await findVolumesMinimumCoolingDays(agentId, clusterId, volumes);
    return volumesWithMinimumCoolingDays.map(volume => {
        const volumeFootprint = volumesFootprint.find(volumeFootprintObj => volumeFootprintObj.name === volume.name && volumeFootprintObj.svmName === volume.svmName);
        const usedSize = get(volumeFootprint, 'performanceTierFootprint', volume.parentVolumeName ? volume.usedSize : volume.physicalUsed) || 0;
        const constituentsAggregates = get(msIdToAggregateNames, `${volume.flexgroupMSid}`, []);
        const isCapacityTiered = aggregates.filter(aggr => aggr.name === volume.aggregateName || constituentsAggregates.includes(aggr.name)).every(aggregate => aggregate.isComposite);
        const volumeAggregates = constituentsAggregates.length ? aggregates.filter(aggr => constituentsAggregates.includes(aggr.name)) : aggregates.filter(aggr => aggr.name === volume.aggregateName);
        const restrictedAggregate = volumeAggregates.find(aggregate => !aggregate.objectStoreName && aggregate.tieringRestriction.isTieringRestricted);
        const isRestricted = Boolean(restrictedAggregate);
        const isHardRestricted = volumeAggregates.some(aggregate => !aggregate.objectStoreName && aggregate.tieringRestriction.isTieringRestricted && !aggregate.isObjectStoreAttachEligible);
        const restrictionReason = isRestricted ? get(restrictedAggregate, 'tieringRestriction.restrictionReason', 'unknown restriction reason') : undefined;
        return {
            name: volume.name,
            svmName: volume.svmName,
            size: volume.sizeTotal,
            usedSize,
            state: volume.state,
            volumeType: volume.volumeType,
            aggregateName: volumeAggregates.map(aggr => aggr.name).join(','),
            capacityTierUsedSize: isCapacityTiered ? get(volumeFootprint, 'capacityTierFootprint') : 0,
            tieringPolicy: volume.tieringPolicy,
            tierInactiveUserData: volume.tierInactiveUserData,
            tierInactiveUserDataPercent: volume.tierInactiveUserDataPercent,
            constituentsAggregates,
            minimumCoolingDays: volume.minimumCoolingDays,
            snapshotsUsedSize: volume.snapshotsUsedSize || 0, // on offline volume there is no snapshotUsedSize
            isHardRestricted,
            isRestricted,
            restrictionReason,
            isTiered: volumeAggregates.some(aggregate => aggregate.objectStoreName
                && (volume.tieringPolicy ? volume.tieringPolicy !== 'none' : true))
        };
    });
}

async function createOntapClusterPropertiesObject(agentId, clusterId, accountId) {
    logger.debug('Creating Ontap cluster properties object:', {
        agentId, clusterId, accountId
    });
    const [clusterInfoObj, nodes, lifs] = await Promise.all([
        describeClusterInfo(agentId, clusterId, accountId),
        describeNodes(agentId, clusterId, accountId),
        describeLifs(agentId, clusterId, accountId)]);
    const clusterMgmtLif = lifs.find(lif => lif.role === 'cluster_mgmt');

    return {
        clusterAddress: clusterMgmtLif.address,
        clusterUuid: clusterInfoObj.clusterUuid,
        clusterName: clusterInfoObj.clusterName,
        nodeModels: nodes.map(nodeModel => nodeModel.model),
        lifs: lifs.map(lif => ({
            name: lif.interfaceName, status: lif.status, ipSpace: lif.ipSpace, role: lif.role
        }))
    };
}

async function findTierLevel(agentId, clusterId, objectStoreConfig, accountId) {
    const { providerType } = objectStoreConfig;
    switch (providerType) {
        case AWS: {
            try {
                const { s3Name } = objectStoreConfig;
                const { storageClass } = await getAwsTierLevel(agentId, s3Name, accountId);
                return storageClass ? STORAGE_CLASSES_NEW_AGENT[AWS][storageClass] : STANDARD;
            } catch (err) {
                logger.warn('Failed to find AWS tier level - skipping', err);
                return STANDARD;
            }
        }
        case AZURE: {
            try {
                const { server } = objectStoreConfig;
                const storageAccountName = first(server.split('.'));
                const allStorageAccounts = await getAzureStorageAccounts(agentId, accountId);
                const storageAccount = allStorageAccounts.find(storageAccountObj => storageAccountObj.name === storageAccountName);
                const tierLevel = storageAccount?.accessTier || HOT;
                return STORAGE_CLASSES_NEW_AGENT[AZURE][tierLevel];
            } catch (err) {
                logger.warn('Failed to find AZURE tier level - skipping', err);
                return STORAGE_CLASSES_NEW_AGENT[AZURE][HOT];
            }
        }
        case GCP: {
            return STANDARD;
        }
        default: {
            logger.info('Received provider', providerType, 'no tier level');
        }
    }
}

function findClusterProviderType(objectStoreConfig) {
    const providerType = get(objectStoreConfig, 'providerType');
    return OCCM_PROVIDER_TO_CLOUD_PROVIDER[providerType];
}

async function createCapacityFeaturesObject(agentId, clusterId, accountId) {
    logger.info('Creating capacity features object:', {
        agentId, clusterId, accountId
    });
    const objectStoreConfigs = await describeObjectStoreConfig(agentId, clusterId, accountId);
    const objectStoreConfigObject = first(objectStoreConfigs);
    let objectStoreConfig;
    let tierLevel;
    if (objectStoreConfigObject) {
        const {
            server, usedSpace, s3Name, objectStoreName
        } = objectStoreConfigObject;
        objectStoreConfig = {
            server, usedSpace, s3Name, objectStoreName, providerType: findClusterProviderType(objectStoreConfigObject)
        };
        tierLevel = await findTierLevel(agentId, clusterId, objectStoreConfig, accountId);
    }
    return {
        objectStoreConfigExtended: {
            objectStoreConfig,
            tierLevel
        }
    };
}

async function clusterInfo(agentId, clusterId, accountId) {
    try {
        logger.info('Calculating cluster info', {
            agentId, clusterId, accountId
        });
        let ontapClusterProperties;
        let capacityFeatures = {};
        const ontapVersion = await getOntapVersion(agentId, clusterId, accountId);
        if (ontapVersion > 9.1) {
            [ontapClusterProperties, capacityFeatures] = await Promise.all([
                createOntapClusterPropertiesObject(agentId, clusterId, accountId),
                createCapacityFeaturesObject(agentId, clusterId, accountId)]);
            ontapClusterProperties.ontapVersion = ontapVersion;
        } else {
            // in case of ontap version less than 9.1 there is no object store config api
            ontapClusterProperties = await createOntapClusterPropertiesObject(agentId, clusterId, accountId);
            ontapClusterProperties.ontapVersion = ontapVersion;
        }
        return {
            clusterId,
            publicId: clusterId,
            name: ontapClusterProperties.clusterName,
            status: { status: ON_STATUS },
            ontapClusterProperties,
            capacityFeatures
        };
    } catch (err) {
        const clusterFromDB = await Cluster.findOne({ accountId: getAccountId(), clusterId }).exec();
        return {
            name: clusterFromDB ? clusterFromDB.clusterName : undefined,
            clusterId,
            publicId: clusterId,
            status: { status: FAILED_STATUS, message: err.message || err.status }
        };
    }
}

async function enableColdDataReportingOperation(agentId, clusterId) {
    const clusterAggregates = await describeAggregates(agentId, clusterId);
    const aggregatesToEnableColdDataReporting = clusterAggregates.filter(aggr => (aggr.isObjectStoreAttachEligible && !aggr.isComposite) && !aggr.isInactiveDataReportingEnabled);
    return Promise.all(aggregatesToEnableColdDataReporting.map(async aggr => {
        try {
            await enableColdDataReporting(agentId, clusterId, aggr.name, true);
        } catch (err) {
            logger.error(`Failed to activate cold data reporting on aggregate ${aggr.name} on cluster ${clusterId}`);
        }
    }));
}

async function getFabricpoolLicenses(agentId, clusterId, accountId) {
    const data = await licenseStatus(agentId, clusterId, accountId);
    const licenseObject = data.find(obj => obj.package.toLowerCase() === 'fabricpool');
    if (licenseObject) {
        // convert maxCapacity and currentCapacity (always GB) to bytes
        licenseObject.maxCapacity = capacityToBytes({ size: licenseObject.maxCapacity, unit: 'GB' });
        licenseObject.currentCapacity = capacityToBytes({ size: licenseObject.currentCapacity, unit: 'GB' });
        return licenseObject;
    }
}

module.exports = {
    parseOntapVersion,
    getFullOntapVersion,
    checkTieringRestrictions,
    getAggregates,
    flexgroupAggregates,
    findVolumesMinimumCoolingDays,
    getVolumesExtended,
    getSvmVolumes,
    getOntapVersion,
    createOntapClusterPropertiesObject,
    createCapacityFeaturesObject,
    clusterInfo,
    findTierLevel,
    enableColdDataReportingOperation,
    getFabricpoolLicenses,
    getAggregatesByAggregateNames
};
