const config = require('config');
const { groupBy } = require('lodash');
const { collectChargingInfoFromDB } = require('../models/cluster');
const { MpSubscription } = require('../models/mp-subscription');
const {
    TIERING_AZURE_SERVICE_NAME_SUBSCRIPTION,
    CHARGING_DIMENSION, OCCM_CHARGING_DIMENSION, OCCM_SERVICE_NAME_SUBSCRIPTION, TIERING_SERVICE_NAME_SUBSCRIPTION,
    GCP_CHARGING_DIMENSION, AWS, GCP, AZURE, AZURE_OCCM_CHARGING_DIMENSION
} = require('../utils/consts');
const charge = require('../lib/charging-service');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();
const chargingLogger = getLogger('charging');

function findDimensionByServiceNameAndCloudProvider(cloudProvider, serviceName) {
    // using CLOUD_PROVIDER object because from subscription service cloudProvider returns in lower case
    if (cloudProvider === AWS) {
        if (serviceName === OCCM_SERVICE_NAME_SUBSCRIPTION) {
            return OCCM_CHARGING_DIMENSION;
        }
        if (serviceName === TIERING_SERVICE_NAME_SUBSCRIPTION) {
            return CHARGING_DIMENSION;
        }
    } else if (cloudProvider === GCP) {
        return GCP_CHARGING_DIMENSION;
    } else if (cloudProvider === AZURE) {
        if (serviceName === OCCM_SERVICE_NAME_SUBSCRIPTION) {
            return AZURE_OCCM_CHARGING_DIMENSION;
        }
        if (serviceName === TIERING_AZURE_SERVICE_NAME_SUBSCRIPTION) {
            return OCCM_CHARGING_DIMENSION;
        }
    }
}

async function transformToChargingObjects(chargingFromDB, timestamp) {
    const info = await Promise.all(chargingFromDB.map(async subscription => {
        const { subscriptionId, accountsAndClusters } = subscription;
        try {
            const { serviceName, cloudProvider } = await MpSubscription.findOne({ subscriptionId }).exec();
            const dimension = findDimensionByServiceNameAndCloudProvider(cloudProvider, serviceName);
            const groupByAccounts = groupBy(accountsAndClusters, 'accountId');
            const accounts = Object.keys(groupByAccounts).flatMap(accountId => {
                const resources = groupByAccounts[accountId].map(obj => ({
                    resourceId: obj.resourceId,
                    val: Math.floor(obj.val / (1024 ** config.get('charging.multiplier')))
                })).filter(obj => obj.val);
                return { accountId, resources };
            }).filter(account => account.resources.length);

            return {
                subscriptionId,
                accounts,
                timestamp,
                dimension
            };
        } catch (err) {
            logger.error(`Failed to create charging object of subscriptionId ${subscriptionId}`, err);
        }
    }));
    return info.filter(subscription => subscription.accounts.length);
}

async function collectChargingInfo() {
    try {
        const timestamp = new Date().toISOString();
        const chargingFromDb = await collectChargingInfoFromDB();
        logger.info('charging data from DB', JSON.stringify(chargingFromDb, null, 4));
        const charging = await transformToChargingObjects(chargingFromDb, timestamp);
        logger.info('Charging after transform', JSON.stringify(charging));
        if (charging && charging.length) {
            chargingLogger.info('CHARGING is ', JSON.stringify(charging));
        }
        return charging;
    } catch (err) {
        logger.error('Error collecting charging info', err);
    }
}

async function collectInfoAndCharge() {
    logger.info('Collect charging operation');
    const charging = await collectChargingInfo();
    if (charging && charging.length) {
        await charge(charging);
    }
}

module.exports = {
    collectChargingInfo,
    collectInfoAndCharge
};
