const { ApolloError } = require('apollo-server');
const {
    isNil, get, pick, sortBy
} = require('lodash');
const {
    CVO, VSA, UNKNOWN_STATUS, ON_STATUS, AWS_REGIONS,
    OBJECT_STORE_UNAVAILABLE_STATE, NO_TIERING_STATUS, FAILED_STATUS, AUTO,
    AWS, AGENT_ACTIVE_STATUS, DEFAULT_AWS_REGION,
    SGWS, ONTAP_VERSIONS_FEATURES_SUPPORT
} = require('../utils/consts');
const {
    getAgentIdByClusterId, getResourcesOfValidAgents,
    filterOutDeletedClusters, getAllClustersInAccount
} = require('./multi-tenancy-operation');
const { getAccountId } = require('../utils/async-local-storage');
const {
    clusterProviderType, checkAggregatesRestrictions, checkIfCvoCluster,
    findMinimumOntapVersionRequiredForCloudProvider, validateEligibleForTiering
} = require('./cloud-manager-operations');
const {
    calculatePerformanceCapacity, calculateSavingsOpportunitiesAndSavings, volumeInactiveUserData, calculateUsedTieredAndColdData
} = require('./calculations-operations');
const {
    getTieredVolumesSummary
} = require('./volume-operations');
const { findValidAgents } = require('./agents-operations');
const { registerClusterAutoDiscover } = require('./clusters-collection-operations');
const { deleteClusterFromAccount, getResourcesByAccount, getResources } = require('../lib/cloud-manager/multi-tenancy');
const { Cluster, findLicenseInfoOfCluster } = require('../models/cluster');
const { getAgents, getAgent } = require('../lib/cloud-manager/agents');
const { removeCluster } = require('../lib/cloud-manager/clusters');
const {
    getVolumesExtended, getAggregates,
    enableColdDataReportingOperation, clusterInfo
} = require('./agent-zapi-operations');
const { describeLifs, describeSvms } = require('../lib/agent/zapi');

const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function isClusterEligibleForTiering(agentCloudProvider, aggregates, ontapClusterProperties, clusterName, resourceType) {
    logger.info('Is ClusterEligible For Tiering', { agentCloudProvider, clusterName, resourceType });
    const nodeModels = get(ontapClusterProperties, 'nodeModels');
    const clusterAddress = get(ontapClusterProperties, 'clusterAddress');
    const ontapVersion = get(ontapClusterProperties, 'ontapVersion');
    if (resourceType === CVO) {
        const reason = 'Data tiering isn’t supported with this ontap cluster. Refer to the docs for details about supported systems';
        return { isEligibleForTiering: false, reason };
    }
    // ontap version is either parsed version or undefined
    if (ontapVersion) {
        const eligibleVersionPerProvider = findMinimumOntapVersionRequiredForCloudProvider(agentCloudProvider);
        if (ontapVersion >= eligibleVersionPerProvider) {
            const aggregateRestrictionResult = checkAggregatesRestrictions(aggregates);
            if (aggregateRestrictionResult) {
                const isClusterIsCvo = checkIfCvoCluster(nodeModels, clusterName, clusterAddress);
                const reason = isClusterIsCvo ? 'Data tiering isn’t supported with this on-prem cluster. Refer to the docs for details about supported systems' : undefined;
                return { isEligibleForTiering: !isClusterIsCvo, reason };
            }
            return { isEligibleForTiering: false, reason: 'Tiering data from HDDs isn’t currently supported' };
        }
        return { isEligibleForTiering: false, reason: 'You need to upgrade ONTAP to version 9.4 or later' };
    }
    // in case of failed or unknown clusters there is no ontapVersion
    return { isEligibleForTiering: true };
}

async function findTieringStats(accountId) {
    logger.info(`find tiering stats for account ${accountId}`);
    try {
        const result = await Cluster.aggregate([{ $match: { accountId } },
            { $group: { _id: '$accountId', tieredCapacity: { $sum: '$currentCapacity' }, numberOfClusters: { $sum: 1 } } }]);
        return { tieredCapacity: result[0]?.tieredCapacity ?? 0, numberOfClusters: result[0]?.numberOfClusters ?? 0 };
    } catch (err) {
        return {
            tieredCapacity: 0,
            numberOfClusters: 0
        };
    }
}

async function findClusterStatus(accountId, clusterId, agentIdParam) {
    logger.debug('Find cluster status', { accountId, clusterId });
    const agentId = agentIdParam || await getAgentIdByClusterId(clusterId, accountId);
    const { agent } = await getAgent(agentId, accountId);
    if (agent.status === AGENT_ACTIVE_STATUS) {
        try {
            await describeSvms(agentId, clusterId);
            return { status: ON_STATUS };
        } catch (err) {
            logger.error('Failed to find cluster status', err);
            return { status: FAILED_STATUS, message: err.message || err.status };
        }
    }
    return { status: UNKNOWN_STATUS, message: 'No active connector found for resource' };
}

async function markAsDeletedOperation(clusterId, agentId) {
    const accountId = getAccountId();
    const clusterStatus = await findClusterStatus(accountId, clusterId, agentId);
    if ([FAILED_STATUS, UNKNOWN_STATUS].includes(clusterStatus.status)) {
        return Cluster.updateOne({ accountId, clusterId }, { $set: { deleted: true, sendNotification: false } }).exec();
    }
    throw new ApolloError(`Cannot remove cluster with status ${clusterStatus.status}`, 403);
}

async function removeClusterOperation(clusterId, agentId) {
    const accountId = getAccountId();
    try {
        await removeCluster(clusterId, agentId);
    } catch (err) {
        await deleteClusterFromAccount(clusterId);
    } finally {
        await Cluster.deleteOne({ accountId, clusterId }).exec();
    }
}

function tieringStatus(cluster, allAggregates) {
    logger.debug('tiered status', { cluster, allAggregates });
    const { status: { status: clusterStatus } } = cluster;
    if (clusterStatus === ON_STATUS) {
        const tieredAggregates = allAggregates.filter(aggregate => aggregate.objectStoreName);
        const isObjectStoreUnavailable = tieredAggregates.length > 0 && tieredAggregates.every(aggregate => aggregate.availabilityState === OBJECT_STORE_UNAVAILABLE_STATE);
        if (isObjectStoreUnavailable) {
            return { status: NO_TIERING_STATUS, message: 'No connection to the object storage' };
        }
    }
    return {
        status: clusterStatus,
        message: get(cluster, 'status.message')
    };
}

// disabled for now - we want to allow users to re-discover their clusters
/* async function checkIfClusterIsAlreadyDiscovered(accountId, clusterUuid) {
    // assumption - our DB is updated ( is updated on agenda, discover and get clusters(
    const cluster = await Cluster.findOne({ clusterUuid, accountId }).exec();
    if (cluster) {
        throw new HttpError(`Cluster with clusterUuid ${clusterUuid} was already discovered`, 403);
    }
} */

async function updateClusterColdDataReporting(ontapVersion, clusterId, agentId, accountId) {
    // ontap version is parsed ontap version
    if (ontapVersion >= ONTAP_VERSIONS_FEATURES_SUPPORT.ENABLE_COLD_DATA_REPORTING) {
        const clusterFromDB = await Cluster.findOne({ clusterId, accountId });
        if (!clusterFromDB) {
            await enableColdDataReportingOperation(agentId, clusterId);
        }
    } else {
        logger.info(`Can't enable cold data reporting on cluster ${clusterId} due to its ONTAP version: ${ontapVersion}. Skipping...`);
    }
}

function regionByS3Endpoint(s3Endpoint) {
    if (s3Endpoint === 's3.amazonaws.com') {
        return AWS_REGIONS[DEFAULT_AWS_REGION];
    }
    const awsRegionRegex = /s3-(\S*).amazonaws.com/;
    const regexResult = awsRegionRegex.exec(s3Endpoint);
    // if regex has result - we are in S3 otherwise it is azure/gcp which we dont need to extract the region
    return (regexResult) ? AWS_REGIONS[regexResult[1]] : s3Endpoint;
}

async function adjustClusterResponse(cluster, updateDb = true) {
    const {
        accountId: accountFromResource,
        publicId, clusterId: newId, capacityFeatures, resourceType, status: { status: clusterStatus },
        name, ontapClusterProperties, agentCloudProvider, agentId, agentSystemId
    } = cluster;
    const accountId = accountFromResource || getAccountId();
    const clusterId = publicId || newId; // temporary until all will be with new agent
    logger.info('Adjusting cluster response for cluster', {
        clusterId, accountId, clusterStatus, agentCloudProvider
    });
    const ontapVersion = get(cluster, 'ontapClusterProperties.ontapVersion');
    const [allVolumes, allAggregates] = (clusterStatus === ON_STATUS) ? await Promise.all([
        getVolumesExtended(agentId, clusterId),
        getAggregates(agentId, clusterId, accountId)]) : [[], []];
    const { isEligibleForTiering, reason: isEligibleForTieringReason } = isClusterEligibleForTiering(agentCloudProvider, allAggregates, ontapClusterProperties, name, resourceType);
    const supportedVolumes = allVolumes.filter(volume => !volume.isHardRestricted);
    const allocatedCapacity = (clusterStatus === ON_STATUS) ? allAggregates.reduce((acc, aggr) => acc + aggr.totalCapacity, 0) : undefined;
    const volumesWithoutTieringArr = supportedVolumes.filter(volume => !volume.isTiered);
    const { performanceInactiveCapacityNotTiered, performanceTierUsedCapacity } = calculateUsedTieredAndColdData(supportedVolumes, volumesWithoutTieringArr, AUTO);
    const performanceInactiveCapacity = supportedVolumes.reduce((acc, vol) => acc + volumeInactiveUserData(vol), 0);
    const tieredCapacity = get(cluster, 'capacityFeatures.objectStoreConfigExtended.objectStoreConfig.usedSpace', 0);
    const performanceAndTieredCapacity = performanceTierUsedCapacity + tieredCapacity;
    const { currentSavings, savingOpportunity } = calculateSavingsOpportunitiesAndSavings(performanceTierUsedCapacity, tieredCapacity, performanceInactiveCapacityNotTiered, clusterId, resourceType);
    const totalSnapshotUsedSize = supportedVolumes.reduce((acc, volume) => acc + volume.snapshotsUsedSize, 0);
    const isEstimated = volumesWithoutTieringArr.length && supportedVolumes.some(volume => isNil(volume.tierInactiveUserData));
    const providerType = get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.providerType');
    const response = {
        clusterId,
        name,
        status: tieringStatus(cluster, allAggregates),
        ontapClusterProperties: clusterStatus === ON_STATUS ? pick(ontapClusterProperties, ['clusterAddress', 'ontapVersion', 'clusterUuid']) : undefined,
        capacityInformation: clusterStatus === ON_STATUS ? {
            usedPerformanceAndTieredCapacity: performanceAndTieredCapacity,
            performanceCapacity: calculatePerformanceCapacity(performanceTierUsedCapacity, performanceInactiveCapacity, totalSnapshotUsedSize, isEstimated),
            allocatedCapacity,
            savingOpportunity
        } : undefined,
        tierInformation: get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig') ? {
            bucketName: get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.s3Name'),
            providerType,
            objectStoreName: get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.objectStoreName'),
            regionName: providerType === AWS ? regionByS3Endpoint(get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.server')) : undefined,
            tierLevel: providerType === SGWS ? undefined : get(capacityFeatures, 'objectStoreConfigExtended.tierLevel'),
            currentSavings
        } : undefined,
        agentCloudProvider,
        agentId,
        agentSystemId,
        volumes: getTieredVolumesSummary(supportedVolumes, allVolumes, ontapVersion, isEstimated),
        resourceType,
        isEligibleForTiering,
        isEligibleForTieringReason
    };
    // status on because otherwise we insert the DB with not eligible clusters
    if (isEligibleForTiering) {
        // registering isEligible for tiering in clusters collection

        await updateClusterColdDataReporting(ontapVersion, clusterId, agentId, accountId);

        if (updateDb) {
            const clusterFromDB = await Cluster.findOne({ clusterId, accountId });

            // in order for us not to get into a loop in the subscription - update the cluster on db only if it doesnt exists or was not updated in the last half an hour
            if (!clusterFromDB || clusterFromDB && clusterFromDB?.updatedAt.getTime() < Date.now() - 30 * 60 * 1000) {
                await registerClusterAutoDiscover(accountId, response);
            }
        }
    }
    // if cluster is FAILED or UNKNOWN it is not enforced
    response.licenseInfo = (clusterStatus === ON_STATUS) ? await findLicenseInfoOfCluster(clusterId, accountId) : undefined;
    return response;
}

async function getIpSpaces(clusterId, agentIdParam) {
    // find agentId
    const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
    try {
        const lifs = await describeLifs(agentId, clusterId);
        return [...new Set(lifs.filter(lif => {
            const { role, ipSpace } = lif;
            return ipSpace && role === 'intercluster';
        }).map(lif => lif.ipSpace))];
    } catch (err) {
        logger.error('Failed to receive svms lifs with error', err);
        if (err.status === '401') {
            throw new ApolloError('Invalid ONTAP credentials', 401);
        }
        throw new ApolloError(`Failed to receive ipspaces ${err.message}.`, 403);
    }
}

async function getClusterDetails(clusterId) {
    const accountId = getAccountId();
    const resources = await getResourcesByAccount(accountId);
    // filter by clusterId
    let foundResource;
    foundResource = resources.find(resource => resource.resourceIdentifier === clusterId);
    if (!foundResource && process.env.DEMO) {
        // if occm would have send agentId n the cluster query we wouldnt need to find the resource by hard coded agentId
        foundResource = resources.find(resource => resource.agentId === 'Nw4Q2O1kdnLtvhwegGalFnodEHUfPJWhclients');
    }
    if (foundResource) {
        const { agentId, resourceClass } = foundResource;
        const { agent } = await getAgent(agentId, accountId);
        const agentDetails = {
            useDockerInfa: agent.useDockerInfra,
            agentStatus: agent.status,
            resourceType: resourceClass === VSA ? CVO : resourceClass,
            agentId,
            agentCloudProvider: agent.provider,
            agentSystemId: agent.systemId
        };
        if (agentDetails.agentStatus === AGENT_ACTIVE_STATUS) {
            if (agentDetails.useDockerInfa) {
                const cluster = await clusterInfo(agentDetails.agentId, clusterId);
                Object.assign(cluster, agentDetails);
                return adjustClusterResponse(cluster, false);
            }
            throw new ApolloError('We have made improvements to Cloud Tiering. The improvements to Cloud Tiering require you to rediscover your cluster', 'NO_VALID_AGENTS');
        } else {
            return {
                publicId: clusterId,
                resourceType: resourceClass === VSA ? CVO : resourceClass,
                clusterId,
                status: { status: 'UNKNOWN', message: 'No active connector found for resource' }

            };
        }
    }
    throw new ApolloError(`Cluster with id ${clusterId} was not found`, 404);
}

async function getLazyClusters(agentId, workspaceId) {
    const accountId = getAccountId();
    const { agents } = await getAgents(accountId);
    const allResources = await getResources(accountId, workspaceId);
    const resourcesByAgentId = agentId ? allResources.filter(resource => resource.agentId === agentId) : allResources;
    const validAgents = findValidAgents(agents);
    if (validAgents.length) {
        const resourcesAndAgents = getResourcesOfValidAgents(resourcesByAgentId, validAgents);
        const filteredResources = await filterOutDeletedClusters(resourcesAndAgents);
        return filteredResources.map(resource => ({
            agentId: resource?.agent?.agentId,
            agentCloudProvider: resource?.agent?.provider,
            clusterId: resource.resourceIdentifier,
            resourceType: resource.resourceType,
            agentSystemId: resource?.agent?.systemId
        }));
    }
    return [];
}

async function getClusters(agentId, workspaceId) {
    const accountId = getAccountId();
    const clustersInAccount = await getAllClustersInAccount(accountId, workspaceId, agentId);
    logger.info(`Received clusters ${JSON.stringify(clustersInAccount)} registering on DB and trying to adjust`);
    const adjustedClusters = await Promise.all(clustersInAccount.map(cluster => adjustClusterResponse(cluster)));
    return sortBy(adjustedClusters, 'name');
}

module.exports = {
    adjustClusterResponse,
    getIpSpaces,
    findTieringStats,
    tieringStatus,
    validateEligibleForTiering,
    isClusterEligibleForTiering,
    regionByS3Endpoint,
    updateClusterColdDataReporting,
    clusterProviderType,
    removeClusterOperation,
    markAsDeletedOperation,
    getClusterDetails,
    getLazyClusters,
    getClusters
};
