const Promise = require('bluebird');
const { get } = require('lodash');
const { Cluster } = require('../models/cluster');
const { clusterProviderType, findOntapVersion } = require('./cloud-manager-operations');
const { getActiveAgentsInternal } = require('./agents-operations');
const { findSubscribedSubscriptionByAccountIdAndCloudProvider, updateAccountFreeTrial } = require('./subscription-operations');
const { getFabricpoolLicenses, clusterInfo } = require('./agent-zapi-operations');
const {
    SGWS,
    AWS, ONPREM, ON_STATUS, NO_TIERING_STATUS, UNKNOWN_STATUS
} = require('../utils/consts');

const getLogger = require('../utils/logger/logger');

const logger = getLogger();

async function updateCluster(accountId, clusterInformation, license, agentLocation, updateSubscription) {
    logger.info('Update cluster', {
        accountId, clusterInformation, license, agentLocation
    });
    const {
        agentId, clusterId, isEligibleForTiering, status: { status }
    } = clusterInformation;
    const lastSeen = new Date();
    // we get the response after adjusting - ON clusters can have ON status or NO_TIERING status which indicates on problem on the tiering
    if ([ON_STATUS, NO_TIERING_STATUS].includes(status)) {
        const { ontapClusterProperties: { clusterUuid, ontapVersion }, name: clusterName } = clusterInformation;
        const cloudProvider = get(clusterInformation, 'tierInformation.providerType');
        const tieredCapacity = get(clusterInformation, 'tierInformation.currentSavings.capacity', 0);
        const update = {
            $set: {
                agentId,
                clusterUuid,
                ontapVersion,
                lastSeen,
                cloudProvider,
                agentLocation,
                currentCapacity: tieredCapacity,
                clusterName,
                isEligibleForTiering,
                sendNotification: false
            }
        };

        if (license) {
            const { serialNumber, maxCapacity } = license;
            if (serialNumber === '22654892') {
                logger.info('Found PAYGO license for cluster', clusterId);
                update.$set.tieringLicense = { serialNumber, maxCapacity };
            } else {
                logger.info(`Found BYOL license for cluster ${clusterId} with serialNumber ${serialNumber}`);
                update.$set.byolLicense = { serialNumber, maxCapacity };
                update.$set.sendNotification = true;
                // in case of BYOL license - free trial is canceled
                await updateAccountFreeTrial(accountId, new Date());
            }
        }
        if (updateSubscription) {
            // find subscription with the accountId and the right provider
            const providerToFind = cloudProvider || (agentLocation === ONPREM ? AWS : agentLocation);
            logger.info('Trying to find subscription for clusterId', clusterId, 'and provider', providerToFind);
            const subscription = await findSubscribedSubscriptionByAccountIdAndCloudProvider(accountId, providerToFind);
            if (subscription) {
                update.$set.subscriptionId = subscription.subscriptionId;
                update.$set.sendNotification = true;
            }
        }
        return Cluster.findOneAndUpdate({ accountId, clusterId }, update, { upsert: true, new: true });
    }

    // if the status is  failed or unknown we dont have data on the cluster
    // no last seen because it is not alive
    return Cluster.findOneAndUpdate({
        accountId, clusterId
    }, { $set: { agentLocation, agentId, sendNotification: false } }, { upsert: true });
}

async function findInformationAndUpdateCluster(clusterObj) {
    const {
        clusterId, accountId, agentId, agentLocation, isEligibleForTiering
    } = clusterObj;
    try {
        const clusterInformation = await clusterInfo(agentId, clusterId, accountId);
        logger.info('Get cluster Information on update clusters function', JSON.stringify(clusterInformation));
        const { status: { status, message } } = clusterInformation;
        const ontapVersion = findOntapVersion(clusterInformation);
        if ((status === UNKNOWN_STATUS && message.includes('Invalid resource'))) {
            logger.info('Invalid resource , deleting from our DB', clusterId, accountId);
            await Cluster.deleteOne({ accountId, clusterId }).exec();
        } else {
            // we will want to call to license status only if the aff is with minimum required ontapversion for fabric pool
            const license = (status === ON_STATUS && ontapVersion >= 9.2) ? await getFabricpoolLicenses(agentId, clusterId, accountId) : undefined;
            // update clusterInformation to be similar to clusterResponse
            clusterInformation.isEligibleForTiering = isEligibleForTiering;
            clusterInformation.clusterId = clusterInformation.publicId;
            clusterInformation.agentId = agentId;
            clusterInformation.tierInformation = {
                providerType: clusterProviderType(clusterInformation),
                currentSavings: {
                    capacity: get(clusterInformation, 'capacityFeatures.objectStoreConfigExtended.objectStoreConfig.usedSpace', 0)
                }
            };
            return await updateCluster(accountId, clusterInformation, license, agentLocation, false);
        }
    } catch (err) {
        logger.error(`Failed to update cluster ${clusterId} of accountId ${accountId} with err`, err);
    }
}
// runs every 1H <- agenda
async function updateClusters() {
    const clusters = await Cluster.aggregate([
        {
            $lookup: {
                from: 'accounts',
                localField: 'accountId',
                foreignField: 'accountId',
                as: 'account'
            }
        },
        { $unwind: '$account' },
        {
            $match: {
                $or: [{ subscriptionId: { $exists: true } },
                    { 'account.freeTrialExpiryDate': { $gt: Date.now() } }]
            }
        }
    ]).exec();
    logger.info('number of all clusters', clusters.length);
    const allActiveAgents = await getActiveAgentsInternal();
    const allActiveAgentsIds = allActiveAgents.map(agent => agent.agentId);
    const clustersWithActiveAgent = clusters.filter(cluster => allActiveAgentsIds.includes(cluster.agentId));
    logger.info('clusters with active agents ', clustersWithActiveAgent.length);
    let result;
    if (process.env.NODE_ENV === 'production') {
        result = await Promise.map(clustersWithActiveAgent, async clusterObj => {
            try {
                logger.info('Update cluster - promise map - start', clusterObj.clusterId);
                const updatedCluster = await findInformationAndUpdateCluster(clusterObj);
                logger.info('Update cluster - promise map -done', clusterObj.clusterId);
                return updatedCluster;
            } catch (err) {
                logger.error('ERROR updating cluster', JSON.stringify(clusterObj), err);
            }
        }, { concurrency: 30 });
    } else {
        result = await Promise.all(clustersWithActiveAgent.map(async clusterObj => {
            try {
                return await findInformationAndUpdateCluster(clusterObj);
            } catch (err) {
                logger.error('ERROR updating cluster', JSON.stringify(clusterObj), err);
            }
        }));
    }
    logger.info('Update clusters is done');
    return result;
}

// invoked on:
// auto discover
async function registerClusterAutoDiscover(accountId, clusterInformation) {
    logger.info('Register cluster auto discover', { accountId, clusterInformation });
    const { clusterId, status: { status }, agentCloudProvider } = clusterInformation;
    const ontapVersion = get(clusterInformation, 'ontapClusterProperties.ontapVersion');
    const agentId = get(clusterInformation, 'agentId');
    const license = [ON_STATUS, NO_TIERING_STATUS].includes(status) && ontapVersion >= 9.2 ? await getFabricpoolLicenses(agentId, clusterId, accountId) : undefined;
    return updateCluster(accountId, clusterInformation, license, agentCloudProvider, true);
}

async function updateClusterSubscription(accountId, cloudProvider, serviceName) {
    try {
        logger.info('Updating cluster subscription', {
            cloudProvider, serviceName, accountId
        });
        const subscription = await findSubscribedSubscriptionByAccountIdAndCloudProvider(accountId, cloudProvider);
        const clustersOfAccount = await Cluster.find({ accountId }).exec();
        // find subscription to update
        const clustersIdsToUpdate = clustersOfAccount.filter(cluster => {
            const { cloudProvider: clusterCloudProvider, agentLocation } = cluster;
            return clusterCloudProvider === cloudProvider || agentLocation === cloudProvider;
        }).map(filteredCluster => filteredCluster.clusterId);
        if (subscription && clustersIdsToUpdate.length) {
            const { subscriptionId } = subscription;
            logger.info(`Updating clusters ${clustersIdsToUpdate} with subscription ${JSON.stringify(subscription)}`);
            await Cluster.updateMany({ accountId, clusterId: { $in: clustersIdsToUpdate } }, { $set: { subscriptionId, sendNotification: true } });
        } else {
            logger.info(`Deleting clusters ${clustersIdsToUpdate} subscriptionId`);
            await Cluster.updateMany({ accountId, clusterId: { $in: clustersIdsToUpdate } }, { $unset: { subscriptionId: '' }, $set: { sendNotification: true } });
        }
    } catch (err) {
        logger.error(`Failed to update subscription and clusters for account ${accountId}`, err);
    }
}

async function updateClusterWithCloudProviderAndSubscriptionId(accountId, clusterId, cloudProvider) {
    const update = { $set: { cloudProvider, sendNotification: true } };
    if (cloudProvider === SGWS) {
        update.$unset = { subscriptionId: '' };
    } else {
        const subscription = await findSubscribedSubscriptionByAccountIdAndCloudProvider(accountId, cloudProvider);
        if (subscription) {
            update.$set.subscriptionId = subscription.subscriptionId;
        }
    }
    await Cluster.update({ accountId, clusterId }, update);
}

module.exports = {
    updateClusters,
    findInformationAndUpdateCluster,
    updateClusterWithCloudProviderAndSubscriptionId,
    updateClusterSubscription,
    registerClusterAutoDiscover
};
