const { get } = require('lodash');
const bytes = require('bytes');
const { ApolloError } = require('apollo-server');
const {
    signLicense, sendLicenseRequestToIPA, sendLookupRequestToIPA
} = require('../lib/ipa');
const { updateAccountFreeTrial } = require('./subscription-operations');
const { applyLicense } = require('../lib/agent/zapi');
const { Account } = require('../models/account');
const { encryptPassword } = require('../utils/utils');
const { Cluster } = require('../models/cluster');
const { getAccountId } = require('../utils/async-local-storage');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

async function generateLicense(capacity, clusterId) {
    const fabricPoolLicense = {
        serialNumber: '22654892', version: '1', product: 'fabricpool', licenses: { capacity, type: 'capacity', package: 'FabricPool' }
    };
    logger.info(`About to generate signed license ${JSON.stringify(fabricPoolLicense)} on clusterId ${clusterId}`);
    return signLicense(fabricPoolLicense);
}

async function createLicenseAndApply(clusterId, licenseCapacity, accountId, agentId) {
    const license = await generateLicense(licenseCapacity, clusterId);
    logger.info(`About to apply license ${JSON.stringify(license)} on cluster ${clusterId}`);
    return applyLicense(agentId, clusterId, license);
}

async function applyBYOLLicense(nssUserName, nssPassword, serialNumber, packageName, cluster,
    newLicenseCapacity, shouldApplyLicenseToCluster = false) {
    // register to ipa
    const {
        clusterId, clusterUuid, agentId, accountId
    } = cluster;
    const license = await sendLicenseRequestToIPA(nssUserName, nssPassword, serialNumber, packageName,
        'register', clusterUuid);
    const statusCode = license?.statusResp?.statusCode;
    if (statusCode !== 'SUCCESS') { // check if we had any error while registering the license
        throw new ApolloError(`Failed to apply BYOL license: ${get(license, 'statusResp.message')}`, 403);
    }

    try {
        if (shouldApplyLicenseToCluster) { // then also apply license to cluster
            logger.info('Applying BYOL license to cluster');
            await applyLicense(agentId, clusterId, license);
        }
        logger.info('Updating cluster license in data base');
        // update byol SN and capacity in DB
        await Cluster.findOneAndUpdate({ clusterId, accountId }, { $set: { byolLicense: { maxCapacity: newLicenseCapacity, serialNumber }, sendNotification: true } }).exec();
        await updateAccountFreeTrial(accountId, new Date());
    } catch (err) {
        logger.error('Failed to apply license. Unregistering license', err);
        // Unregister to ipa
        const data = await sendLicenseRequestToIPA(nssUserName, nssPassword, serialNumber, packageName, 'Unregister', clusterUuid);
        const unregisteredStatusCode = data?.statusResp?.statusCode;
        if (unregisteredStatusCode !== 'SUCCESS') { // check if we had any error while registering the license
            throw new ApolloError(`Failed to unregister BYOL license: ${get(data, 'statusResp.message')}`, 403);
        }
        throw new ApolloError(err.message, 403);
    }
}

async function applyByolLicenseOperation(nssUserName, nssPassword, clusterId, serialNumber) {
    logger.info('Applying BYOL license', {
        nssUserName, nssPassword, clusterId, serialNumber
    });
    const accountId = getAccountId();

    // SN starts with 36, or 312
    const packageName = serialNumber.startsWith('36') ? 'FABRICPOOL-TB' : 'FABRICPOOL-AFF-TB';
    const { records: { license_record: data } } = await sendLookupRequestToIPA(nssUserName, nssPassword, serialNumber);
    const statusCode = data?.statusResp?.statusCode;
    if (statusCode !== 'SUCCESS') {
        throw new ApolloError(`Failed to apply BYOL license: ${get(data, 'statusResp.message')}`, 403);
    }

    const { statusResp: { licenses: { capacity } } } = data; // capacity in TB
    const newLicenseCapacity = bytes(`${capacity}TB`);
    const clusterResult = await Cluster.aggregate([{ $match: { clusterId, accountId } },
        {
            $lookup: {
                from: 'accounts',
                localField: 'accountId',
                foreignField: 'accountId',
                as: 'accountData'
            }
        },
        { $unwind: '$accountData' }]).exec();
    if (clusterResult.length) {
        const [cluster] = clusterResult;
        const { accountData: account } = cluster;
        // save nss credentials in db if needed
        if (!account.nssCredentials) {
            const encryptedNssPassword = encryptPassword(nssPassword);
            await Account.findOneAndUpdate({ accountId }, { $set: { nssCredentials: { nssUserName, nssPassword: encryptedNssPassword } } }).exec();
        }
        if (cluster.byolLicense) { // cluster has byol license
            throw new ApolloError('BYOL license already applied to this cluster.', 403);
        } else if (cluster.tieringLicense) { // cluster has no byol license in the DB and has payho installed
            if (newLicenseCapacity > cluster.tieringLicense.maxCapacity) { // if new license tb greater then tiering license max cap
                logger.info('Cluster does not have BYOL license, but has tiering license. New BYOL license capacity', newLicenseCapacity, 'is bigger than existing tiering license:', cluster.tieringLicense.maxCapacity);
                await applyBYOLLicense(nssUserName, nssPassword, serialNumber, packageName, cluster, newLicenseCapacity, true);
            } else {
                logger.info('Cluster does not have BYOL license, but has tiering license. New BYOL license capacity', newLicenseCapacity, 'is less or equal to existing tiering license:', cluster.tieringLicense.maxCapacity);
                await applyBYOLLicense(nssUserName, nssPassword, serialNumber, packageName, cluster, newLicenseCapacity);
            }
        } else { // if no paygo and no byol
            logger.info('Cluster has no BYOL license nor paygo.');
            await applyBYOLLicense(nssUserName, nssPassword, serialNumber, packageName, cluster, newLicenseCapacity, true);
        }
    } else {
        throw new ApolloError(`Couldn't find cluster with id ${clusterId}`, 403);
    }
}

module.exports = {
    createLicenseAndApply, generateLicense, applyByolLicenseOperation
};
