const bytes = require('bytes');
const config = require('config');
const { get } = require('lodash');
const { findClustersToIncreaseLicense } = require('../models/account');
const getLogger = require('../utils/logger/logger');
const { bytesToCapacity, decryptPassword } = require('../utils/utils');
const { createLicenseAndApply } = require('./ipa-operations');
const { Cluster } = require('../models/cluster');
const { sendLicenseRequestToIPA } = require('../lib/ipa');
const { applyLicense } = require('../lib/agent/zapi');

const logger = getLogger();

// offline operation
async function updateLicense(cluster) {
    logger.info('update license capacity to cluster', JSON.stringify(cluster));
    const {
        agentId,
        maxCapacity, clusterId, accountId, currentCapacity, licenseCapacityRatio
    } = cluster;
    try {
        logger.info(`License capacity ratio of cluster ${clusterId} in ${accountId} is: 
        ${licenseCapacityRatio}, license capacity threshold: ${config.get('increase-license-capacity-ratio')}, current capacity: ${currentCapacity} bytes, max capacity: ${maxCapacity} bytes`);
        const { size: newLicenseCapacity } = bytesToCapacity(maxCapacity + bytes('10TB'));
        logger.info(`Going to update license of accountId: ${accountId}, clusterId: ${clusterId} with license capacity of ${newLicenseCapacity}`);
        await createLicenseAndApply(clusterId, newLicenseCapacity, accountId, agentId);
    } catch (err) {
        logger.error(`Failed to update license for Cluster ${clusterId}. Skipping...`, err);
    }
}

async function updateLicenses() {
    const clusters = await findClustersToIncreaseLicense(config.get('increase-license-capacity-ratio'));
    logger.info('clusters to increase license', JSON.stringify(clusters));
    await Promise.all(clusters.map(async cluster => {
        try {
            await updateLicense(cluster);
            logger.info('Successfully finished update license for cluster', JSON.stringify(cluster));
        } catch (err) {
            logger.error('Failed to update license for cluster', JSON.stringify(cluster));
        }
    }));
}

async function updateByolLicenses() {
    logger.info('Checking if there are any clusters with BYOL licenses that needs to be updated in DB');
    // find clusters with byol license in data base
    const clustersWithByol = await Cluster.aggregate([{ $match: { byolLicense: { $exists: true } } },
        {
            $lookup: {
                from: 'accounts',
                localField: 'accountId',
                foreignField: 'accountId',
                as: 'accountData'
            }
        },
        { $unwind: '$accountData' }]).exec();
    logger.info('Found clusters with byol license:', clustersWithByol);

    await Promise.all(clustersWithByol.map(async cluster => {
        try {
            const { accountId, clusterUuid, accountData: account } = cluster;
            if (account.nssCredentials) {
                const { clusterId, byolLicense: { serialNumber, maxCapacity } } = cluster;
                const { nssCredentials: { nssUserName, nssPassword: nssEncryptedPassword } } = account;
                // decrypt nss password
                const nssPassword = decryptPassword(nssEncryptedPassword);
                let packageName;
                if (serialNumber.startsWith('36')) {
                    packageName = 'FABRICPOOL-TB';
                } else {
                    packageName = 'FABRICPOOL-AFF-TB';
                }
                const license = await sendLicenseRequestToIPA(nssUserName, nssPassword, serialNumber, packageName, 'download', clusterUuid);
                const statusCode = get(license, 'statusResp.statusCode');
                if (statusCode !== 'SUCCESS') {
                    throw new Error(`Failed to get BYOL license info: ${get(license, 'statusResp.message')}`);
                }
                const { statusResp: { licenses: { capacity } } } = license;
                const licenseCapacity = bytes(`${capacity}TB`);
                if (maxCapacity !== licenseCapacity) { // if max capacity from db different than license capacity from ipa
                    logger.info('Capacities are different. Applying BYOL license to cluster');
                    // apply byol to cluster
                    const { agentId } = cluster;
                    await applyLicense(agentId, clusterId, license, false);
                    // update data base
                    logger.info('Updating data base with new capacity');
                    await Cluster.findOneAndUpdate({ clusterId, accountId }, { $set: { 'byolLicense.maxCapacity': licenseCapacity, sendNotification: false } });
                }
            }
        } catch (err) {
            logger.error('Failed to update BYOL license in DB', JSON.stringify(err));
        }
    }));
}

module.exports = {
    updateLicenses, updateLicense, updateByolLicenses
};
