const { ApolloError } = require('apollo-server');
const {
    groupBy, isEmpty, first, sortBy
} = require('lodash');
const moment = require('moment');
const { Cluster } = require('../models/cluster');
const {
    getResources, getAccounts, getWorkspaces, createWorkspace,
    getResourcesByAccount
} = require('../lib/cloud-manager/multi-tenancy');
const { jwtGetUserInfo } = require('../lib/auth0');
const { getAgent, getAgents } = require('../lib/cloud-manager/agents');
const {
    setAccountId, getUserToken, getAccountId, setClusterId
} = require('../utils/async-local-storage');
const {
    UNKNOWN_STATUS, VSA, CVO,
    FREE_TRIAL_DAYS, AGENT_ACTIVE_STATUS
} = require('../utils/consts');
const HttpError = require('../utils/errors');
const { Account } = require('../models/account');
const { findAgentDetailsForResources } = require('./agents-operations');
const { clusterInfo } = require('./agent-zapi-operations');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

async function shouldAutomaticExtend() {
    try {
        const userMetadata = await jwtGetUserInfo(getUserToken());
        const email = userMetadata.email.trim().toLowerCase();
        logger.info(`New account in the DB for user ${email} `);
        return email.includes('@netapp') || email.includes('@test-netapp');
    } catch (err) {
        logger.error('Received error while trying to get userInfo ', err);
        // this is an unknown email - it will not get automatic extension
        return false;
    }
}

async function findAccount(accountId) {
    logger.info('find accounts', accountId);
    const accounts = await getAccounts(accountId);
    const account = accounts.find(accountObject => accountObject.accountPublicId === accountId);
    if (account) {
        if (account.isGov) {
            logger.error(`account with id ${accountId} is govCloud`);
            throw new ApolloError(`account  ${accountId} is not allowed to use SAAS`, 403);
        }
        const accountFromDb = await Account.findOne({ accountId: account.accountPublicId }).exec();
        const automaticExtension = accountFromDb ? accountFromDb.automaticExtension : await shouldAutomaticExtend();
        // eslint-disable-next-line
        const freeTrialExpiryDate = accountFromDb ? (automaticExtension ? moment().add(FREE_TRIAL_DAYS, 'days').valueOf() : accountFromDb.freeTrialExpiryDate) : (process.env.IS_SIMULATOR ? (automaticExtension ? moment().add(FREE_TRIAL_DAYS, 'days').valueOf() : account.freeTrialExpiryDate) : undefined);
        await Account.findOneAndUpdate({ accountId: account.accountPublicId }, {
            $setOnInsert: {
                automaticExtension,
                freeTrialExpiryDate
            }
        }, { upsert: true, new: true });
    } else {
        logger.error(`account with id ${accountId} not found for user`);
        throw new ApolloError(`account with id ${accountId} not found`);
    }
    return account.accountPublicId;
}

async function findAccountFromArgs(accountIdArg) {
    const accountIdFromCls = getAccountId();
    if (accountIdArg && accountIdArg !== accountIdFromCls) {
        const accountId = await findAccount(accountIdArg);
        setAccountId(accountId);
        return accountId;
    }
    return accountIdFromCls;
}

async function findWorkspace(accountPublicId) {
    const workspaces = await getWorkspaces(accountPublicId);
    logger.debug(`received workspaces ${JSON.stringify(workspaces)} of account ${accountPublicId}`);
    let workspace = first(sortBy(workspaces, workspaceToChoose => workspaceToChoose.workspaceName));
    if (isEmpty(workspaces)) {
        const workspaceName = 'Default';
        workspace = await createWorkspace(accountPublicId, workspaceName);
    }
    return workspace.workspacePublicId;
}

async function getClustersByResources(resources, accountId) {
    const resourcesInfo = await Promise.all(resources.map(async cluster => {
        const {
            agentStatus, agentCloudProvider, agentSystemId, agentId, resourceIdentifier: clusterId, resourceClass
        } = cluster;
        try {
            const clusterDetails = {
                resourceType: resourceClass === VSA ? CVO : resourceClass,
                agentId,
                agentCloudProvider,
                agentSystemId
            };
            if (agentStatus === AGENT_ACTIVE_STATUS) {
                const clusterResponse = await clusterInfo(agentId, clusterId);
                Object.assign(clusterDetails, clusterResponse);
            } else {
                logger.info(`agent ${agentId} has status ${agentStatus}- ${clusterId} is unknown`);
                const clusterFromDB = await Cluster.findOne({ accountId, clusterId }).exec();
                Object.assign(clusterDetails, {
                    publicId: clusterId,
                    clusterId,
                    agentId,
                    resourceType: resourceClass === VSA ? CVO : resourceClass,
                    name: clusterFromDB ? clusterFromDB.clusterName : undefined,
                    status: { status: UNKNOWN_STATUS, message: 'No active Connector found for resource' }
                });
            }
            return clusterDetails;
        } catch (err) {
            logger.error(`Failed receiving cluster information for cluster ${clusterId} with error ${err}`);
            const clusterFromDB = await Cluster.findOne({ accountId, clusterId }).exec();
            return {
                publicId: clusterId,
                clusterId,
                agentId,
                name: clusterFromDB ? clusterFromDB.clusterName : undefined,
                resourceType: resourceClass === VSA ? CVO : resourceClass,
                status: { status: 'FAILED', message: err.message }
            };
        }
    }));
    return resourcesInfo.flat();
}

async function getAgentIdByClusterId(clusterId, accountId, isInternal = false, setClusterIdParam = true) {
    if (setClusterIdParam) {
        // save clusterId for audit group on 'willSendResponse'
        setClusterId(clusterId);
    }

    // get all resources
    const resources = await getResourcesByAccount(accountId, isInternal);
    // filter by clusterId
    const foundResource = resources.find(resource => resource.resourceIdentifier === clusterId);
    if (foundResource) {
        return foundResource.agentId;
    }
    throw new HttpError(`Cluster with id ${clusterId} was not found`, 404);
}

async function getAgentDetailsByClusterId(clusterId, accountId) {
    const agentId = await getAgentIdByClusterId(clusterId, accountId);
    const { agent } = await getAgent(agentId, accountId);
    return {
        useDockerInfa: agent.useDockerInfra,
        agentStatus: agent.status,
        agentId,
        agentCloudProvider: agent.provider,
        agentSystemId: agent.systemId
    };
}

async function filterOutDeletedClusters(clusters) {
    const clusterIds = clusters.map(cluster => cluster.resourceIdentifier);
    const clustersFromDB = await Cluster.find({ clusterId: { $in: clusterIds } }).exec();
    return clusters.filter(cluster => {
        const clusterFoundOnDB = clustersFromDB.find(clusterObj => clusterObj.clusterId === cluster.resourceIdentifier);
        return !clusterFoundOnDB || !clusterFoundOnDB.deleted;
    });
}

async function getAllClustersInAccount(accountId, workspaceId, agentId) {
    const resources = await getResources(accountId, workspaceId);
    const resourcesByAgentId = agentId ? resources.filter(resource => resource.agentId === agentId) : resources;
    const { agents } = await getAgents(accountId);

    const resourcesInfo = await findAgentDetailsForResources(resourcesByAgentId, agents, accountId);
    // check if it has at-least one new infrastructure agent
    const filteredResources = await filterOutDeletedClusters(resourcesInfo);
    const resourcesWithNewAgent = filteredResources.filter(resource => resource.useDockerInfra);
    // in case of no resources at all we wont want to throw an error.
    if (resourcesWithNewAgent.length || !resourcesByAgentId.length) {
        return getClustersByResources(resourcesWithNewAgent, accountId);
    }
    return [];
}

function getResourcesOfValidAgents(allResources, validAgents) {
    const resourceByAgentId = groupBy(allResources, 'agentId');
    return Object.keys(resourceByAgentId)
        .filter(agentId => validAgents.find(agent => agent.agentId === agentId))
        .flatMap(agentId => {
            const agent = validAgents.find(agentObj => agentObj.agentId === agentId);
            const agentResources = resourceByAgentId[agentId];
            return agentResources.map(resource => {
                const { resourceClass, ...rest } = resource;
                return {
                    ...rest,
                    agent,
                    resourceType: resourceClass === VSA ? CVO : resourceClass
                };
            });
        });
}

async function verifyClusterInWorkspace(clusterId, workspaceId) {
    const clusters = await getResources(getAccountId(), workspaceId);
    if (!clusters.find(resource => resource.resourceIdentifier === clusterId)) {
        throw new ApolloError(`cluster ${clusterId} was not found on workspaceId ${workspaceId}`, 403);
    }
}

module.exports = {
    findWorkspace,
    getResourcesOfValidAgents,
    findAccount,
    getAgentDetailsByClusterId,
    getAllClustersInAccount,
    getAgentIdByClusterId,
    findAccountFromArgs,
    filterOutDeletedClusters,
    verifyClusterInWorkspace
};
