const {
    sortBy, first
} = require('lodash');
const moment = require('moment');
const { Account } = require('../models/account');
const { findAccountsWithSubscriptions } = require('../models/subscription');
const { findLicenseInfoOfCluster } = require('../models/cluster');
const { findSubscriptionForProvider, findSubscriptionOfAccountIdAndCloudProvider } = require('../models/mp-subscription');
const { listSubscriptionsForService } = require('../lib/subscription-service');
const { getAccountId } = require('../utils/async-local-storage');
const {
    SUBSCRIBED, OCCM_SERVICE_NAME_SUBSCRIPTION, AWS, AZURE, GCP,
    UNSUBSCRIBED, TIERING_SERVICE_NAME_SUBSCRIPTION, UPDATE_LICENSE_INFO_EVENT, FREE_TRIAL_DAYS, TIERING_AZURE_SERVICE_NAME_SUBSCRIPTION
} = require('../utils/consts');
const { updateMessages } = require('../graphql/mongo-async-iterator');
const HttpError = require('../utils/errors');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function calculateDaysLeftForTrial(freeTrialExpiryDate) {
    if (freeTrialExpiryDate) {
        const daysLeftForTrial = moment(freeTrialExpiryDate).startOf('day').diff(moment().startOf('day'), 'days');
        return (daysLeftForTrial >= 0) ? daysLeftForTrial : -1;
    }
}

// exists for the migration - can be deleted after the next release ( 15.11.19)
async function listAllSubscriptionsForAService(serviceName, nextContinuationToken = undefined, subscriptions = []) {
    const subscriptionsResponse = await listSubscriptionsForService(serviceName, nextContinuationToken);
    const { items, continuationToken } = subscriptionsResponse;
    const updatedSubscriptions = subscriptions.concat(items);
    if (continuationToken) {
        return listAllSubscriptionsForAService(serviceName, continuationToken, updatedSubscriptions);
    }
    return updatedSubscriptions;
}

async function listAllSubscriptions() {
    const cloudTieringSubscriptions = await listAllSubscriptionsForAService(TIERING_SERVICE_NAME_SUBSCRIPTION);
    const occmSubscriptions = await listAllSubscriptionsForAService(OCCM_SERVICE_NAME_SUBSCRIPTION);
    return occmSubscriptions.concat(cloudTieringSubscriptions);
}
//= ======================================================================

function findSubscription(subscriptionList) {
    logger.info('find subscription', subscriptionList);
    if (subscriptionList.length > 0) {
        const sortedSubscriptions = sortBy(subscriptionList.map(subscription => {
            const {
                serviceName, createdAt, status, cloudProvider, subscriptionId
            } = subscription;
            const scoreByServiceName = {
                [TIERING_SERVICE_NAME_SUBSCRIPTION]: 2,
                [TIERING_AZURE_SERVICE_NAME_SUBSCRIPTION]: 2,
                [OCCM_SERVICE_NAME_SUBSCRIPTION]: 1
            };
            return {
                score: scoreByServiceName[serviceName], status, cloudProvider, subscriptionId, createdAt, serviceName
            };
        }), ['score', 'createdAt']);
        // here in case the subscriptionList includes unsubscribed subscriptions as well - was relevant for subscriptionStatus field .
        const subscription = sortedSubscriptions.find(subscriptionObj => subscriptionObj.status === SUBSCRIBED);
        return subscription || first(sortedSubscriptions);
    }
}

async function trialExpiryCheck() {
    const accounts = await findAccountsWithSubscriptions(UPDATE_LICENSE_INFO_EVENT);
    await Promise.all(accounts.map(async account => {
        const { accountId } = account;
        try {
            await updateMessages(accountId, UPDATE_LICENSE_INFO_EVENT, {
                licenseInfo: {
                    accountId,
                    daysLeftForTrial: calculateDaysLeftForTrial(account.freeTrialExpiryDate)
                }
            });
        } catch (err) {
            logger.error('failed to publish stats for account', account, err);
        }
    }));
}

async function createSubscriptionsArray(accountId) {
    const subscriptionsArray = await findSubscriptionForProvider(accountId);
    return [AWS, AZURE, GCP].map(cloudProvider => {
        const subscription = subscriptionsArray.find(subscriptionObj => subscriptionObj.provider === cloudProvider);
        return subscription ? { provider: subscription.provider.toUpperCase(), status: subscription.status } : { provider: cloudProvider, status: UNSUBSCRIBED };
    });
}

async function findLicenseInfo() {
    const accountId = getAccountId();
    const account = await Account.findOne({ accountId }).exec();
    const daysLeftForTrial = await calculateDaysLeftForTrial(account.freeTrialExpiryDate);
    const subscriptions = await createSubscriptionsArray(accountId);
    return {
        accountId, daysLeftForTrial, subscriptions
    };
}
async function shouldBeAllowedToTier(clusterId) {
    const clusterLicenseInfo = await findLicenseInfoOfCluster(clusterId, getAccountId());
    if (clusterLicenseInfo.shouldEnforce) {
        throw new HttpError('Cluster is not allowed to tier data- please check your license or your subscription', 403);
    }
}

async function updateAccountFreeTrial(accountId, freeTrialExpiryDate = moment().add(FREE_TRIAL_DAYS, 'days').valueOf()) {
    logger.info('update account', { accountId, freeTrialExpiryDate });
    const daysLeftForTrial = calculateDaysLeftForTrial(freeTrialExpiryDate);
    const subscriptions = await createSubscriptionsArray(accountId);

    // if we update the free trial date for today (BYOL case )-
    // we should only update the ones with no automaticExtensions
    // ( otherwise we will get into a loop- delete expiration and then adding it again )
    const query = (daysLeftForTrial)
        ? { accountId, $or: [{ freeTrialExpiryDate: { $exists: false } }, { freeTrialExpiryDate: null }] }
        : { accountId, automaticExtension: false };
    logger.info('before updating account', query, freeTrialExpiryDate);
    const account = await Account.findOneAndUpdate(
        query, { $set: { freeTrialExpiryDate } }, { new: true }
    );
    if (account) {
        return updateMessages(accountId, UPDATE_LICENSE_INFO_EVENT, {
            licenseInfo: {
                accountId,
                daysLeftForTrial,
                subscriptions
            }
        });
    }
}

async function findSubscribedSubscriptionByAccountIdAndCloudProvider(accountId, cloudProvider) {
    const subscribedSubscriptions = await findSubscriptionOfAccountIdAndCloudProvider(accountId, cloudProvider);
    return findSubscription(subscribedSubscriptions);
}

module.exports = {
    calculateDaysLeftForTrial,
    trialExpiryCheck,
    findSubscription,
    shouldBeAllowedToTier,
    findLicenseInfo,
    listAllSubscriptions,
    listAllSubscriptionsForAService,
    updateAccountFreeTrial,
    findSubscribedSubscriptionByAccountIdAndCloudProvider,
    createSubscriptionsArray
};
