const Promise = require('bluebird');
const moment = require('moment');
const { ApolloError } = require('apollo-server');
const {
    partition, get, compact, groupBy, find, first
} = require('lodash');
const {
    HUBSPOT_FIELDS, SGWS, FREE_TRIAL_DAYS, STANDARD,
    ALL, BACKUP, ONTAP_VERSIONS_FEATURES_SUPPORT,
    AWS, AZURE, NONE, CLOUD_PROVIDER_TO_OCCM_PROVIDER,
    GCP,
    GCP_STORAGE_ENDPOINT
} = require('../utils/consts');
const { getAccountId } = require('../utils/async-local-storage');
const {
    getOntapVersion, getAggregatesByAggregateNames,
    getSvmVolumes, getAggregates, flexgroupAggregates
} = require('./agent-zapi-operations');
const {
    describeLifs,
    describeObjectStoreConfig, createObjectStoreConfig, modifyVolume, modifyVolumeMinimumCoolingDays, attachObjectStore
} = require('../lib/agent/zapi');
const { getAzureStorageAccountKeys } = require('../lib/agent/azure');
const { getAgentIdByClusterId, verifyClusterInWorkspace } = require('./multi-tenancy-operation');
const { updateAccountFreeTrial } = require('./subscription-operations');
const { s3EndpointByRegion, blobStorageEndpointByRegion } = require('./tiering-endpoint-operations');
const { updateHubspotContactByUserToken } = require('../lib/hubspot');
const { createLicenseAndApply } = require('./ipa-operations');
const {
    volumesTostring, volumeAggregates, validateEligibleForTiering
} = require('./cloud-manager-operations');
const { verifySgwsPermission } = require('./check-permission-operations');
const getCallerIdentity = require('../lib/aws/sts');
const { findRestrictedVolumes, filterEligibleVolumes } = require('./volume-operations');
const { supportedVolumeSettings } = require('./tiering-policy-operations');
const HttpError = require('../utils/errors');
const { validateSchema } = require('../utils/schema-validation');
const volumePropertiesSchema = require('../joi-schemas/tiering-schemas');
const { shouldBeAllowedToTier } = require('./subscription-operations');
const { updateClusterWithCloudProviderAndSubscriptionId } = require('./clusters-collection-operations');
const { findTierLevel } = require('./agent-zapi-operations');
const { regionByS3Endpoint } = require('./cluster-operations');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

const INITIAL_LICENSE_CAPACITY_TB = 10;

function filterRequestedVolumes(volumes, volumeIdentifiers) {
    return volumes.reduce((accumulator, volume) => {
        const requestVolume = volumeIdentifiers.find(volumeIdentifier => volumeIdentifier.name === volume.name && volumeIdentifier.svmName === volume.svmName);
        if (requestVolume) {
            volume.requestedTieringPolicy = requestVolume.tieringPolicy;
            volume.requestedMinimumCoolingDays = requestVolume.minimumCoolingDays;
            accumulator.requestedVolumes.push(volume);
        } else {
            accumulator.notRequestedVolumes.push(volume);
        }
        return accumulator;
    }, { requestedVolumes: [], notRequestedVolumes: [] });
}

async function createFBLicense(clusterId, agentId, accountId) {
    try {
        await createLicenseAndApply(clusterId, INITIAL_LICENSE_CAPACITY_TB, accountId, agentId);
        logger.info(`license apply on ${clusterId} successfully executed`);
    } catch (err) {
        logger.error(`Failed to create and apply license on ${clusterId}. skipping...`, err);
    }
}

function validateTieringPolicy(volume, clusterId, ontapVersion) {
    const supportedTieringPoliciesforVolumes = supportedVolumeSettings(ontapVersion, volume.volumeType === 'dp').map(settings => settings.tieringPolicy);
    if (!supportedTieringPoliciesforVolumes.includes(volume.requestedTieringPolicy)) {
        throw new ApolloError(`${volume.name} with svm ${volume.svmName} does not support ${volume.requestedTieringPolicy}. It supports the following tiering policies ${supportedTieringPoliciesforVolumes}`, 400);
    }
}

async function modifyVolumeTieringPolicy(agentId, clusterId, requestedVolumes, notRequestedVolumes, objectStoreName, ontapVersion) {
    logger.info('About to attach and modify volumes', requestedVolumes);
    const aggregates = await getAggregates(agentId, clusterId);

    const tieringPolicyErrors = [];
    const tieringPolicySuccess = [];
    const restrictedVols = findRestrictedVolumes(requestedVolumes, aggregates);
    // update restricted volumes
    const eligibleVolumes = requestedVolumes.filter(requestedVol => !find(restrictedVols, {
        name: requestedVol.name,
        svmName: requestedVol.svmName
    }) && requestedVol.state === 'online');
    const relevantAggregatesOfVolumes = [...new Set(eligibleVolumes.flatMap(vol => volumeAggregates(vol)))];
    const relevantAggregates = await getAggregatesByAggregateNames(agentId, clusterId, relevantAggregatesOfVolumes);
    const [aggregatesWithObjectStore, aggregatesWithoutObjectStore] = partition(relevantAggregates, aggregate => aggregate.objectStoreName);
    logger.info('aggregates with objectStore', JSON.stringify(aggregatesWithObjectStore), 'aggregates without objectStore', JSON.stringify(aggregatesWithoutObjectStore));
    const aggregatesWithoutObjectStoreNames = aggregatesWithoutObjectStore.map(aggregate => aggregate.name);
    // update tiering policy for not-requested volumes - because the default tiering policy of a volume is snapshot_only and we dont see it until there is an object store attached . ( occm returns null)
    const volumesToChangeToNone = notRequestedVolumes.filter(volume => aggregatesWithoutObjectStoreNames.includes(...volumeAggregates(volume)) && volume.tieringPolicy !== NONE);
    logger.info('Volumes not selected with policy to change', volumesTostring(volumesToChangeToNone));
    await Promise.map(volumesToChangeToNone, async volume => {
        try {
            await modifyVolume(agentId, clusterId, volume.svmName, volume.name, NONE);
        } catch (err) {
            logger.error('Failed to change volume tierting poicy to NONE');
        }
    });

    // we attach object store to all eligible aggregates
    const failedAggregates = compact(await Promise.all(aggregatesWithoutObjectStoreNames.map(async aggregate => {
        try {
            const allowFlexgroup = ontapVersion >= ONTAP_VERSIONS_FEATURES_SUPPORT.FLEX_GROUP;
            await attachObjectStore(agentId, clusterId, aggregate, objectStoreName, false, allowFlexgroup);
        } catch (err) {
            logger.error(`Error while trying to attach object store to aggregate ${aggregate}. ${err.message}`);
            return { name: aggregate, reason: err.message };
        }
    })));

    const { failedVolumes, volumesToChangeTieringPolicy } = filterEligibleVolumes(eligibleVolumes, failedAggregates);

    // update tiering policy for requested volumes
    await Promise.map(volumesToChangeTieringPolicy, async volume => {
        try {
            validateTieringPolicy(volume, clusterId, ontapVersion);
            if (volume.requestedTieringPolicy) {
                await modifyVolume(agentId, clusterId, volume.svmName, volume.name, volume.requestedTieringPolicy);
                volume.changedTieringPolicy = volume.requestedTieringPolicy;
                tieringPolicySuccess.push(volume);
            }
        } catch (err) {
            const failedVolume = { name: volume.name, svmName: volume.svmName, reason: err.message };
            tieringPolicyErrors.push(failedVolume);
        }
    }, { concurrency: 20 });
    return { tieringPolicySuccess, tieringPolicyErrors: tieringPolicyErrors.concat(failedVolumes) };
}

function validateMinimumCoolingDays(volume, ontapVersion) {
    if (ontapVersion < 9.4) {
        throw new ApolloError('changing minimum cooling days is supported only from 9.4 version');
    }
    const { tieringPolicy, requestedTieringPolicy } = volume;
    const currentTieringPolicy = requestedTieringPolicy || tieringPolicy;
    if ([ALL, BACKUP, NONE].includes(currentTieringPolicy)) {
        throw new ApolloError(`minimum cooling days parameter is not allowed on ${NONE}, ${ALL}, ${BACKUP} volume tiering policy`);
    }
}

function divideVolumesToOperations(volumeProperties) {
    return volumeProperties.reduce((accumulator, volume) => {
        if (volume.requestedTieringPolicy) {
            accumulator.tieringPolicyVolumes.push(volume);
        } if (volume.requestedMinimumCoolingDays) {
            accumulator.coolingDaysVolumes.push(volume);
        }
        return accumulator;
    }, { tieringPolicyVolumes: [], coolingDaysVolumes: [] });
}

async function validateModifyVolumesParams(clusterId, volumeProperties, objectStoreName, provider) {
    validateSchema(volumeProperties, volumePropertiesSchema);
    const volumesByNameAndSvmName = groupBy(volumeProperties, volume => `${volume.name}_${volume.svmName}`);
    const duplicateVolumes = Object.values(volumesByNameAndSvmName).filter(volumeArr => volumeArr.length > 1);
    if (duplicateVolumes.length) {
        logger.error('Received the same volume twice', JSON.stringify(duplicateVolumes));
        throw new ApolloError('You entered the same volume twice please try again', 400);
    }
    if (!objectStoreName) {
        throw new HttpError('Cannot change volumes properties - No object store config available on cluster', clusterId, 'please create one and try again');
    }
    if (provider !== CLOUD_PROVIDER_TO_OCCM_PROVIDER.SGWS) {
        await shouldBeAllowedToTier(clusterId);
    }
}

async function modifyCoolingDays(agentId, clusterId, volumes, ontapVersion) {
    logger.info('modify cooling days for volumes', volumes, { clusterId, ontapVersion });
    const minimumCoolingDaysSuccess = [];
    const modifyCoolingDaysErrors = [];
    await Promise.all(volumes.map(async volume => {
        const { requestedMinimumCoolingDays: minimumCoolingDays, svmName, name: volumeName } = volume;
        try {
            validateMinimumCoolingDays(volume, ontapVersion);
            await modifyVolumeMinimumCoolingDays(agentId, clusterId, volumeName, svmName, minimumCoolingDays);
            volume.changedCoolingDays = minimumCoolingDays;
            minimumCoolingDaysSuccess.push(volume);
        } catch (err) {
            logger.error('Failed to modify cooling days', err);
            modifyCoolingDaysErrors.push({ name: volume.name, svmName: volume.svmName, reason: err.message });
        }
    }));
    return { modifyCoolingDaysErrors, minimumCoolingDaysSuccess };
}

function handleFlexgroupAggregateName(svmVolumes) {
    const msIdToAggregateNames = flexgroupAggregates(svmVolumes);
    return svmVolumes.map(volume => {
        volume.aggregateName = volume.aggregateName || get(msIdToAggregateNames, `${volume.flexgroupMSid}`, []).join(',');
        return volume;
    });
}

async function modifyVolumeOperation(clusterId, volumeProperties, agentIdParam) {
    try {
        const accountId = getAccountId();
        await updateAccountFreeTrial(accountId, moment().add(FREE_TRIAL_DAYS, 'days').valueOf());
        const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, accountId);
        const [ontapVersion, objectStoreConfigs] = await Promise.all([
            getOntapVersion(agentId, clusterId, accountId),
            describeObjectStoreConfig(agentId, clusterId, accountId)]);
        const objectStoreConfigObject = first(objectStoreConfigs);
        const { objectStoreName, providerType } = objectStoreConfigObject;
        await validateModifyVolumesParams(clusterId, volumeProperties, objectStoreName, providerType);
        const svmVolumes = await getSvmVolumes(agentId, clusterId);
        const updatesSvmVolumes = handleFlexgroupAggregateName(svmVolumes);
        const volumes = updatesSvmVolumes.filter(volume => !(volume.isVserverRoot || volume.flexgroupConstituentIndex));
        const { requestedVolumes, notRequestedVolumes } = filterRequestedVolumes(volumes, volumeProperties);
        // verify requestedVolume exist
        if (!requestedVolumes.length) {
            throw new HttpError('Requested volumes were not found');
        }
        const { tieringPolicyVolumes, coolingDaysVolumes } = divideVolumesToOperations(requestedVolumes);
        // handle change tieringPolicy request
        const { tieringPolicySuccess, tieringPolicyErrors } = await modifyVolumeTieringPolicy(agentId, clusterId, tieringPolicyVolumes, notRequestedVolumes, objectStoreName, ontapVersion);
        // if we failed on tieringPolicy we will not try to change cooling days
        const volumesToUpdateMinimumCoolingDays = coolingDaysVolumes.filter(volume => !tieringPolicyErrors.find(errorVolume => errorVolume.name === volume.name && errorVolume.svmName === volume.svmName));
        // handle change minimumCoolingDays
        const { modifyCoolingDaysErrors, minimumCoolingDaysSuccess } = await modifyCoolingDays(agentId, clusterId, volumesToUpdateMinimumCoolingDays, ontapVersion);
        // if the same volume had tieringPolicy change and minimumCoolingDays change we will have duplicates
        const groupByNameAndSvmName = groupBy(minimumCoolingDaysSuccess.concat(tieringPolicySuccess), volume => `${volume.name}_${volume.svmName}`);

        const allSuccessfulVolumes = Object.values(groupByNameAndSvmName).map(value => {
            const [volume1, volume2] = value;
            Object.assign(volume1, { tieringPolicy: get(volume1, 'ch', get(volume2, 'tieringPolicy')) });
            const tieringPolicy = get(volume1, 'changedTieringPolicy', get(volume2, 'changedTieringPolicy'));
            return {
                name: volume1.name,
                svmName: volume1.svmName,
                tieringPolicy,
                minimumCoolingDays: get(volume1, 'changedCoolingDays', get(volume2, 'changedCoolingDays')),
                isTiered: tieringPolicy !== 'none'
            };
        });

        return {
            errors: modifyCoolingDaysErrors.concat(tieringPolicyErrors),
            data: allSuccessfulVolumes
        };
    } catch (err) {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.modify_volume);
        throw err;
    }
}

async function validateClusterTieringSetup(cloudProvider, clusterId, agentId, ipSpace) {
    const ontapVersion = await getOntapVersion(agentId, clusterId);
    const lifs = await describeLifs(agentId, clusterId);
    validateEligibleForTiering(ontapVersion, cloudProvider);
    // validate the ipSpace is in the cluster and it is on
    const interClusterLifwithIpSpace = lifs.find(lif => lif.ipSpace === ipSpace && lif.role === 'intercluster');
    if (!interClusterLifwithIpSpace) {
        throw new ApolloError(`There is no intercluster lif that contains ipSpace ${ipSpace} on cluster ${clusterId}`, 403);
    }
    if (cloudProvider !== SGWS) {
        await shouldBeAllowedToTier(clusterId);
    }
}

async function setupAwsClusterTiering(accountId, options) {
    const {
        aws: {
            agentId: agentIdParam, workspaceId,
            clusterId, ipSpace, credentials: { accessKey, secretKey }, region, bucket
        }
    } = options;
    await verifyClusterInWorkspace(clusterId, workspaceId);
    const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, accountId);
    await validateClusterTieringSetup(AWS, clusterId, agentId, ipSpace);
    await createFBLicense(clusterId, agentId, accountId);
    await getCallerIdentity(accessKey, secretKey);
    const objectStoreName = await createObjectStoreConfig(CLOUD_PROVIDER_TO_OCCM_PROVIDER.AWS, agentId, clusterId, s3EndpointByRegion(region), bucket, ipSpace, { keys: { accessKey, secretKey } });
    await updateClusterWithCloudProviderAndSubscriptionId(accountId, clusterId, AWS);
    const tierLevel = await findTierLevel(agentId, clusterId, { providerType: AWS, s3Name: bucket }, accountId);
    return {
        clusterId,
        tierInformation: {
            objectStoreName,
            regionName: regionByS3Endpoint(region),
            bucketName: bucket,
            providerType: AWS,
            tierLevel
        }
    };
}

async function setupAzureClusterTiering(accountId, options) {
    const {
        azure: {
            agentId: agentIdParam, clusterId, ipSpace, workspaceId,
            azureStorageAccount: azureAccount, azureResourceGroup, region, bucket
        }
    } = options;
    await verifyClusterInWorkspace(clusterId, workspaceId);
    const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, accountId);
    await validateClusterTieringSetup(AZURE, clusterId, agentId, ipSpace);
    await createFBLicense(clusterId, agentId, accountId);
    const { keys } = await getAzureStorageAccountKeys(agentId, azureAccount, azureResourceGroup);
    const serverEndPoint = blobStorageEndpointByRegion(region);
    const objectStoreName = await createObjectStoreConfig(CLOUD_PROVIDER_TO_OCCM_PROVIDER.AZURE, agentId, clusterId, serverEndPoint, bucket, ipSpace,
        { azureAccount, azurePrivateKey: keys[0].value });
    await updateClusterWithCloudProviderAndSubscriptionId(accountId, clusterId, AZURE);
    const tierLevel = await findTierLevel(agentId, clusterId, { providerType: AZURE, server: serverEndPoint }, accountId);

    return {
        clusterId,
        tierInformation: {
            objectStoreName,
            bucketName: bucket,
            providerType: AZURE,
            tierLevel
        }
    };
}

async function setupGcpClusterTiering(accountId, options) {
    const {
        gcp: {
            agentId: agentIdParam, workspaceId,
            clusterId, ipSpace, credentials: keys, bucket
        }
    } = options;
    await verifyClusterInWorkspace(clusterId, workspaceId);
    const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, accountId);
    await validateClusterTieringSetup(GCP, clusterId, agentId, ipSpace);
    await createFBLicense(clusterId, agentId, accountId);
    const objectStoreName = await createObjectStoreConfig(CLOUD_PROVIDER_TO_OCCM_PROVIDER.GCP, agentId, clusterId,
        GCP_STORAGE_ENDPOINT, bucket,
        ipSpace, { keys });
    await updateClusterWithCloudProviderAndSubscriptionId(accountId, clusterId, GCP);
    return {
        clusterId,
        tierInformation: {
            objectStoreName,
            bucketName: bucket,
            providerType: GCP,
            tierLevel: STANDARD
        }
    };
}

async function setupSgwsClusterTiering(accountId, options) {
    const {
        sgws: {
            agentId: agentIdParam, workspaceId,
            clusterId, ipSpace,
            credentials: keys, host, port, bucket
        }
    } = options;
    await verifyClusterInWorkspace(clusterId, workspaceId);
    const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, accountId);
    await validateClusterTieringSetup(SGWS, clusterId, agentId, ipSpace);
    await verifySgwsPermission(agentId, clusterId, bucket, keys, host, port);
    const objectStoreName = await createObjectStoreConfig(CLOUD_PROVIDER_TO_OCCM_PROVIDER.SGWS, agentId, clusterId,
        host, bucket, ipSpace, { keys, port });
    await updateClusterWithCloudProviderAndSubscriptionId(accountId, clusterId, SGWS);
    return {
        clusterId,
        tierInformation: {
            objectStoreName,
            bucketName: bucket,
            providerType: SGWS
        }
    };
}

async function setupClusterOperation(cloudProvider, options) {
    try {
        logger.info('setup Cluster Operation', JSON.stringify({
            cloudProvider, options
        }));
        const accountId = getAccountId();
        await updateAccountFreeTrial(accountId, moment().add(FREE_TRIAL_DAYS, 'days').valueOf());
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.object_store_config, moment().utc().startOf('day').valueOf());
        let response;
        switch (cloudProvider) {
            case AWS: {
                response = await setupAwsClusterTiering(accountId, options);
                break;
            }
            case AZURE: {
                response = await setupAzureClusterTiering(accountId, options);
                break;
            }
            case GCP: {
                response = await setupGcpClusterTiering(accountId, options);
                break;
            }
            case SGWS: {
                response = await setupSgwsClusterTiering(accountId, options);
                break;
            }
            default: {
                logger.error('Received unknown cloudProvider', cloudProvider);
                throw new ApolloError(`Received unknown cloudProvider ${cloudProvider}`, 403);
            }
        }
        return response;
    } catch (err) {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.object_store_config);
        throw err;
    }
}

module.exports = {
    setupClusterOperation,
    modifyVolumeOperation
};
