const os = require('os');
const Agenda = require('agenda');
const config = require('config');
const getLogger = require('./logger/logger');
const { setRequestId, setTask, getLocalStorage } = require('./async-local-storage');
const {
    AGENDA_PROCESS_EVERY, AGENDA_JOB_NAME_SAAS_METERING,
    AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY,
    AGENDA_JOB_NAME_TRIAL_EXPIRY_STATUS, AGENDA_JOB_NAME_UPDATE_CLUSTERS_COLLECTION, AGENDA_JOB_NAME_BACKUP_DB,
    AGENDA_JOB_UPDATE_BYOL_LICENSES_CAPACITY
} = require('./consts');
const { collectInfoAndCharge } = require('../operations/charging-operations');
const {
    updateLicenses,
    updateByolLicenses
} = require('../operations/license-operations');
const { trialExpiryCheck } = require('../operations/subscription-operations');
const { updateClusters } = require('../operations/clusters-collection-operations');
const { generateRequestId } = require('./utils');
const backup = require('./backup-db');

const logger = getLogger();

function define(agenda, name, interval, fn) {
    agenda.define(name, { lockLifetime: 10000 }, async (job, done) => {
        try {
            return await getLocalStorage().run(new Map(), async () => {
                setRequestId(generateRequestId());
                setTask(name);
                logger.info(`Starting job ${name}`);
                try {
                    return await fn();
                } catch (err) {
                    logger.error(`Failed to run job ${name}`, err);
                } finally {
                    logger.info(`Job ${name} completed`);
                    done();
                }
            });
        } catch (err) {
            logger.error('>>>>>>>>>>>>>ERROR', err);
        }
    });

    agenda.every(interval, name);
}

async function initAgenda(db) {
    if (!process.env.IS_STANDBY) {
        const agenda = new Agenda({
            name: `${os.hostname}-${process.pid}`,
            processEvery: AGENDA_PROCESS_EVERY,
            mongo: db,
            sort: { nextRunAt: 1 }
        });

        await new Promise((resolve, reject) => {
            agenda.on('ready', () => {
                logger.info('AGENDA ready');
                resolve();
            });

            agenda.on('error', err => {
                logger.error('Got agenda error', err);
                reject(err);
            });
        });

        agenda.start();

        define(agenda, AGENDA_JOB_NAME_SAAS_METERING, config.get('agenda.charging-interval'), () => collectInfoAndCharge());

        define(agenda, AGENDA_JOB_NAME_TRIAL_EXPIRY_STATUS, config.get('agenda.trial-expiry-status-interval'), () => trialExpiryCheck());

        define(agenda, AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY, config.get('agenda.license-capacity-increase-interval'), () => updateLicenses());

        define(agenda, AGENDA_JOB_NAME_UPDATE_CLUSTERS_COLLECTION, config.get('agenda.update-clusters-collection-interval'), () => {
            if (!process.env.DEMO) {
                return updateClusters();
            }
        });

        define(agenda, AGENDA_JOB_NAME_BACKUP_DB, config.get('agenda.db-backup-interval'), () => backup());

        define(agenda, AGENDA_JOB_UPDATE_BYOL_LICENSES_CAPACITY, config.get('agenda.byol-licenses-capacity-update-interval'), () => updateByolLicenses());
    }
}

module.exports = initAgenda;
