const { EventEmitter, once } = require('events');
const config = require('config');
const ms = require('ms');
const { queue } = require('async');
const { UNKNOWN_STATUS } = require('./consts');
const { isTieringContainerHealthy } = require('../operations/agents-operations');
const { getAgentsInternal } = require('../lib/cloud-manager/agents');
const { clusterInfo } = require('../operations/agent-zapi-operations');
const { adjustClusterResponse } = require('../operations/cluster-operations');
const { Cluster } = require('../models/cluster');
const { getResourcesOfValidAgents, filterOutDeletedClusters } = require('../operations/multi-tenancy-operation');
const { findValidAgents } = require('../operations/agents-operations');
const { getResources } = require('../lib/cloud-manager/multi-tenancy');
const {
    getAccountId, setRequestId, setAccountId, setUserPortalId, getLocalStorage, setUserName, getUserPortalId, getUserName
} = require('./async-local-storage');
const { generateRequestId } = require('./utils');

const getLogger = require('./logger/logger');

const logger = getLogger();

// Create a new object that ...
class MyEventArray extends EventEmitter {
    push(value) {
        this.emit('push', value);
    }
}

async function getClustersMetadata(accountId, agentId, workspaceId, resourceType) {
    const { agents } = await getAgentsInternal(accountId);
    const allResources = await getResources(accountId, workspaceId, true);
    const resourcesByAgentId = agentId ? allResources.filter(resource => resource.agentId === agentId) : allResources;
    const validAgents = findValidAgents(agents);
    const resourcesAndAgents = getResourcesOfValidAgents(resourcesByAgentId, validAgents);
    const filteredResources = await filterOutDeletedClusters(resourcesAndAgents);
    const resources = resourceType ? filteredResources.filter(resource => resource.resourceType === resourceType) : filteredResources;
    return resources.map(resource => {
        const { resourceIdentifier, ...rest } = resource;
        return { ...rest, clusterId: resourceIdentifier };
    });
}

async function createResources(resource, state) {
    // resourceAccountId for change streams
    const { agent, clusterId, accountId: resourceAccountId } = resource;
    const { agentId } = agent;
    const accountId = resourceAccountId || getAccountId();
    const clusterDetails = {
        agentId: resource?.agent?.agentId,
        agentCloudProvider: resource?.agent?.provider,
        agentSystemId: resource?.agent?.systemId,
        clusterId,
        accountId,
        resourceType: resource.resourceType
    };
    try {
        if (isTieringContainerHealthy(agent)) {
            const clusterResponse = await clusterInfo(agentId || agent.agentId, clusterId, accountId);
            Object.assign(clusterDetails, clusterResponse);
        } else {
            const clusterFromDB = await Cluster.findOne({ accountId, clusterId }).exec();
            Object.assign(clusterDetails, {
                clusterId,
                agentId: resource?.agent?.agentId,
                name: clusterFromDB ? clusterFromDB.clusterName : undefined,
                status: {
                    status: UNKNOWN_STATUS,
                    message: 'No active Connector found for resource'
                }
            });
        }
        const clusterResponse = await adjustClusterResponse(clusterDetails, resource.updateDb);
        state.push(clusterResponse);
    } catch (err) {
        logger.error('Failed to find cluster details for cluster', clusterId, err);
    }
}

// if we use recursive function we dont return {value:null} and then only return data when we have some
async function getNext(state) {
    if (state) {
        const cluster = await once(state, 'push');
        state.removeAllListeners('push');
        return { value: { clusterInfo: cluster }, done: false };
    }
    return { done: true };
}

// problems with setInterval -
// if the server stopped - everything is ok
// if there was disconnect - we should receive an error and disconnect

async function getMyIterator(accountId, agentId, workspaceId, resourceType) {
    let state = new MyEventArray();
    const resourcesQueue = queue(async resource => createResources(resource, state), config.get('subscription-number-of-clusters'));

    let resources = await getClustersMetadata(accountId, agentId, workspaceId, resourceType);
    resources.forEach(resource => resourcesQueue.push(resource));

    const userPortalId = getUserPortalId();
    const userName = getUserName();

    Cluster.watch([
        {
            $match: {
                $and: [{ operationType: { $in: ['insert', 'update', 'replace'] } },
                    {
                        'fullDocument.accountId': accountId,
                        'fullDocument.agentId': agentId,
                        'fullDocument.sendNotification': true
                    }]
            }
        },
        { $project: { _id: 1, ns: 1, fullDocument: 1 } }
    ], { fullDocument: 'updateLookup' })
        .on('change', data => getLocalStorage().run(new Map(), async () => {
            setUserName(userName);
            setUserPortalId(userPortalId);
            setRequestId(generateRequestId());
            setAccountId(accountId);
            logger.info('Change from change streams', data);
            const { fullDocument } = data;
            logger.info('>>>>>>>>>>fullDocument', fullDocument);
            const { clusterId: clusterIdFromUpdatedCluster } = fullDocument;
            const resourceToUpdate = resources.find(resource => resource.clusterId === clusterIdFromUpdatedCluster);
            if (resourceToUpdate) {
                // in case of byol we need to update all clusters - enforcement ,
                // in case of subscriptionId or cloudProvider we only need to update the specific cluster
                if (fullDocument.byolLicense) {
                    resources.forEach(resource => resourcesQueue.push({ ...resource, accountId, updateDb: false }));
                } else {
                    resourcesQueue.push({ ...resourceToUpdate, accountId, updateDb: false });
                }
            }
        }))
        .on('error', err => logger.info('>>>>>>ERROR, change stream', err));

    let count = 0;
    const intervalId = generateRequestId();
    const interval = setInterval(async () => {
        try {
            const requestId = generateRequestId();
            setRequestId(requestId);
            logger.info(`Set Interval ${intervalId} number ${count}  requestId ${requestId}`);
            resources = await getClustersMetadata(accountId, agentId, workspaceId, resourceType);
            resources.forEach(resource => resourcesQueue.push(resource));
            count += 1;
        } catch (err) {
            logger.error('>>>>>>>>>>>error', err);
        }
    }, ms('3m'));

    const iterator = {
        next: () => getNext(state),
        throw: err => {
            logger.info('error async iterator', err);
            clearInterval(interval);
            throw new Error(err.message);
        },
        return: async () => {
            logger.info('<<<<<<<<<Return async iterator');
            logger.info('CLEARING interval');
            clearInterval(interval);
            state = undefined;
            logger.info('AFTER clearing interval');
            return { done: true };
        }
    };

    return { [Symbol.asyncIterator]: () => iterator };
}

module.exports = getMyIterator;
