const moment = require('moment');
const { createReadStream, promises: fsPromises } = require('fs');
const { exec } = require('child_process');
const {
    DEFAULT_AWS_REGION, DB_CONNECTION_STRING, BACKUP_AWS_SECRET_KEY,
    BACKUP_AWS_ACCESS_KEY, AWS, BACKUP_BUCKET
} = require('./consts');
const { upload, getS3 } = require('../lib/aws/s3');
const getLogger = require('./logger/logger');

const logger = getLogger();

async function execute(command, timeout, env) {
    return new Promise((resolve, reject) => {
        exec(command, { timeout, env }, (error, stdout, stderr) => {
            if (error) {
                logger.error('Failed to execute command', error);
                reject(stderr || stdout || 'Command execution - timed out');
            } else {
                resolve(stdout);
            }
        });
    });
}

async function backup() {
    if (!process.env.IS_SIMULATOR) {
        const tempFile = `${process.env.NODE_ENV}-${moment().format('YYYY-MM-DD_HH-mm-ss')}.zmdb`;
        try {
            logger.info('Trying to create mongodb backup');
            const command = `mongodump --gzip --archive=${tempFile} --uri=${DB_CONNECTION_STRING}`;
            await execute(command);
            // #3 upload to S3
            logger.info('uploading to S3 backup file', tempFile);
            const s3 = getS3(AWS, DEFAULT_AWS_REGION, BACKUP_AWS_ACCESS_KEY, BACKUP_AWS_SECRET_KEY);
            const readStream = createReadStream(`./${tempFile}`);
            await upload(s3, BACKUP_BUCKET, tempFile, readStream);
        } catch (err) {
            logger.error('Failed to create DB backup', err);
            throw err;
        } finally {
            logger.info('Removing temporary file');
            await fsPromises.unlink(`./${tempFile}`);
        }
    } else {
        logger.info('Simulator- no need to backup DB');
    }
}

module.exports = backup;
