const Agent = require('agentkeepalive');
const ms = require('ms');
const config = require('config');
const { get } = require('lodash');
const got = require('got');
const HttpError = require('./errors');
const getLogger = require('./logger/logger');
const ZapiCache = require('./cache');

const logger = getLogger('got');

const agent = new Agent();

let cache;
if (!process.env.IS_SIMULATOR) {
    cache = new ZapiCache();
}

const hooks = {
    beforeRequest: [
        options => {
            const {
                method, headers, json, url: { href }
            } = options;
            logger.info(`${method} request with id ${headers['x-request-id']} to: ${href} body:`, json);
        }
    ],
    afterResponse: [
        response => {
            logger.info(`${response.req?.method} response status: ${response.statusCode} from: ${response.url}  body:`, JSON.stringify(response.body), 'took:', response.timings?.phases.total ?? 0, 'isFromCache:', response.isFromCache, response.headers['content-length']);
            return response;
        }
    ],
    beforeError: [
        error => {
            logger.error('Received error status:', get(error, 'response.status', get(error, 'response.statusCode', error.code || 500)), 'from:', get(error, 'response.url'), 'message', get(error, 'response.body'), 'Raw error:', error, 'took', error.timings?.phases.total);
            if (error.response) {
                // applicable for ParseError, HTTPError, got.MaxRedirectsError
                return new HttpError(get(error, 'response.body.message') || get(error, 'response.body'), get(error, 'response.body.statusCode') || get(error, 'response.statusCode') || get(error, 'code'));
            }
            return new HttpError(error.message, error.code);
        }
    ],
    beforeRetry: [
        (options, error, retryCount) => logger.info(`Before retry number: ${retryCount} on request to: ${options.url.href}, error: ${error}`)
    ]
};

const gotInstanceForAgent = got.extend({
    retry: config.get('got.retries-agent'),
    timeout: ms(config.get('got.timeout')),
    resolveBodyOnly: true,
    responseType: 'json',
    // agent: { http: agent },
    mutableDefaults: true,
    cache,
    hooks
});

const gotInstanceForServices = got.extend({
    retry: config.get('got.retries-services'),
    timeout: ms(config.get('got.timeout')),
    resolveBodyOnly: true,
    responseType: 'json',
    agent: { http: agent },
    mutableDefaults: true,
    hooks
});

const gotInstanceForOuterServices = got.extend({
    retry: config.get('got.retries-services'),
    timeout: ms(config.get('got.timeout')),
    resolveBodyOnly: true,
    responseType: 'json',
    agent: { http: agent },
    mutableDefaults: true,
    hooks
});

/*
function getCache() {
    return cache;
}
*/

module.exports = {
    gotInstanceForAgent, gotInstanceForServices, gotInstanceForOuterServices /* getCache */
};
