const log4js = require('log4js');
const config = require('config');
const stackTrace = require('stack-trace');
const {
    cloneDeep, isString, isError
} = require('lodash');
const path = require('path');
const { readFileSync } = require('fs');
const { format } = require('util');
const {
    getUserName, getTask, getAccountId, getRequestId
} = require('../async-local-storage');
const hideSecretsHelper = require('../hide-secrets');

function hideSecrets(logEventData) {
    try {
        const jsonObj = JSON.parse(logEventData);
        hideSecretsHelper(jsonObj);
        return JSON.stringify(jsonObj);
    } catch (err) {
        if (!isString(logEventData)) {
            hideSecretsHelper(logEventData);
        }
        return logEventData;
    }
}

function configureLayoutWithToken() {
    const configFileContent = JSON.parse(readFileSync(config.get('paths.log4js-config-file')));
    Object.keys(configFileContent.appenders).forEach(appenderType => {
        const { layout } = configFileContent.appenders[appenderType];
        if (layout) {
            const { pattern } = layout;
            if (pattern) {
                layout.tokens = {
                    requestId: () => getRequestId() || 'system',
                    accountId: () => getAccountId() || 'unknown',
                    userName: () => getUserName() || getTask() || 'unknown',
                    message: logEvent => format(...logEvent.data.map(obj => {
                        if (isError(obj)) {
                            return hideSecrets(obj);
                        }
                        // used to return hideSecrets(obj) without cloneDeep.
                        const objClone = cloneDeep(obj);
                        return hideSecrets(objClone);
                    }))
                };
            }
        }
    });

    log4js.configure(configFileContent);
}

configureLayoutWithToken();

function getLogger(category = 'server') {
    const logger = log4js.getLogger(category);
    const trace = stackTrace.parse(new Error('something went wrong'));
    logger.addContext('module', trace[1].getFileName().replace(/(^.*[\\/]src[\\/])|(^.*[\\/]cloudtiering[\\/])/, '').split(path.sep).join('.'));
    return logger;
}

module.exports = getLogger;
