const moment = require('moment');
const mongoose = require('mongoose');
const { Account } = require('../../src/models/account');
const { Cluster } = require('../../src/models/cluster');
const { MpSubscription } = require('../../src/models/mp-subscription');
const {
    TIERING_AZURE_SERVICE_NAME_SUBSCRIPTION, DB_CONNECTION_STRING, SUBSCRIBED, TIERING_SERVICE_NAME_SUBSCRIPTION, CHARGING_DIMENSION, GCP, AWS, AZURE,
    GCP_CHARGING_DIMENSION, OCCM_SERVICE_NAME_SUBSCRIPTION, OCCM_CHARGING_DIMENSION
} = require('../../src/utils/consts');
const {
    collectChargingInfo
} = require('../../src/operations/charging-operations');
const { createAccount, createSubscription, createCluster } = require('../testUtils');
const getLogger = require('../../src/utils/logger/logger');
const charge = require('../../src/lib/charging-service');

const logger = getLogger();

jest.mock('../../src/lib/charging-service');

beforeAll(async () => {
    try {
        await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
    } catch (e) {
        logger.error('Cannot connect to mongoDB');
    }
});

afterAll(async done => {
    try {
        await mongoose.connection.close();
        done();
    } catch (e) {
        logger.error('Error closing connection', e);
    }
});

describe('collectChargingInfo', () => {
    beforeEach(async () => {
        await Account.deleteMany();
        await MpSubscription.deleteMany();
        await Cluster.deleteMany();
    });

    test('Account with free trial and cluster with no subscription attached - should return an empty array', async () => {
        const accountId = '12';
        await Account.create(createAccount(accountId, moment().add(3, 'days')));
        await Cluster.create(createCluster(accountId, '1', '1', AWS, undefined,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        expect(await collectChargingInfo()).toEqual([]);
        expect(charge).not.toHaveBeenCalled();
    });

    test('Account with free trial and cluster with subscription attached - should return an empty array', async () => {
        const accountId = '10';
        const subscriptionId = '1';
        await Account.create(createAccount(accountId, moment().add(3, 'days')));
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Cluster.create(createCluster(accountId, '1', '1', AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        expect(await collectChargingInfo()).toEqual([]);
    });

    test('Account without free trial and no byol license and no subscription should return an empty array', async () => {
        const accountId = '13';
        await Account.create(createAccount(accountId));
        await Cluster.create(createCluster(accountId, '1', '1', AWS, undefined,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        expect(await collectChargingInfo()).toEqual([]);
    });

    test('Account without free trial and no byol license and with subscription should return an empty array', async () => {
        const accountId = '14';
        const subscriptionId = '1';
        await Account.create(createAccount(accountId));
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Cluster.create(createCluster(accountId, '1', '1', AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        expect(await collectChargingInfo()).toEqual([]);
    });

    test('Account without free trial and valid byol license and no subscription should return an empty array', async () => {
        const accountId = '16';
        await Account.create(createAccount(accountId));
        await Cluster.create(createCluster(accountId, '1', '1', AWS, undefined,
            undefined, { serialNumber: '123', maxCapacity: 2199023255552 }, 1099511627776, new Date()));
        expect(await collectChargingInfo()).toEqual([]);
    });

    test('Account without free trial and non valid byol license but without subscription should return an empty array', async () => {
        const accountId = '17';
        await Account.create(createAccount(accountId));
        await Cluster.create(createCluster(accountId, '1', '1', AWS, undefined,
            undefined, { serialNumber: '123', maxCapacity: 1099511627776 }, 2199023255552, new Date()));
        expect(await collectChargingInfo()).toEqual([]);
    });

    test('Account with expired free trial and expired byol license and subscription should return array with one cluster', async () => {
        const accountId = '19';
        const subscriptionId = '3';
        const clusterUuid = '1';
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Account.create(createAccount(accountId, moment().subtract(3, 'days')));
        await Cluster.create(createCluster(accountId, '1', clusterUuid, AWS, subscriptionId,
            undefined, { serialNumber: '123', maxCapacity: 1099511627776 }, 2199023255552, new Date()));
        const charging = await collectChargingInfo();
        expect(charging).toHaveLength(1);
        expect(charging[0]).toHaveProperty('timestamp');
        expect(charging[0]).toMatchObject({ subscriptionId, dimension: CHARGING_DIMENSION, accounts: [{ accountId, resources: [{ resourceId: clusterUuid, val: 1 }] }] });
    });

    test('Account with expired free trial and with subscription, 2 clusters with the same clusterUuid - one has earlier lastSeen - should return the one that updated last', async () => {
        const accountId = '20';
        const subscriptionId = '4';
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Account.create(createAccount(accountId, moment().subtract(3, 'days')));
        const clusterUuid = '1';
        await Cluster.create(createCluster(accountId, '1', clusterUuid, AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 2199023255552, moment().subtract(1, 'days')));
        await Cluster.create(createCluster(accountId, '2', clusterUuid, AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        const charging = await collectChargingInfo();
        expect(charging).toHaveLength(1);
        expect(charging[0]).toHaveProperty('timestamp');
        expect(charging[0]).toMatchObject({ subscriptionId, dimension: CHARGING_DIMENSION, accounts: [{ accountId, resources: [{ resourceId: clusterUuid, val: 1 }] }] });
    });

    test('Account with expired free trial and with aws  gcp and azure subscriptions, 3 clusters one with tiering for aws, gcp, azure', async () => {
        const accountId = '20';
        const subscriptionId = '4';
        const subscriptionId2 = '5';
        const subscriptionId3 = '6';
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, GCP, subscriptionId2, OCCM_SERVICE_NAME_SUBSCRIPTION));
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AZURE, subscriptionId3, TIERING_AZURE_SERVICE_NAME_SUBSCRIPTION));
        await Account.create(createAccount(accountId, moment().subtract(3, 'days')));
        const clusterUuid = '1';
        const clusterUuid2 = '2';
        const clusterUuid3 = '3';
        await Cluster.create(createCluster(accountId, '1', clusterUuid, AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        await Cluster.create(createCluster(accountId, '2', clusterUuid2, GCP, subscriptionId2,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        await Cluster.create(createCluster(accountId, '3', clusterUuid3, AZURE, subscriptionId3,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        const charging = await collectChargingInfo();
        expect(charging).toHaveLength(3);
        const gcpSubscriptionElement = charging.find(obj => obj.subscriptionId === subscriptionId2);
        const awsSubscriptionElement = charging.find(obj => obj.subscriptionId === subscriptionId);
        const azureSubscriptionElement = charging.find(obj => obj.subscriptionId === subscriptionId3);

        expect(gcpSubscriptionElement).toHaveProperty('timestamp');
        expect(awsSubscriptionElement).toHaveProperty('timestamp');
        expect(azureSubscriptionElement).toHaveProperty('timestamp');
        expect(gcpSubscriptionElement).toMatchObject(
            { subscriptionId: subscriptionId2, dimension: GCP_CHARGING_DIMENSION, accounts: [{ accountId, resources: [{ resourceId: clusterUuid2, val: 1 }] }] }
        );
        expect(awsSubscriptionElement).toMatchObject(
            { subscriptionId, dimension: CHARGING_DIMENSION, accounts: [{ accountId, resources: [{ resourceId: clusterUuid, val: 1 }] }] }
        );
        expect(azureSubscriptionElement).toMatchObject(
            { subscriptionId: subscriptionId3, dimension: OCCM_CHARGING_DIMENSION, accounts: [{ accountId, resources: [{ resourceId: clusterUuid3, val: 1 }] }] }
        );
    });

    test('2 accounts with expired free trial and with the same subscription , 2 clusters with the same clusterUuid - one has earlier lastSeen - should return the one that updated last', async () => {
        const accountId1 = '19';
        const accountId2 = '20';
        const subscriptionId = '3';
        await MpSubscription.create(createSubscription(accountId1, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await MpSubscription.create(createSubscription(accountId2, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Account.create(createAccount(accountId1, moment().subtract(3, 'days')));
        await Account.create(createAccount(accountId2, moment().subtract(3, 'days')));
        const clusterUuid = '1';
        await Cluster.create(createCluster(accountId1, '1', clusterUuid, AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 2199023255552, moment().subtract(1, 'days')));
        await Cluster.create(createCluster(accountId2, '2', clusterUuid, AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        const charging = await collectChargingInfo();
        expect(charging).toHaveLength(1);
        expect(charging[0]).toHaveProperty('timestamp');
        expect(charging[0]).toMatchObject({ subscriptionId, dimension: CHARGING_DIMENSION, accounts: [{ accountId: accountId2, resources: [{ resourceId: clusterUuid, val: 1 }] }] });
    });

    test('2 accounts with expired free trial and with different subscriptions , 2 clusters with the same clusterUuid - one has earlier lastSeen - should return the one that updated last', async () => {
        const accountId1 = '19';
        const accountId2 = '20';
        const subscriptionId = '3';
        const subscriptionId2 = '4';
        await MpSubscription.create(createSubscription(accountId1, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await MpSubscription.create(createSubscription(accountId2, SUBSCRIBED, AWS, subscriptionId2, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Account.create(createAccount(accountId1, moment().subtract(3, 'days')));
        await Account.create(createAccount(accountId2, moment().subtract(3, 'days')));
        const clusterUuid = '1';
        await Cluster.create(createCluster(accountId1, '1', clusterUuid, AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 2199023255552, moment().subtract(1, 'days')));
        await Cluster.create(createCluster(accountId2, '2', clusterUuid, AWS, subscriptionId2,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        const charging = await collectChargingInfo();
        expect(charging).toHaveLength(1);
        expect(charging[0]).toHaveProperty('timestamp');
        expect(charging[0]).toMatchObject({ subscriptionId: subscriptionId2, dimension: CHARGING_DIMENSION, accounts: [{ accountId: accountId2, resources: [{ resourceId: clusterUuid, val: 1 }] }] });
    });

    test('account with expired free trial and subscription , cluster with less than 1 TB- should return an empty array', async () => {
        const accountId1 = '19';
        const subscriptionId = '3';
        await MpSubscription.create(createSubscription(accountId1, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Account.create(createAccount(accountId1, moment().subtract(3, 'days')));
        const clusterUuid = '1';
        await Cluster.create(createCluster(accountId1, '2', clusterUuid, AWS, subscriptionId,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627, new Date()));
        const charging = await collectChargingInfo();
        expect(charging).toHaveLength(0);
    });

    test('Account with expired free trial and 2 subscriptions - should appear once in charging', async () => {
        const accountId = '19';
        const subscriptionId = '3';
        const subscriptionId2 = '4';
        const clusterUuid = '1';
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AWS, subscriptionId2, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Account.create(createAccount(accountId, moment().subtract(3, 'days')));
        await Cluster.create(createCluster(accountId, '1', clusterUuid, AWS, subscriptionId,
            undefined, { serialNumber: '123', maxCapacity: 1099511627776 }, 2199023255552, new Date()));
        const charging = await collectChargingInfo();
        expect(charging).toHaveLength(1);
        expect(charging[0]).toHaveProperty('timestamp');
        expect(charging[0]).toMatchObject({ subscriptionId, dimension: CHARGING_DIMENSION, accounts: [{ accountId, resources: [{ resourceId: clusterUuid, val: 1 }] }] });
    });
});
