const cloudManagerInfo = require('../../src/operations/cloud-manager-operations');
const {
    OBJECT_STORE_AVAILABLE_STATE, AWS, AZURE, GCP
} = require('../../src/utils/consts');

const compositeAggregates = [
    {
        name: 'aggr1',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        },
        availabilityState: OBJECT_STORE_AVAILABLE_STATE
    },
    {
        name: 'aggr2',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        },
        availabilityState: OBJECT_STORE_AVAILABLE_STATE
    }
];

const nonCompositeAggregates = [
    {
        name: 'aggr1',
        state: 'online',
        objectStoreName: null,
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        }
    },
    {
        name: 'aggr2',
        state: 'online',
        objectStoreName: null,
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        }
    }
];

const VOLUMES = [{
    name: 'vol1',
    svmName: 'svm_onprem',
    aggregateName: 'aggr1',
    state: 'online',
    constituentsAggregates: []
}, {
    name: 'vol2',
    svmName: 'svm_onprem',
    aggregateName: 'aggr1',
    state: 'online',
    constituentsAggregates: []
}];

describe('isTieredVolume', () => {
    test('should return tiered if tiered aggregate', () => {
        expect(cloudManagerInfo.isTieredVolume(VOLUMES[0], compositeAggregates)).toBeTruthy();
    });

    test('should return tiered if tiered aggregate and no tiering policy', () => {
        const volume = VOLUMES[0];
        volume.tieringPolicy = null;
        expect(cloudManagerInfo.isTieredVolume(volume, compositeAggregates)).toBeTruthy();
    });

    test('should return non tiered if not tiered aggregate', () => {
        expect(cloudManagerInfo.isTieredVolume(VOLUMES[0], nonCompositeAggregates)).not.toBeTruthy();
    });

    test('should return tiered if flex group and all aggregates are tiered', () => {
        const volume = VOLUMES[0];
        volume.aggregateName = 'aggr1,aggr2';
        expect(cloudManagerInfo.isTieredVolume(VOLUMES[0], compositeAggregates)).toBeTruthy();
    });

    test('should return tiered if flex group and some aggregates are tiered', () => {
        const volume = VOLUMES[0];
        volume.aggregateName = 'aggr1,aggr2';
        expect(cloudManagerInfo.isTieredVolume(VOLUMES[0], [compositeAggregates[0], nonCompositeAggregates[1]])).toBeTruthy();
    });

    test('should return non tiered if flex group and all aggregates are not tiered', () => {
        const volume = VOLUMES[0];
        volume.aggregateName = 'aggr1,aggr2';
        expect(cloudManagerInfo.isTieredVolume(VOLUMES[0], nonCompositeAggregates)).not.toBeTruthy();
    });
});

describe('validateEligibleForTiering', () => {
    test('should return true', () => {
        expect(cloudManagerInfo.validateEligibleForTiering(9.4, AWS)).toBeUndefined();
    });
    test('should throws when invalid ontap version AND AWS', () => {
        function validateInvalidVersion() {
            cloudManagerInfo.validateEligibleForTiering(9.1, AWS);
        }
        expect(validateInvalidVersion).toThrow('Tiering is not allowed. ONTAP Version 9.1 is not supported');
    });

    test('should throws when invalid ontap version AND AZURE', () => {
        function validateInvalidVersion() {
            cloudManagerInfo.validateEligibleForTiering(9.3, AZURE);
        }
        expect(validateInvalidVersion).toThrow('Tiering is not allowed. ONTAP Version 9.3 is not supported');
    });

    test('should throws when invalid ontap version AND GCP', () => {
        function validateInvalidVersion() {
            cloudManagerInfo.validateEligibleForTiering(9.4, GCP);
        }
        expect(validateInvalidVersion).toThrow('Tiering is not allowed. ONTAP Version 9.4 is not supported');
    });
});
