const moment = require('moment');
const subscriptionService = require('../../src/lib/subscription-service');
const {
    calculateDaysLeftForTrial, findSubscription, listAllSubscriptionsForAService
} = require('../../src/operations/subscription-operations');
const {
    UNSUBSCRIBED, SUBSCRIBED, TIERING_SERVICE_NAME_SUBSCRIPTION, OCCM_SERVICE_NAME_SUBSCRIPTION
} = require('../../src/utils/consts');

jest.mock('../../src/lib/subscription-service');

function createSubscriptionItem(subscriptionId, name, subscriptionStatus, createdAt, serviceName = TIERING_SERVICE_NAME_SUBSCRIPTION, cloudProvider = 'aws') {
    return {
        subscriptionId,
        cloudProvider,
        name,
        subscriptionStatus,
        serviceName,
        createdAt
    };
}

function createDBSubscriptionItem(subscriptionId, status, createdAt, serviceName = TIERING_SERVICE_NAME_SUBSCRIPTION, cloudProvider = 'aws') {
    return {
        subscriptionId,
        cloudProvider,
        status,
        serviceName,
        createdAt
    };
}

describe('calculateDaysLeftForTrial', () => {
    it('should return 0', async () => {
        expect(calculateDaysLeftForTrial(moment())).toEqual(0);
    });

    it('should return -1', async () => {
        expect(calculateDaysLeftForTrial(moment().subtract(1, 'days'))).toEqual(-1);
    });

    it('should return a number - 3 days', async () => {
        expect(calculateDaysLeftForTrial(moment().add(3, 'days'))).toEqual(3);
    });
});

describe('findSubscription', () => {
    it(`receive an array with occm ${SUBSCRIBED} subscription and tiering ${SUBSCRIBED} should return occm subscription`, () => {
        const subscription1 = createDBSubscriptionItem('subscriptionId', SUBSCRIBED, moment());
        const subscription2 = createDBSubscriptionItem('subscriptionId2', SUBSCRIBED, moment().subtract('1', 'days'), OCCM_SERVICE_NAME_SUBSCRIPTION);
        const subscription = findSubscription([subscription1, subscription2]);
        subscription2.score = 1;
        expect(subscription).toEqual(subscription2);
    });
    it(`receive an array with occm ${UNSUBSCRIBED} subscription and tiering ${UNSUBSCRIBED} should return occm subscription`, () => {
        const subscription1 = createDBSubscriptionItem('subscriptionId', UNSUBSCRIBED, moment());
        const subscription2 = createDBSubscriptionItem('subscriptionId2', UNSUBSCRIBED, moment().subtract('1', 'days'), OCCM_SERVICE_NAME_SUBSCRIPTION);
        const subscription = findSubscription([subscription1, subscription2]);
        subscription2.score = 1;
        expect(subscription).toEqual(subscription2);
    });
    it(`receive an array with occm ${SUBSCRIBED} subscription and tiering ${UNSUBSCRIBED} should return occm subscription`, () => {
        const subscription1 = createDBSubscriptionItem('subscriptionId', UNSUBSCRIBED, moment());
        const subscription2 = createDBSubscriptionItem('subscriptionId2', SUBSCRIBED, moment().subtract('1', 'days'), OCCM_SERVICE_NAME_SUBSCRIPTION);
        const subscription = findSubscription([subscription1, subscription2]);
        subscription2.score = 1;
        expect(subscription).toEqual(subscription2);
    });
});

describe('listAllSubscriptionsForAService', () => {
    it('list subscriptions with continuationToken, should call the function and return all the items', async () => {
        const subscription1 = createSubscriptionItem('subscriptionId', 'customerIdentifier1', UNSUBSCRIBED, moment());
        const subscription2 = createSubscriptionItem('subscriptionId2', 'customerIdentifier2', SUBSCRIBED, moment());
        subscriptionService.listSubscriptionsForService.mockReturnValueOnce({ items: [subscription1], continuationToken: 'a1' }).mockReturnValueOnce({ items: [subscription2] });
        const subscriptions = await listAllSubscriptionsForAService(TIERING_SERVICE_NAME_SUBSCRIPTION);
        expect(subscriptionService.listSubscriptionsForService).toHaveBeenCalledWith(TIERING_SERVICE_NAME_SUBSCRIPTION, 'a1');
        expect(subscriptions).toEqual([subscription1, subscription2]);
    });

    it('list subscriptions with continuationToken, should call the function and return only all subscriptions', async () => {
        const subscription1 = createSubscriptionItem('subscriptionId', 'customerIdentifier1', UNSUBSCRIBED, moment(), TIERING_SERVICE_NAME_SUBSCRIPTION, 'aws');
        const subscription2 = createSubscriptionItem('subscriptionId2', 'customerIdentifier2', SUBSCRIBED, moment(), TIERING_SERVICE_NAME_SUBSCRIPTION, 'gcp');
        subscriptionService.listSubscriptionsForService.mockReturnValueOnce({ items: [subscription1], continuationToken: 'a1' }).mockReturnValueOnce({ items: [subscription2] });
        expect(subscriptionService.listSubscriptionsForService).toHaveBeenCalledWith(TIERING_SERVICE_NAME_SUBSCRIPTION, 'a1');
        const subscriptions = await listAllSubscriptionsForAService(TIERING_SERVICE_NAME_SUBSCRIPTION);
        expect(subscriptions).toEqual([subscription1, subscription2]);
    });

    it('listSubscriptions without continuationToken, should return the items received', async () => {
        const subscription1 = createSubscriptionItem('subscriptionId', 'customerIdentifier1', UNSUBSCRIBED, moment());
        const subscription2 = createSubscriptionItem('subscriptionId2', 'customerIdentifier2', SUBSCRIBED, moment().subtract('1', 'days'));
        subscriptionService.listSubscriptionsForService.mockReturnValue({ items: [subscription2, subscription1] });
        const subscriptions = await listAllSubscriptionsForAService(TIERING_SERVICE_NAME_SUBSCRIPTION);
        expect(subscriptions).toEqual([subscription2, subscription1]);
    });

    it('listSubscriptions without continuationToken, received gcp and aws subscriptions, should return all subscriptions', async () => {
        const subscription1 = createSubscriptionItem('subscriptionId', 'customerIdentifier1', UNSUBSCRIBED, moment(), TIERING_SERVICE_NAME_SUBSCRIPTION, 'gcp');
        const subscription2 = createSubscriptionItem('subscriptionId2', 'customerIdentifier2', SUBSCRIBED, moment().subtract('1', 'days'), TIERING_SERVICE_NAME_SUBSCRIPTION, 'aws');
        subscriptionService.listSubscriptionsForService.mockReturnValue({ items: [subscription2, subscription1] });
        const subscriptions = await listAllSubscriptionsForAService(TIERING_SERVICE_NAME_SUBSCRIPTION);
        expect(subscriptions).toEqual([subscription2, subscription1]);
    });

    it('listSubscriptions without continuationToken,and no items should return empty array', async () => {
        subscriptionService.listSubscriptionsForService.mockReturnValue({ items: [] });
        const subscriptions = await listAllSubscriptionsForAService(TIERING_SERVICE_NAME_SUBSCRIPTION);
        expect(subscriptions).toEqual([]);
    });
});
