const { URL } = require('url');
const { nth } = require('lodash');
const nock = require('nock');
const {
    AGENT_GET_AWS_STORAGE_CLASS,
    AGENT_ENDPOINT, AGENT_API_PREFIX, AGENT_LIST_AWS_BUCKETS,
    AGENT_LIST_AWS_REGIONS
} = require('../../../src/utils/consts');
const { USERS, createScenarioForUser } = require('../simulator-state');
const { createBucketRequestSchema, setAwsStorageClassRequestSchema } = require('../../../src/joi-schemas/agent-schemas');
const { validateRequest } = require('../../../src/utils/schema-validation');
const { findUserId, escapeForwardSlash } = require('../simulator-utils');

const awsAgentScope = nock(AGENT_ENDPOINT)
    // AWS regions
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_LIST_AWS_REGIONS}`))
    .delay(100)
    .reply(() => [200, ['cn-north-1', 'sa-east-1', 'ap-southeast-1',
        'ap-south-1', 'eu-central-1', 'eu-west-1', 'us-west-1', 'us-east-2', 'us-east-1']])
    // get tier level
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_GET_AWS_STORAGE_CLASS}`))
    .query(query => query.bucket)
    .delay(100)
    .reply(function (url) {
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const requestUri = new URL(url, AGENT_ENDPOINT);
            const bucket = requestUri.searchParams.get('bucket');
            const bucketFound = USERS[userId].BUCKETS.AWS.find(bucketObj => bucketObj.name === bucket);
            return bucketFound ? [200, { storageClass: bucketFound.storageClass }] : [400, 'bucket not found'];
        }
        return [400, 'user not found'];
    })
    // AWS buckets
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_LIST_AWS_BUCKETS}`))
    .delay(100)
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const userId = findUserId(this.req.headers.authorization);
        const accountId = nth(requestUri.pathname.split('/'), 2);
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser(userId, accountId);
        }
        return [200, USERS[userId].BUCKETS.AWS];
    })
    // create bucket
    .post(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_LIST_AWS_BUCKETS}`), body => {
        try {
            validateRequest(body, createBucketRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const userId = findUserId(this.req.headers.authorization);
        const accountId = nth(requestUri.pathname.split('/'), 2);
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser(userId, accountId);
        }
        // body has name and region
        const { name } = body;
        const bucket = USERS[userId].BUCKETS.AWS.find(bucketObj => bucketObj.name === name);
        if (bucket) {
            return [400, { message: `A bucket with name ${name} already exist. please try again`, statusCode: 400 }];
        }
        USERS[userId].BUCKETS.AWS.push(body);
        return [200, body];
    })
    // change tier level
    .put(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_GET_AWS_STORAGE_CLASS}`), body => {
        try {
            validateRequest(body, setAwsStorageClassRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(function (url, body) {
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const { bucket, storageClass } = body;
            const indexOfBucket = USERS[userId].BUCKETS.AWS.findIndex(bucketObj => bucketObj.name === bucket);
            if (indexOfBucket > -1) {
                const bucketToChange = USERS[userId].BUCKETS.AWS[indexOfBucket];
                bucketToChange.storageClass = storageClass;
                USERS[userId].BUCKETS.AWS[indexOfBucket] = bucketToChange;
                return [200];
            }
            return [400, `bucket not found, ${body}`];
        }
        return [400, 'user not found'];
    });

awsAgentScope.persist(true);
