const { URL } = require('url');
const nock = require('nock');
const { nth } = require('lodash');
const {
    AGENT_ENDPOINT, AGENT_API_PREFIX, AGENT_API_PROVIDERS_PREFIX,
    AGENT_SET_GCP_STORAGE_CLASS, AGENT_LIST_GCP_BUCKETS, AGENT_LIST_GCP_REGIONS
} = require('../../../src/utils/consts');
const { setGcpStorageClassRequestSchema, createBucketRequestSchema } = require('../../../src/joi-schemas/agent-schemas');
const { validateSchema } = require('../../../src/utils/schema-validation');
const { USERS, createScenarioForUser } = require('../simulator-state');
const { findUserId, escapeForwardSlash } = require('../simulator-utils');

const gcpAgentScope = nock(AGENT_ENDPOINT)
    // GCP regions
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_REGIONS}`))
    .delay(100)
    .reply(() => [200, [
        'southeastasia', 'asia-east1', 'asia-east2', 'asia-northeast1', 'asia-northeast2', 'asia-northeast3', 'asia-south1', 'asia-southeast1', 'australia-southeast1', 'europe-north1', 'europe-west1', 'europe-west2', 'europe-west3', 'europe-west4', 'europe-west6', 'northamerica-northeast1', 'southamerica-east1', 'us-central1', 'us-east1', 'us-east4', 'us-west1', 'us-west2', 'us-west3']])
    // gcp buckets
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_BUCKETS}`))
    .delay(100)
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const userId = findUserId(this.req.headers.authorization);
        const accountId = nth(requestUri.pathname.split('/'), 2);

        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser(userId, accountId);
        }
        return [200, USERS[userId].BUCKETS.GCP];
    })
    // create bucket
    .post(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_BUCKETS}`), body => {
        try {
            validateSchema(body, createBucketRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const accountId = nth(requestUri.pathname.split('/'), 2);
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser(userId, accountId);
        }
        const { region, name } = body;
        const bucket = { region, name, storageClass: 'standard' };
        const bucketFound = USERS[userId].BUCKETS.GCP.find(bucketObj => bucketObj.name === name);
        if (bucketFound) {
            return [400, { message: `A bucket with name ${name} already exist. please try again`, statusCode: 400 }];
        }
        USERS[userId].BUCKETS.GCP.push(bucket);
        return [200, bucket];
    })
    .put(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}${AGENT_SET_GCP_STORAGE_CLASS}`), body => {
        try {
            validateSchema(body, setGcpStorageClassRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(function (uri, body) {
        const userId = findUserId(this.req.headers.authorization);
        // TODO right now it doesnt influence on the cluster -
        if (USERS[userId]) {
            const { bucket, storageClass } = body;
            const indexOfBucket = USERS[userId].BUCKETS.GCP.findIndex(bucketObj => bucketObj.name === bucket);
            if (indexOfBucket > -1) {
                const bucketToChange = USERS[userId].BUCKETS.GCP[indexOfBucket];
                bucketToChange.storageClass = storageClass;
                USERS[userId].BUCKETS.GCP[indexOfBucket] = bucketToChange;
                return [200];
            }
            return [400, `bucket not found, ${body}`];
        }
        return [400, 'user not found'];
    });

gcpAgentScope.persist(true);
