const { URL } = require('url');
const hyperid = require('hyperid');
const nock = require('nock');
const {
    last, set, get, nth
} = require('lodash');
const { validateResponse } = require('../../../src/utils/schema-validation');
const { agentSchema } = require('../../../src/joi-schemas/agents-schemas');
const {
    API_PATH_AGENTS, API_PATH_AGENTS_INTERNAL, AGENTS_MGMT_ENDPOINT
} = require('../../../src/utils/consts');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const { USERS, createScenarioForUser } = require('../simulator-state');

const agentsScope = nock(AGENTS_MGMT_ENDPOINT)
    // create agent
    .post(API_PATH_AGENTS, body => body.accountId)
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const userId = findUserId(this.req.headers['x-user-token']);
        const { accountId, agent: placement } = requestBody;
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser(userId, accountId);
        }
        const agentId = hyperid().uuid;
        const agentAndPlacement = { agentId, ...placement };

        validateResponse(agentAndPlacement, agentSchema);
        agentAndPlacement.systemId = hyperid().uuid;
        set(USERS, `${userId}.AGENTS.${accountId}.${agentId}`, agentAndPlacement);
        return agentAndPlacement;
    })

    // get agents
    .get(API_PATH_AGENTS)
    .query(query => query.account_id)
    .delay(100)
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENTS_MGMT_ENDPOINT);
        const accountId = requestUri.searchParams.get('account_id');
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId]) {
            const agentsObject = get(USERS, `${userId}.AGENTS.${accountId}`, {});
            const agentsResponse = { agents: Object.values(agentsObject) };
            return [200, JSON.stringify(agentsResponse)];
        }
        return [403, { statusCode: 403, message: 'User is not authorized to perform this action' }];
    })

    // get agent status
    .get(escapeForwardSlash(`${API_PATH_AGENTS}/.+`))
    .query(query => query.account_id)
    .delay(100)
    .reply(function (uri) {
        const userId = findUserId(this.req.headers['x-user-token']);
        const requestURL = new URL(uri, AGENTS_MGMT_ENDPOINT);
        const accountId = requestURL.searchParams.get('account_id');
        const agentId = last(requestURL.pathname.split('/'));
        if (USERS[userId]) {
            const agent = get(USERS, `${userId}.AGENTS.${accountId}.${agentId}`);
            if (agent) {
                return [200, { agent }];
            }
            return [404, { message: 'Agent not found', statusCode: 404 }];
        }
        return [404, 'User was not found'];
    })
    // delete/remove agent
    .delete(escapeForwardSlash(`${API_PATH_AGENTS}/.+`))
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, AGENTS_MGMT_ENDPOINT);
        const agentId = last(requestURL.pathname.split('/'));
        const userId = findUserId(this.req.headers['x-user-token']);
        const accountId = this.req.headers['x-tenancy-account-id'];
        if (USERS[userId]) {
            const agent = get(USERS, `${userId}.AGENTS${accountId}.${agentId}`);
            if (agent) {
                delete USERS[userId].AGENTS[accountId][agentId];
                return [200];
            }
            return [404,
                {
                    message: `Agent ${agentId} was not found`,
                    statusCode:
                        404
                }];
        }
        return [404, 'USER was not found'];
    })

    // change agent status
    .post(escapeForwardSlash(`${API_PATH_AGENTS}/.+/change-status`), body => body.status)
    .query(query => query.account_id)
    .delay(100)
    .reply(function (uri, body) {
        const { status } = body;
        const requestUri = new URL(uri, AGENTS_MGMT_ENDPOINT);
        const agentId = nth(requestUri.pathname.split('/'), 4);
        const userId = findUserId(this.req.headers.authorization);
        const accountId = requestUri.searchParams.get('account_id');
        const requestedAgent = get(USERS, `${userId}.AGENTS.${accountId}.${agentId}`);
        if (requestedAgent) {
            set(USERS, `${userId}.AGENTS.${accountId}.${agentId}.status`, status);
            return [200, requestedAgent];
        }
        return [404, {
            message: 'Agent not found',
            statusCode: 404
        }];
    })
// get agents by accountId (Internal)
    .get(API_PATH_AGENTS_INTERNAL)
    .query(query => query.account_id)
    .delay(100)
    .reply(uri => {
        const requestUri = new URL(uri, AGENTS_MGMT_ENDPOINT);
        const accountId = requestUri.searchParams.get('account_id');
        const allAgentsOfAllAccount = Object.values(USERS).filter(user => user.AGENTS[accountId]).map(user => Object.values(user.AGENTS[accountId])).flat(); // offline operation
        const allAgents = allAgentsOfAllAccount.map(agentObj => {
            const { clientInfraStatus, ...rest } = agentObj;
            return { clientInfraStatus, agent: { ...rest } };
        }).flat();
        if (allAgents) {
            return [200, { agents: allAgents }];
        }
        return [404, {
            message: 'Agents not found',
            statusCode: 404
        }];
    })
    // get agents (Internal)
    .get(API_PATH_AGENTS_INTERNAL)
    .delay(100)
    .reply(() => {
        const accountToAgents = Object.values(USERS).map(user => user.AGENTS); // offline operation
        const allAgentsObjects = accountToAgents.map(account => Object.values(account)).flat();
        const allAgents = allAgentsObjects.map(account => Object.values(account)).flat();
        if (allAgents) {
            return [200, {
                agents: allAgents.map(agentObj => {
                    const { clientInfraStatus, ...rest } = agentObj;
                    return { clientInfraStatus, agent: { ...rest } };
                })
            }];
        }
        return [404, {
            message: 'Agents not found',
            statusCode: 404
        }];
    });

agentsScope.persist(true);
