const {
    cloneDeep, isEmpty, get, random, set
} = require('lodash');
const randomize = require('randomatic');
const moment = require('moment');
const hyperid = require('hyperid');
const getLogger = require('../../src/utils/logger/logger');
const agents = require('./resources/agents');
const clusters = require('./resources/clusters-agent');
const svms = require('./resources/svms');
const objectStoreConfig = require('./resources/object-store-config');
const objectStores = require('./resources/object-stores');
const aggregates = require('./resources/aggregates-agent');
const volumes = require('./resources/volumes-agent');
const volumeFootPrints = require('./resources/volumesfootprint');
const buckets = require('./resources/buckets');
const gcpBuckets = require('./resources/gcpBuckets');
const sgwsBuckets = require('./resources/sgwsBuckets');
const containers = require('./resources/containers');
const storageAccounts = require('./resources/storageAccounts');
const resourceGroups = require('./resources/resourceGroups');
const mapping = require('./resources/mapping');
const byolLicenses = require('./resources/byol-licenses');

const logger = getLogger('simulator');

const MAPPING = {};

const USERS = {};
const SUBSCRIPTIONS = [];
setInterval(() => logger.info('USERS_STATE', JSON.stringify(USERS, null, 4)), 30 * 1000);

function createAccount(accountId, userId, trial, shouldFail, isGov = false) {
    const userIdParts = userId.split('|');
    const randomAccountId = userIdParts.length > 1 ? userIdParts[1] : userIdParts[0];
    const accountPublicId = accountId || `simulator-${randomAccountId}`;
    let freeTrialExpiryDate;
    if (trial === -1) {
        freeTrialExpiryDate = new Date('2/1/2016 00:00').valueOf();
    } else if (trial > 0) {
        freeTrialExpiryDate = new Date(Date.now() + moment.duration(trial, 'days')
            .as('milliseconds'));
    }

    return {
        accountName: `simulator-account-${randomize('Aa0', 8)}`,
        accountPublicId,
        freeTrialExpiryDate,
        shouldFail,
        isGov,
        override: true
    };
}

function createWorkspace() {
    return {
        workspacePublicId: `${randomize('Aa0', 8)}`,
        workspaceName: 'workspace',
        override: true
    };
}

function createScenarioForUser(userId, accountId, mappingObj = MAPPING) {
    const scenarioMapping = mappingObj;
    if (isEmpty(scenarioMapping)) {
        Object.assign(scenarioMapping, { clusters: [], agents: [] });
    }
    // for each cluster - volumes and aggregates should be cloned for each cluster
    const volumesCollection = cloneDeep(volumes);
    const aggregatesCollection = cloneDeep(aggregates);
    const clustersCollection = cloneDeep(clusters);
    const svmsCollection = cloneDeep(svms);
    const objectStoreConfigCollection = cloneDeep(objectStoreConfig);
    const objectStoresCollection = cloneDeep(objectStores);
    const volumeFootprintsCollection = cloneDeep(volumeFootPrints);
    const agentsCollection = cloneDeep(agents);
    const awsBucketsCollection = cloneDeep(buckets);
    const gcpBucketsCollection = cloneDeep(gcpBuckets);
    const azureContainersCollection = cloneDeep(containers);
    const sgwsBucketsCollection = cloneDeep(sgwsBuckets);
    const azureStorageAccountsCollection = cloneDeep(storageAccounts);
    const azureResourceGroupsCollection = cloneDeep(resourceGroups);
    const byolLicensesCollection = cloneDeep(byolLicenses);

    if (scenarioMapping.toCreateDynamically) {
        scenarioMapping.clusters = scenarioMapping.clusters.flatMap(cluster => {
            const { number } = cluster;
            return [...Array(number).keys()].map(() => cluster.cluster);
        });
        scenarioMapping.toCreateDynamically = false;
    }

    const agentsObject = scenarioMapping.agents.reduce((accumulator, agentName) => {
        const agent = agentsCollection.find(agentObject => agentObject.name === agentName);
        accumulator[agent.agentId] = agent;
        return accumulator;
    }, {});

    const workspace = createWorkspace();

    const account = createAccount(accountId, userId, scenarioMapping.trial, scenarioMapping.accountShouldFail, scenarioMapping.isGov);
    const ACCOUNTS = {
        [account.accountPublicId]: {
            account, workspaces: [workspace]
        }
    };

    const clustersObject = scenarioMapping.clusters.reduce((accumulator, cluster) => {
        const volumesForCluster = [];
        const aggregatesForCluster = [];
        const svmsForCluster = [];
        const objectStoreConfigForCluster = [];
        const objectStoresForCluster = [];
        const volumeFootprintsFotCluster = [];
        const {
            clusterId, svms: svmsToAdd, volumes: volumesToAdd, aggregates: aggregatesToAdd, agentName,
            objectStoreConfig: objectStoreConfigToAdd, objectStores: objectStoresToAdd, resourceClass, licenseSerial, licenseCapacity, toCreateDynamically, licenseMaxCapacity
        } = cluster;

        const agent = Object.values(agentsObject).find(agentObj => agentObj.name === agentName);
        aggregatesCollection.forEach(aggregate => {
            if (aggregatesToAdd.includes(aggregate.name)) {
                aggregatesForCluster.push(cloneDeep(aggregate));
            }
        });
        if (toCreateDynamically) {
            volumesToAdd.forEach(volumeObj => {
                const { number, volume: name } = volumeObj;
                const volumeFromTheCollection = volumesCollection.find(volume => volume.name === name);
                const volumefootprintFromTheCollection = volumeFootprintsCollection.find(volume => volume.name === name);
                [...Array(number).keys()].forEach(key => {
                    const clonedVol = cloneDeep(volumeFromTheCollection);
                    const clonedVolFootprint = cloneDeep(volumefootprintFromTheCollection);
                    clonedVol.name = `${clonedVol.name}_${key}`;
                    clonedVolFootprint.name = `${clonedVolFootprint.name}_${key}`;
                    volumesForCluster.push(cloneDeep(clonedVol));
                    volumeFootprintsFotCluster.push(cloneDeep(clonedVolFootprint));
                });
            });
        } else {
            volumesCollection.filter(volume => volumesToAdd.includes(volume.name))
                .forEach(volume => volumesForCluster.push(cloneDeep(volume)));
            volumeFootprintsCollection.filter(volume => volumesToAdd.includes(volume.name))
                .forEach(volume => volumeFootprintsFotCluster.push(cloneDeep(volume)));
        }
        svmsCollection.filter(svm => svmsToAdd.includes(svm.svmName)).forEach(svm => svmsForCluster.push(cloneDeep(svm)));
        objectStoreConfigCollection.filter(objectStoreConfigObj => objectStoreConfigToAdd.includes(objectStoreConfigObj.objectStoreName)).forEach(objectStoreConfigObj => objectStoreConfigForCluster.push(cloneDeep(objectStoreConfigObj)));
        objectStoresCollection.filter(objectStore => objectStoresToAdd.includes(objectStore.name)).forEach(objectStore => objectStoresForCluster.push(objectStore));

        const license = licenseSerial ? [{
            serialNumber: licenseSerial,
            maxCapacity: licenseMaxCapacity || 10240,
            currentCapacity: licenseCapacity,
            package: 'fabricpool'
        }] : undefined;

        // change publicId to be dynamic
        const publicId = hyperid().uuid;
        const clusterToAdd = scenarioMapping.clone ? cloneDeep(clustersCollection[clusterId]) : clustersCollection[clusterId];
        if (scenarioMapping.clone) {
            clusterToAdd.clusterInfo.clusterName = `${clusterToAdd.clusterInfo.clusterName}_${random(0, scenarioMapping.clusters.length)}`;
        }
        const clusterUuid = get(clustersCollection, `${clusterId}.clusterInfo.clusterUuid`);
        if (clusterUuid && !['48247212-fa1e-442b-837b-fd257751143', '10bec0b2-7c82-404f-868a-424cf5158b80', '10bec0b3-6d82-454f-878a-434cf5158b90'].includes(clusterUuid)) {
            clusterToAdd.clusterInfo.clusterUuid = hyperid().uuid;
        }
        clusterToAdd.publicId = publicId;
        accumulator[publicId] = {
            cluster: clusterToAdd,
            volumes: volumesForCluster,
            volumeFootprints: volumeFootprintsFotCluster,
            svms: svmsForCluster,
            objectStoresConfig: objectStoreConfigForCluster,
            objectStores: objectStoresForCluster,
            aggregates: aggregatesForCluster,
            agentId: agent ? agent.agentId : undefined,
            workspaceId: workspace.workspacePublicId,
            license,
            resourceClass
        };
        return accumulator;
    }, {});

    const AGENTS = Object.values(ACCOUNTS).reduce((accumulator, accountObj) => {
        accumulator[accountObj.account.accountPublicId] = agentsObject;
        return accumulator;
    }, {});
    return {
        ACCOUNTS,
        CLUSTERS: clustersObject,
        AGENTS,
        SUBSCRIPTIONS: {},
        BUCKETS: (scenarioMapping.withNoBuckets) ? {
            AWS: [], AZURE: [], GCP: [], SGWS: []
        } : {
            AWS: awsBucketsCollection, AZURE: azureContainersCollection, GCP: gcpBucketsCollection, SGWS: sgwsBucketsCollection
        },
        STORAGE_ACCOUNTS: (scenarioMapping.withNoBuckets) ? [] : azureStorageAccountsCollection,
        RESOURCE_GROUPS: azureResourceGroupsCollection,
        BYOL_LICENSES: byolLicensesCollection
    };
}

function reset(key, userId) {
    if (key) {
        if (userId) {
            logger.info('DELETING key', key, 'For user', userId);
            Object.keys(USERS[userId][key]).forEach(subKey => delete USERS[userId][key][subKey]);
        } else {
            logger.info('DELETING key', key, 'For all users');
            Object.keys(USERS).forEach(user => {
                Object.keys(USERS[user][key]).forEach(subKey => delete USERS[user][key][subKey]);
            });
        }
    } else if (userId) {
        logger.info('DELETING user data from USERS_STATE');
        USERS[userId] = {};
    } else {
        logger.info('DELETING all data for all users');
        Object.keys(USERS).forEach(user => delete USERS[user]);
        SUBSCRIPTIONS.splice(0, SUBSCRIPTIONS.length);
    }
}

function loadScenario(scenarioName, trial, withNoBuckets, accountShouldFail, isGov) {
    reset();
    Object.assign(MAPPING, cloneDeep(mapping[scenarioName || 'default']));
    MAPPING.trial = (trial || MAPPING.trial);
    MAPPING.withNoBuckets = withNoBuckets;
    MAPPING.accountShouldFail = accountShouldFail;
    MAPPING.isGov = isGov;
}

function loadScenrioForUser(scenarioName, userId, accountId, trial, noBuckets, accountsNumber, accountShouldFail) {
    logger.info('Loading scenario', scenarioName, 'for user', userId, 'with noBuckets', noBuckets, 'accounts number', accountsNumber);
    try {
        const mappingForUser = cloneDeep(mapping[scenarioName || 'default']);
        mappingForUser.accountsNumber = accountsNumber || 1;
        if (trial) {
            mappingForUser.trial = trial;
        }
        if (noBuckets) {
            mappingForUser.withNoBuckets = noBuckets;
        }
        if (accountShouldFail) {
            mappingForUser.accountShouldFail = accountShouldFail;
        }
        reset(undefined, userId);
        USERS[userId] = createScenarioForUser(userId, accountId, mappingForUser);
    } catch (err) {
        logger.error('Failed to load scenario', err);
    }
}

// reset user to default state - using by simulator
function resetStateForUser(userId, accountId) {
    logger.info(`reset state for userId ${userId}`);
    USERS[userId] = createScenarioForUser(userId, accountId);
}

function overrideWorkspaceId(userId, accountId, workspaceId) {
    const workspaceObj = USERS[userId].ACCOUNTS[accountId].workspaces.find(workspace => workspace.override);
    // change all clusters workspaceId
    if (workspaceObj) {
        const oldWorkspaceId = workspaceObj.workspacePublicId;
        Object.values(USERS[userId].CLUSTERS).filter(cluster => cluster.workspaceId === oldWorkspaceId).forEach(cluster => set(cluster, 'workspaceId', workspaceId));
        workspaceObj.workspacePublicId = workspaceId;
        workspaceObj.override = false;
    }
}

function overrideAccountId(userId, accountId) {
    // assumption - always one account
    const account = Object.values(USERS[userId].ACCOUNTS)[0];
    if (account) {
        // assumption only one account
        const oldAccountId = account.account.accountPublicId;
        USERS[userId].AGENTS[accountId] = USERS[userId].AGENTS[oldAccountId];
        account.account.accountPublicId = accountId;
        account.account.override = false;
        USERS[userId].ACCOUNTS[accountId] = USERS[userId].ACCOUNTS[oldAccountId];
    }
}

module.exports = {
    overrideWorkspaceId,
    overrideAccountId,
    USERS,
    loadScenario,
    reset,
    loadScenrioForUser,
    resetStateForUser,
    createScenarioForUser,
    SUBSCRIPTIONS
};
