const got = require('got');
const {
    AGENT_GET_AWS_STORAGE_CLASS,
    AGENT_LIST_AWS_BUCKETS, AGENT_LIST_AWS_REGIONS, AGENT_ENDPOINT, AGENT_API_PREFIX, AGENT_API_PROVIDERS_PREFIX
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');
require('../scopes/agent-aws-scope');

describe('aws-agent-scope', () => {
    test('Get regions', async () => {
        const data = await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_AWS_REGIONS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });
        const expectedRegions = ['cn-north-1', 'sa-east-1', 'ap-southeast-1',
            'ap-south-1', 'eu-central-1', 'eu-west-1', 'us-west-1', 'us-east-2', 'us-east-1'];
        expect(data).toEqual(expectedRegions);
    });
    test('Create bucket and receive it back on get buckets', async () => {
        const name = 'test';
        const region = 'us-east-1';
        const bucket = { name, region };
        // create bucket
        await got.post(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_AWS_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: bucket,
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        const buckets = await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId/${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_AWS_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        // get bucket
        const foundBucket = buckets.find(bucketObj => bucketObj.name === name && bucketObj.region === region);
        expect(foundBucket.name).toBe(name);
        expect(foundBucket.region).toBe(region);
    });

    test('Create bucket, try to create the same bucket and fail', async () => {
        const bucket = { name: 'test5', region: 'us-east-1' };
        // create bucket
        await got.post(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_AWS_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: bucket,
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        try {
            await got.post(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_AWS_BUCKETS}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                json: bucket,
                headers: {
                    'X-Agent-Id': 'agentId',
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.body.message).toEqual(`A bucket with name ${bucket.name} already exist. please try again`);
        }
    });

    test('Create bucket,  change storage class', async () => {
        const bucket = { name: 'test6', region: 'us-east-1' };
        // create bucket
        await got.post(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_AWS_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: bucket,
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        // change storage class
        await got.put(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_GET_AWS_STORAGE_CLASS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { bucket: bucket.name, storageClass: 'INTELLIGENT_TIERING' },
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        // verify the storage class changed
        const storageClass = await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId/${AGENT_API_PROVIDERS_PREFIX}${AGENT_GET_AWS_STORAGE_CLASS}?bucket=${bucket.name}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        expect(storageClass).toStrictEqual({ storageClass: 'INTELLIGENT_TIERING' });
    });
});
