const got = require('got');
const {
    AGENT_SET_GCP_STORAGE_CLASS,
    AGENT_LIST_GCP_BUCKETS, AGENT_LIST_GCP_REGIONS, AGENT_ENDPOINT, AGENT_API_PREFIX, AGENT_API_PROVIDERS_PREFIX
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');
require('../scopes/agent-gcp-scope');

describe('gcp-agent-scope', () => {
    test('Get regions', async () => {
        const data = await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_REGIONS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });
        const expectedRegions = [
            'southeastasia', 'asia-east1', 'asia-east2', 'asia-northeast1', 'asia-northeast2', 'asia-northeast3', 'asia-south1', 'asia-southeast1', 'australia-southeast1', 'europe-north1', 'europe-west1', 'europe-west2', 'europe-west3', 'europe-west4', 'europe-west6', 'northamerica-northeast1', 'southamerica-east1', 'us-central1', 'us-east1', 'us-east4', 'us-west1', 'us-west2', 'us-west3'];
        expect(data).toEqual(expectedRegions);
    });
    test('Create bucket and receive it back on get buckets', async () => {
        const bucket = { name: 'test', region: 'asia-east1' };
        // create bucket
        await got.post(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: bucket,
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        const buckets = await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId/${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        // get bucket
        expect(buckets.find(bucketObj => bucketObj.name === bucket.name && bucketObj.region === bucket.region)).toBeDefined();
    });

    test('Create bucket, try to create the same bucket and fail', async () => {
        const bucket = { name: 'test1', region: 'asia-east1' };
        // create bucket
        await got.post(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: bucket,
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        try {
            await got.post(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_BUCKETS}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                json: bucket,
                headers: {
                    'X-Agent-Id': 'agentId',
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.body.message).toEqual(`A bucket with name ${bucket.name} already exist. please try again`);
        }
    });
    test('Create bucket,  change storage class', async () => {
        const bucket = { name: 'test2', region: 'asia-east1' };
        // create bucket
        await got.post(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: bucket,
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        // change storage class
        await got.put(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_SET_GCP_STORAGE_CLASS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { bucket: bucket.name, storageClass: 'archive' },
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        // verify the storage class changed
        const buckets = await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId/${AGENT_API_PROVIDERS_PREFIX}${AGENT_LIST_GCP_BUCKETS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-Agent-Id': 'agentId',
                authorization: signJwtToken()
            }
        });

        const requestedBucket = buckets.find(bucketObj => bucketObj.name === bucket.name && bucketObj.region === bucket.region);
        // get bucket
        expect(requestedBucket.storageClass).toBe('archive');
    });
});
