const got = require('got');
const { signJwtToken } = require('../simulator-utils');
const { reset } = require('../simulator-state');
const {
    AGENTS_MGMT_ENDPOINT, API_PATH_AGENTS, AGENT_ACTIVE_STATUS
} = require('../../../src/utils/consts');
const { createAgent } = require('../../testUtils');
require('../scopes/agents-scope');

const ACCOUNT_1_NAME = 'account-1';
const ACCOUNT_2_NAME = 'account-2';

const agent = createAgent('agent1', AGENT_ACTIVE_STATUS);

const agentToCreate2 = createAgent('agent2', AGENT_ACTIVE_STATUS);

describe('agents mgmt scope', () => {
    beforeEach(() => reset());

    test('agents - create single', async () => {
        const data = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        expect(data.name).toEqual(agent.name);
        expect(data.status).toEqual(agent.status);
    });

    test('agents - get single', async () => {
        const postResponseData = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const getResponseData = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect({ agents: [postResponseData] }).toEqual(getResponseData);
    });

    test('agents - get 2 single agents different user', async () => {
        const postAgentUser1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const getAgentUser1 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const postAgentUser2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const getAgentUser2 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        expect({ agents: [postAgentUser1] }).toEqual(getAgentUser1);
        expect({ agents: [postAgentUser2] }).toEqual(getAgentUser2);

        expect(getAgentUser1.agents).toHaveLength(1);
        expect(getAgentUser2.agents).toHaveLength(1);
    });

    test('agents - get non existent', async () => {
        await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const data = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_2_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(data).toEqual({ agents: [] });
    });

    test('agents - create an agent with one user try to get it with other user - should fail', async () => {
        try {
            await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
                json: { accountId: ACCOUNT_1_NAME, agent },
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
            await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_2_NAME}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken('user2'),
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.body.statusCode).toBe(403);
        }
    });

    test('agents - create multiple agents in same workspace', async () => {
        const agent1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agent2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agents = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(agents).toEqual({ agents: [agent1, agent2] });
    });

    test('agents - create multiple agents in same workspace with different users - make sure each user recives only his agents', async () => {
        const agent1User1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agent2User1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',

            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agentsUser1 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agent1User2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent: agentToCreate2 },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const agent2User2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent: agentToCreate2 },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const agentsUser2 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });

        expect(agentsUser1).toEqual({ agents: [agent1User1, agent2User1] });
        expect(agentsUser2).toEqual({ agents: [agent1User2, agent2User2] });
    });

    test('agents - create multiple agents in different accounts', async () => {
        const agent1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { accountId: ACCOUNT_1_NAME, agent },
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agent2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { accountId: ACCOUNT_2_NAME, agent },
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agentsAccount1 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agentsAccount2 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_2_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(agentsAccount1).toEqual({ agents: [agent1] });
        expect(agentsAccount2).toEqual({ agents: [agent2] });
    });

    test('agents - create multiple agents in different accounts and different users', async () => {
        const agent1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { accountId: ACCOUNT_1_NAME, agent },
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agent2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { accountId: ACCOUNT_2_NAME, agent },
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agentsAccount1 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agentsAccount2 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_2_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const agent1User2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { accountId: 'account_3', agent },
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const agent2User2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { accountId: 'account_4', agent },
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const agentsAccount1User2 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=account_3`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const agentsAccount2User2 = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=account_4`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });

        expect(agentsAccount1).toEqual({ agents: [agent1] });
        expect(agentsAccount2).toEqual({ agents: [agent2] });

        expect(agentsAccount1User2).toEqual({ agents: [agent1User2] });
        expect(agentsAccount2User2).toEqual({ agents: [agent2User2] });
    });

    test('get agent status- existing agent in the account', async () => {
        const agent1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { agent: agentsAccount1 } = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agent1.agentId}?account_id=${ACCOUNT_1_NAME}`,
            {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });

        expect(agentsAccount1.status).toEqual(AGENT_ACTIVE_STATUS);
    });

    test('get agent status- existing agent in the account with multiple users', async () => {
        const agent1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { agent: agentsAccount1 } = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agent1.agentId}?account_id=${ACCOUNT_1_NAME}`,
            {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });

        const agent1User2 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: { accountId: ACCOUNT_1_NAME, agent },
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { agent: agentsAccount1User2 } = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agent1User2.agentId}?account_id=${ACCOUNT_1_NAME}`,
            {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken('user2'),
                    authorization: signJwtToken('user2')
                }
            });
        expect(agentsAccount1.status).toEqual(AGENT_ACTIVE_STATUS);
        expect(agentsAccount1User2.status).toEqual(AGENT_ACTIVE_STATUS);
    });

    test('get agent status, non existing agent in an account', async () => {
        try {
            await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/agentId?account_id=${ACCOUNT_1_NAME}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toBe(404);
        }
    });

    test('get agent status, non existing agent in an account for user', async () => {
        try {
            await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
                json: { accountId: ACCOUNT_1_NAME, agent },
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });

            await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/agentId?account_id=${ACCOUNT_1_NAME}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken('user2'),
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.body).toBe('User was not found');
        }
    });

    test('change agent status', async () => {
        // #1 create an agent
        // #2 change status
        // #3 get agent make sure the status changed
        const agent1 = await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, {
            json: { accountId: ACCOUNT_1_NAME, agent },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        await got.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agent1.agentId}/change-status?account_id=${ACCOUNT_1_NAME}`, {
            json: { status: 'inactive' },
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const { agent: agentsAccount1 } = await got.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agent1.agentId}?account_id=${ACCOUNT_1_NAME}`,
            {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        expect(agentsAccount1.status).toEqual('inactive');
    });
});
