const got = require('got');
const { cloneDeep } = require('lodash');
const { signJwtToken } = require('../simulator-utils');
const { reset } = require('../simulator-state');
const {
    SUBSCRIPTION_BASE_URL, MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS,
    SUBSCRIBED, UNSUBSCRIBED, TIERING_SERVICE_NAME_SUBSCRIPTION
} = require('../../../src/utils/consts');
require('../scopes/subscription-service-scope');
require('../scopes/cloud-tiering-scope');
require('../scopes/multi-tenancy-scope');

const ACCOUNT_1_ID = 'account-1';

const subscription = {
    subscriptionId: 'customerIdentifier1',
    cloudProvider: 'aws',
    name: 'subscription',
    subscriptionStatus: SUBSCRIBED,
    serviceName: TIERING_SERVICE_NAME_SUBSCRIPTION
};

const subscription2 = {
    subscriptionId: 'customerIdentifier2',
    cloudProvider: 'aws',
    subscriptionStatus: UNSUBSCRIBED,
    name: 'subscription2',
    serviceName: TIERING_SERVICE_NAME_SUBSCRIPTION
};

describe('subscription service scope', () => {
    beforeEach(() => reset());

    test('subscription - create subscription', async () => {
        // create account
        const account = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountId } = account;
        subscription.accounts = [accountId];
        // create subscription
        const data = await got.post(`${SUBSCRIPTION_BASE_URL}/${accountId}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: subscription,
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data).toMatchObject(subscription);
    });

    test('subscription - list subscriptions', async () => {
        // create account
        const account = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountId } = account;

        // create subscription
        const subscriptionCreated = await got.post(`${SUBSCRIPTION_BASE_URL}/${accountId}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: subscription,
            headers: {
                authorization: signJwtToken(),
                'X-User-Token': signJwtToken()
            }
        });

        const listSubscriptionsResponse = await got.get(`${SUBSCRIPTION_BASE_URL}/${accountId}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(listSubscriptionsResponse).toEqual([subscriptionCreated]);
    });

    test('subscription - get empty array of subscriptions', async () => {
        // create account
        const account = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const { accountPublicId: accountId } = account;

        const listSubscriptionsResponse = await got.get(`${SUBSCRIPTION_BASE_URL}/${accountId}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                authorization: signJwtToken(),
                'X-User-Token': signJwtToken()
            }
        });

        expect(listSubscriptionsResponse).toEqual([]);
    });

    test('subscription - create multiple subscriptions in same account', async () => {
        // create account
        const account = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountId } = account;

        // create subscription
        const subscriptionCreated = await got.post(`${SUBSCRIPTION_BASE_URL}/${accountId}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: subscription,
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const subscription2Created = await got.post(`${SUBSCRIPTION_BASE_URL}/${accountId}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: subscription2,
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const listSubscriptionsResponse = await got.get(`${SUBSCRIPTION_BASE_URL}/${accountId}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(listSubscriptionsResponse).toEqual([subscriptionCreated, subscription2Created]);
    });

    test('subscription - get subscription of non existent account', async () => {
        try {
            await got.get(`${SUBSCRIPTION_BASE_URL}/${ACCOUNT_1_ID}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toBe(404);
        }
    });

    test('subscriptions- create a subscription with one user try to get it with other user - should an empty array', async () => {
        // create account user1
        const accountUser1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountIdUser1 } = accountUser1;

        // create subscription
        await got.post(`${SUBSCRIPTION_BASE_URL}/${accountIdUser1}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: subscription,
            headers: {
                authorization: signJwtToken()
            }
        });

        try {
            await got.get(`${SUBSCRIPTION_BASE_URL}/${accountIdUser1}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken('user2'),
                    'X-User-Token': signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toEqual(404);
        }
    });

    test('subscriptions- create a subscription to an account that doesnt exist- should fail', async () => {
        // create subscription
        try {
            await got.post(`${SUBSCRIPTION_BASE_URL}/account1`, {
                json: subscription,
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toEqual(404);
        }
    });

    test('subscriptions- create a subscription for different users - verify all are returned', async () => {
        // create account user1
        const accountUser1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountIdUser1 } = accountUser1;

        // create subscription
        await got.post(`${SUBSCRIPTION_BASE_URL}/${accountIdUser1}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: subscription,
            headers: {
                authorization: signJwtToken()
            }
        });

        // create account user2
        const accountUser2 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { accountPublicId: accountIdUser2 } = accountUser2;

        // create subscription
        await got.post(`${SUBSCRIPTION_BASE_URL}/${accountIdUser2}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: subscription2,
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        const subscriptions = await got.get(`${SUBSCRIPTION_BASE_URL}/by-service-name?serviceName=${TIERING_SERVICE_NAME_SUBSCRIPTION}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                authorization: signJwtToken()
            }
        });

        const subscription1WithAccounts = cloneDeep(subscription);
        subscription1WithAccounts.accounts = [accountIdUser1];
        const subscription2WithAccounts = cloneDeep(subscription2);
        subscription2WithAccounts.accounts = [accountIdUser2];
        expect(subscriptions).toEqual({ items: [subscription1WithAccounts, subscription2WithAccounts] });
    });
});
