import React, {useState, useEffect} from 'react'
import _ from 'lodash';
import {extractError, sendMixPanelEvent} from '../../utils/utils';
import {CSSTransition} from 'react-transition-group';
import classNames from 'classnames';
import Isvg from 'react-inlinesvg';
import { MODIFY_AGGREGATES } from '../../graphql/queries';
import InfoPopover from '../reactPopover/InfoPopover';
import {
    FEATURE_STATE,
    MAX_TIERING_FULLNESS_THRESHOLD,
    MIN_TIERING_FULLNESS_THRESHOLD,
    NOTIFICATION_TYPES,
    TIERING_FULLNESS_THRESHOLD_INFO,
    TIERING_IDR_INFO
} from '../../utils/consts';
import CloudSlider from '../widgets/CloudSlider';
import NotificationWidget from '../widgets/NotificationWidget';
import SwitchOnOff from '../widgets/SwitchOnOff';
import ButtonWithError from '../widgets/ButtonWithError';
import NoticeMessage from "../widgets/NoticeMessage";
import {connect} from "react-redux";
import { useMutation } from '@apollo/client'

require("./editAggregatePanel.scss");

const handleError = (action, error, setError) => {
    const extractedError = extractError(error);
    sendMixPanelEvent(`${action}-Error`, extractedError);
    setError(extractedError);
};

const EditAggregatePanel = ({isOpen, close, editAggregate, clusterId, accountId, agentId, workspaceId}) => {
    const [firstTimeInfoSet, setFirstTimeInfoSet] = useState(false);
    const [originalAggregate, setOriginalAggregate] = useState(null);
    const [idr, setIdr] = useState(null);
    const [fullnessThreshold, setFullnessThreshold] = useState(null);
    const [aggregate, setAggregate] = useState(null);
    const [error, setError] = useState(null);

    const [modifyAggregates, { loading }] = useMutation(MODIFY_AGGREGATES, {
        onError: (error) => handleError('EditAggregate', error, setError),
        errorPolicy: 'all'
    })

    useEffect(() => {
        const fields = ['name', 'idr', 'tieringFullnessThreshold'];
        if (isOpen && !firstTimeInfoSet) {
            setOriginalAggregate(_.pick(editAggregate, fields));
            setFirstTimeInfoSet(true);
            setIdr(editAggregate.idr);
            setFullnessThreshold(editAggregate.tieringFullnessThreshold || editAggregate.supportedSettings.tieringFullnessThreshold);
            setAggregate(editAggregate);
        }
        if (!isOpen && firstTimeInfoSet) { //panel is closing clear the inner state
            setFirstTimeInfoSet(false);
            setOriginalAggregate(null);
            setIdr(null);
            setFullnessThreshold(null);
            setError(null);
            setAggregate(null);
        }
    }, [editAggregate, isOpen, firstTimeInfoSet]);

    const modifyComplete = (store, {data: {modifyAggregates: {data, errors}}}) => {
        if (errors && errors.length > 0) {
            setError(errors[0].reason);
        }
        if (!errors || errors.length === 0) { //close only if there are no errors at all.
            close();
        }
    };

    const submit = (modifyAggregates) => {
        const variables = {
            clusterId,
            aggregatesProperties: [{
                name: aggregate.name
            }],
            accountId,
            agentId,
            workspaceId
        };
        if (fullnessThreshold && !_.isEqual(fullnessThreshold, originalAggregate.tieringFullnessThreshold)) {
            variables.aggregatesProperties[0].tieringFullnessThreshold = fullnessThreshold;
        }
        if (idr && !_.isEqual(idr, originalAggregate.idr)) {
            variables.aggregatesProperties[0].idr = idr;
        }
        sendMixPanelEvent('ModifyAggregate');
        setError(null);
        return modifyAggregates({
            variables,
            update: modifyComplete
        });
    };

    const somethingChanged = () => {
        if ((!_.isEqual(idr, originalAggregate.idr)) || (!_.isEqual(fullnessThreshold, originalAggregate.tieringFullnessThreshold))) {
            return true; //something changed.
        }
        return false; //nothing changed
    };


    if (!aggregate || !originalAggregate) {
        return (<CSSTransition in={isOpen} classNames="slide-from-right-edit-aggregate" timeout={400} unmountOnExit>
            <div className="slide-from-right-edit-aggregate-parent">
                <div className={classNames('edit-aggregate-panel')}>
                </div>
            </div>
        </CSSTransition>);
    }

    const showFullnessThresholdNotification = aggregate.supportedSettings.tieringFullnessThresholdState !== FEATURE_STATE.EDITABLE;
    const showIdrNotification = aggregate.supportedSettings.idrState !== FEATURE_STATE.EDITABLE;
    return (
        <CSSTransition in={isOpen} classNames="slide-from-right-edit-aggregate" timeout={400} unmountOnExit>
            <div className="slide-from-right-edit-aggregate-parent">
                <div className={classNames('edit-aggregate-panel')}>
                    <div className="close-edit-aggregate-panel" onClick={close}><Isvg
                        src={require('../../assets/svgs/back-arrow.svg')}/></div>
                    <div className="name-part">
                        <div className="name no-wrap" title={aggregate.name}>{aggregate.name}</div>
                    </div>
                    <>
                        <div className="panel-content">
                            <div className="fullness-threshold-part box">
                                <div className="title">
                                    <InfoPopover wrapperClassName="fullness-threshold-info"
                                                 popoverClassName="fullness-threshold-info"
                                                 popoverText={TIERING_FULLNESS_THRESHOLD_INFO}
                                                 contentStyle={{ width: '460px' }}/>
                                    <span>Start tiering data when the aggregate reaches a specific used capacity percentage</span>
                                </div>
                                {!showFullnessThresholdNotification &&
                                <div className="fullness-threshold-slider">
                                    <div
                                        className="slider-min slider-value">{MIN_TIERING_FULLNESS_THRESHOLD}%
                                    </div>
                                    <div className="slider-comp">
                                        <CloudSlider
                                            min={MIN_TIERING_FULLNESS_THRESHOLD}
                                            max={MAX_TIERING_FULLNESS_THRESHOLD}
                                            valueString={(valueNow) => {
                                                return `${valueNow}%`;
                                            }}
                                            updateChange={(value) => {
                                                setFullnessThreshold(value);
                                                setError(null);
                                            }}
                                            value={fullnessThreshold}
                                        />
                                    </div>
                                    <div
                                        className="slider-max slider-value">{MAX_TIERING_FULLNESS_THRESHOLD}%
                                    </div>
                                </div>}
                                {showFullnessThresholdNotification &&
                                <NoticeMessage className='fullness-threshold'>
                                    {aggregate.supportedSettings.noTieringFullnessThreshold}
                                </NoticeMessage>}
                            </div>
                            <div className="idr-part box">
                                <div className="title-with-switch">
                                    <InfoPopover wrapperClassName="idr-info" popoverClassName="idr-info"
                                                 popoverText={TIERING_IDR_INFO}
                                                 contentStyle={{ width: '460px' }}/>
                                    <div className="title">Activate inactive data reporting (IDR)</div>
                                    <SwitchOnOff onByDefault={idr && idr === 'ON'}
                                                 disabled={aggregate.supportedSettings.idrState === FEATURE_STATE.READONLY || aggregate.supportedSettings.idrState === FEATURE_STATE.NOT_SUPPORTED}
                                                 updateState={(newState) => setIdr(newState ? 'ON' : 'OFF')}/>
                                </div>
                                {idr === 'OFF' && !showIdrNotification &&
                                <NoticeMessage className="idr">Activate inactive data reporting (IDR) to
                                    identify your inactive data and savings opportunities.</NoticeMessage>}
                                {showIdrNotification && <NoticeMessage
                                    className="idr">{aggregate.supportedSettings.noIdrReason}</NoticeMessage>}
                            </div>
                        </div>
                        {error && <NotificationWidget className="error-modify-aggr"
                                                      variant={NOTIFICATION_TYPES.ERROR}>{error}</NotificationWidget>}
                        <ButtonWithError submitting={loading} disable={!somethingChanged()}
                                         type="button" onClick={() => submit(modifyAggregates)}
                                         title={!somethingChanged() ? 'No change detected' : ''}>
                            APPLY
                        </ButtonWithError>
                    </>
                </div>
            </div>
        </CSSTransition>
    );
};

const mapStateToProps = state => {
    return {
        agentId: state.general.agentId,
        accountId: state.general.accountId,
        workspaceId: state.general.workspaceId,
    };
};

export default connect(mapStateToProps)(EditAggregatePanel);
