import React, {useCallback, useMemo, useState} from 'react';
import {Field, reduxForm} from 'redux-form';
import mapStateToProps from '../wizard/wizardMapStateToProps';
import {connect} from 'react-redux';
import stepinize from '../wizard/stepinize';
import classNames from 'classnames';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import SetupWizardDiagram from './SetupWizardDiagram';
import Validator from 'validatorjs';
import {extractError, sendMixPanelEvent} from '../../utils/utils';
import {ControlledSelectField, InputField} from '../forms/forms';
import localStorage from '../../utils/localStorage';
import { GET_IP_SPACES_QUERY, SETUP_CLUSTER_G2 } from '../../graphql/queries';
import LoadingIcon from '../widgets/LoadingIcon';
import {useMutation, useQuery} from '@apollo/client'
import InternalWizardHeader from '../wizard/InternalWizardHeader';
import {PROVIDER_TYPE_AWS, PROVIDER_TYPE_AZURE, PROVIDER_TYPE_GCP, PROVIDER_TYPE_SGWS} from '../../utils/consts';
import {decodeAll} from '../../utils/encoders';
import Button from "../widgets/Button";
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy'
import {get} from 'lodash'

require('./setupIpSpaceStep.scss');

const rules = {
    ipSpace: 'required'
};


const validate = (values) => {
    let errors = {};
    const validator = new Validator(values, rules);
    validator.setAttributeNames({ipSpace: 'IPSpace'});
    validator.passes();
    errors = validator.errors.all();
    return errors;
};

const useIpSpace = ({wizard, change, history, accountId, agentId, workspaceId}) => {
    const clusterId = get(wizard.selectedWizardParams, ['clusterId'], null);
    const clusterName = get(wizard.selectedWizardParams, ['clusterName'], null);
    const provider = get(wizard.selectedWizardParams, ['provider'], null);
    const [queryError, setQueryError] = useState('')

    const handleError = useCallback((error) => {
        if (error) {
            const errorMessage = extractError(error, 'IP Space Step');
            sendMixPanelEvent('SetupCluster-Error', errorMessage);
            setQueryError(errorMessage)
        }
    }, [])

    const {loading, data} = useQuery(GET_IP_SPACES_QUERY, {
        variables: {
            clusterId: clusterId,
            accountId,
            agentId,
            workspaceId
        },
        skip: !clusterId,
        fetchPolicy: 'network-only',
        onError: handleError
    });

    const ipSpaceOptions = useMemo(() => data?.ipSpaces.map((ipSpace) => ({
        value: ipSpace,
        label: ipSpace
    })), [data])

    useRunOnceWhenTruthy(() => {
        if (ipSpaceOptions) {
            const selectedOption = ipSpaceOptions[0]
            if (ipSpaceOptions.length > 1) {
                change('ipSpace', selectedOption)
            }
            if (ipSpaceOptions.length === 1) {
                change('ipSpaceInput', selectedOption.value)
            }
        }
    }, ipSpaceOptions)

    const nextStep = useCallback(() => {
        const {wizardType} = wizard;
        sendMixPanelEvent('ContinueClusterNetwork');
        history.push(`/${wizardType}/${clusterId}/${clusterName}/setup-complete${history.location.search}`);
    }, [wizard, history, clusterId, clusterName])


    const [setupClusterMutation, {loading: loadingSetupCluster}] = useMutation(SETUP_CLUSTER_G2,
        {
            onCompleted: nextStep,
            onError: handleError
        })

    const setupCluster = useCallback(({ipSpace: ipSpaceOption, ipSpaceInput}) => {
        const ipSpace = ipSpaceOption?.value || ipSpaceInput

        const keys = localStorage.get('selectedKeys');
        const region = get(wizard.selectedWizardParams, ['region'], undefined);
        const storageAccount = get(wizard.selectedWizardParams, ['storageAccount'], undefined);
        const resourceGroup = get(wizard.selectedWizardParams, ['resourceGroup'], undefined);
        const bucketPath = provider === 'AZURE' ? 'containerName' : 'bucketName';
        const bucketName = get(wizard.selectedWizardParams, [bucketPath], undefined);
        let host,
            port = null;
        if (provider === PROVIDER_TYPE_SGWS.name) {
            host = decodeAll(get(wizard.selectedWizardParams, 'sg', undefined));
            port = parseInt(get(wizard.selectedWizardParams, 'port', undefined));
        }

        const variables = {cloudProvider: provider};
        switch (provider) {
            case PROVIDER_TYPE_AWS.name :
            case PROVIDER_TYPE_GCP.name : {
                variables[provider.toLowerCase()] = {
                    accountId,
                    agentId,
                    workspaceId,
                    clusterId,
                    ipSpace,
                    bucket: bucketName,
                    region,
                    credentials: keys
                };
                break;
            }
            case PROVIDER_TYPE_AZURE.name : {
                variables.azure = {
                    accountId,
                    agentId,
                    workspaceId,
                    clusterId,
                    ipSpace,
                    bucket: bucketName,
                    region,
                    azureStorageAccount: storageAccount,
                    azureResourceGroup: resourceGroup
                };
                break;
            }
            case PROVIDER_TYPE_SGWS.name : {
                variables.sgws = {
                    accountId,
                    agentId,
                    workspaceId,
                    clusterId,
                    ipSpace,
                    bucket: bucketName,
                    port: port ? port : undefined,
                    host,
                    credentials: keys
                };
                break;
            }
            default: {
                console.log(`%c Error unknown provider in setupCluster ${provider}`, 'font-weight:bold; color:red');
            }
        }
        sendMixPanelEvent('SetupCluster');
        return setupClusterMutation({variables});
    }, [wizard, clusterId, provider, setupClusterMutation, accountId, agentId, workspaceId])

    return {
        loading,
        data,
        error: queryError,
        ipSpaceOptions,
        setupCluster,
        loadingSetupCluster
    }
}


const SetupIpSpaceStep = (props) => {
    const {loading, error, ipSpaceOptions, setupCluster, loadingSetupCluster} = useIpSpace(props)
    const {handleSubmit} = props;

    const getIPSpaceUX = useCallback(() => {
        const numOfOptions = ipSpaceOptions.length;
        if (numOfOptions > 1) {
            return (
                <Field component={ControlledSelectField} label="IPSpace"
                       name="ipSpace" options={ipSpaceOptions}
                       className="ip-space"></Field>
            );
        } else {
            return (
                <div>
                    <Field name="ipSpaceInput" component={InputField} label="IPSpace"
                           type="text" disabled={true}></Field>
                    {numOfOptions === 1 && <div className="ipspace-info">Only one suitable IPSpace was found</div>}
                </div>
            );
        }

    }, [ipSpaceOptions])

    return (
        <div className={classNames('setup-ip-spaces-step new-wizard-step')}>
            <div className="panels-container">
                <div className="left-panel">
                    <div className="pane center">
                        <h1>Tiering Setup</h1>
                        <InternalWizardHeader {...props}/>
                        <div className="step-title-info">Select the IPSpace on the cluster that you want to use to
                            connect to your object storage provider.
                        </div>
                    </div>
                    <form onSubmit={handleSubmit(setupCluster)}>
                        {(loading || !ipSpaceOptions) && !error &&
                        <LoadingIcon text={'IPSpaces'} className="loading-ip-spaces"/>}
                        {ipSpaceOptions && <div className="pane center">
                            {getIPSpaceUX()}
                        </div>
                        }
                        <div className="wizard-button">
                            <ButtonsPanelWithError error={error}
                                                   submitting={loadingSetupCluster}>
                                <Button type="submit" disabled={loadingSetupCluster}>Continue</Button>
                            </ButtonsPanelWithError>
                        </div>

                    </form>
                    )
                </div>
                <SetupWizardDiagram flowStep="cluster-discovered"/>
            </div>
        </div>
    );
}

let IpSpaceStepForm = reduxForm({
    form: 'setupIpSpaceForm',
    validate
})(SetupIpSpaceStep);

export default connect(mapStateToProps)(stepinize(IpSpaceStepForm, {
    name: () => `Setup IP Space`
}));

