import React, { useState, useMemo, useCallback } from 'react';
import { calculateNextStep, getNextStepComponent } from '../wizard/WizardStarter';
import { Route, Switch } from 'react-router';
import { Field, formValueSelector, reduxForm } from 'redux-form';
import mapStateToProps from '../wizard/wizardMapStateToProps';
import { connect } from 'react-redux';
import stepinize from '../wizard/stepinize';
import classNames from 'classnames';
import SetupWizardDiagram from './SetupWizardDiagram';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import { CHANGE_TIER_LEVEL_G } from '../../graphql/queries';
import { DEFAULT_STORAGE_CLASS, STORAGE_CLASSES, TIERING_SETUP_TITLES } from '../../utils/consts';
import _ from 'lodash';
import { extractError, sendMixPanelEvent } from '../../utils/utils';
import { ControlledSelectField } from '../forms/forms';
import InternalWizardHeader from '../wizard/InternalWizardHeader';
import Button from '../widgets/Button';
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy'
import { useMutation } from '@apollo/client'

require('./setupStorageClassStep.scss');

const handleError = (error, action, setError) => {
    const errorMessage = extractError(error, 'Storage Class Step');
    sendMixPanelEvent(action, errorMessage);
    setError(errorMessage)
}

const SetupStorageClassStep = (props) => {
    const { wizard, history, accountId, agentId, handleSubmit, change, storageClassFromForm } = props
    const provider = _.get(wizard.selectedWizardParams, ['provider'], null);
    const storageClass = provider ? _.get(wizard.selectedWizardParams, ['storageClass'], STORAGE_CLASSES[provider][DEFAULT_STORAGE_CLASS].storageClass) : null;
    const bucketName = _.get(wizard.selectedWizardParams, ['bucketName'], undefined);
    const [error, setError] = useState(null)

    const storageClassOptions = useMemo(() => _.map(STORAGE_CLASSES[provider], storageClass => {
        return {
            label: storageClass.storageClassName,
            value: storageClass.storageClass
        };
    }), [provider]);

    useRunOnceWhenTruthy(() => {
        if (storageClass) {
            const selectedStorageClass = _.find(storageClassOptions, { value: storageClass }) || '';
            if (selectedStorageClass) {
                change('storageClass', selectedStorageClass)
            }
        }

    }, storageClassOptions)


    const nextStep = () => {
        const nextStep = calculateNextStep(wizard);
        sendMixPanelEvent('ContinueStorageClass');
        wizard.nextStep(`/${storageClassFromForm.value}/${nextStep}`, history);
    }

    const [configureBucketMutation, { loading: loadingChangeTierLevel }] = useMutation(CHANGE_TIER_LEVEL_G, {
        onCompleted: nextStep,
        onError: (error) => handleError(error, 'ConfigureStorageClass-Error', setError),
        errorPolicy: 'all'
    })

    const configureBucket = useCallback((values) => {
        const {storageClass} = values
        sendMixPanelEvent('ConfigureStorageClass');
        return configureBucketMutation({
            variables: {
                cloudProvider: provider,
                [provider.toLowerCase()]: {
                    accountId,
                    agentId,
                    bucket: bucketName,
                    level: storageClass.value
                }
            }
        });
    }, [accountId, agentId, bucketName, configureBucketMutation, provider])

    const getContentUX = (provider) => {
        switch (provider) {
            case 'AZURE' : {
                return (
                    <div className="pane center">
                        <div className="storage-class-container">
                            <label>Access Tier</label>
                            <Field component={ControlledSelectField}
                                   name="storageClass"
                                   options={storageClassOptions}
                                   className="storage-class"></Field>
                        </div>
                    </div>

                );
            }
            case 'AWS':
            case 'GCP': {
                return (
                    <>
                        <div className="pane center">
                            <div>
                                <div className="storage-class-container">
                                    <label>Storage Class</label>
                                    <Field component={ControlledSelectField}
                                           name="storageClass"
                                           options={storageClassOptions}
                                           className="storage-class" noSearch={true}></Field>
                                </div>
                            </div>
                        </div>
                    </>
                );
            }
            default : {
                return <div>Unknown Provider {provider}</div>;
            }
        }
    }

    if (!provider) {
        return <div></div>;
    }

    return (
        <div className={classNames('setup-storage-class-step new-wizard-step')}>
            <div className="panels-container">
                <div className="left-panel">
                    <div className="pane center">
                        <h1>Tiering Setup</h1>
                        <InternalWizardHeader {...props}/>
                        {TIERING_SETUP_TITLES[provider].title2}
                    </div>
                    <form onSubmit={handleSubmit(configureBucket)}>
                        <div>
                            <div className="step-content">
                                {getContentUX(provider)}
                            </div>
                            <div className="wizard-button">
                                <ButtonsPanelWithError error={error} submitting={loadingChangeTierLevel}>
                                    <Button type="submit" disabled={loadingChangeTierLevel}>Continue</Button>
                                </ButtonsPanelWithError>
                            </div>
                        </div>
                    </form>
                </div>
                <SetupWizardDiagram flowStep="cluster-discovered"/>
            </div>
        </div>
    );
}

const nextRoutes = ({ match, wizard }) => {
    const nextStep = calculateNextStep(wizard, 'Setup Storage Class');
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:storageClass/${nextStep}`} render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    );
};

let StorageClassStepForm = reduxForm({
    form: 'setupStorageClassForm'
})(SetupStorageClassStep);

const selector = formValueSelector('setupStorageClassForm');
const combineStateObjects = (state, ownProps) => {
    const storageClass = selector(state, 'storageClass');
    return Object.assign({}, mapStateToProps(state, ownProps), {
        storageClassFromForm: storageClass
    });
};
export default connect(combineStateObjects)(stepinize(StorageClassStepForm, {
    name: () => `Setup Storage Class`,
    nextRoutes
}));

