import React, {useMemo, useState} from 'react';
import {GET_LAZY_CLUSTERS_QUERY, GET_LICENSE_INFO_QUERY} from '../../../graphql/queries';
import LoadingIcon from '../../widgets/LoadingIcon';
import Isvg from 'react-inlinesvg';
import {
    buildCapacityObject,
    getClusterLicenseInfo,
    getClusterNameTooltip,
} from '../../../utils/utils';
import {useQuery} from '@apollo/client'
import classNames from 'classnames';
import _ from 'lodash';
import SearchableSelectableTable from '../../table/SearchableSelectableTable';
import {ERROR_SERVER_ERROR, GENERAL_ERROR, routesObject} from '../../../actions';
import {
    BYOL,
    CLUSTER_STATUS,
    DASHBOARD_FILTER_TYPE,
    LICENSE_PAGE_INFO,
    PAYGO,
    PROVIDERS,
    SUBSCRIBED
} from '../../../utils/consts';
import {openDialog} from '../../dialog/Dialog';
import SubscriptionFlowDialog from './SubscriptionFlowDialog';
import ReactPopover from './../../reactPopover/ReactPopover';
import {connect} from 'react-redux';
import Button from "../../widgets/Button";
import ActivateByolDialog from "./ActivateByolDialog";
import useQueryParam from "../../../hooks/useQueryParam";
import useRunOnceWhenTruthy from "../../../hooks/useRunOnceWhenTruthy";
import SkeletonLicenseRow from "../../skeletons/SkelatonLicenseRow";
import {checkShouldEnforce} from "../../../utils/enforcmentUtil";

require('./licensePage.scss');

const LicenseCard = ({type, subscribed, associatedClusters}) => {
    return (
        <div className={classNames('license-card', type.toLowerCase())}>
            <div className="text">
                <div className="title">{`${type} Marketplace`}</div>
            </div>
            <div className="button-div">
                <div className="associated-clusters">
                    <span className="number">{associatedClusters}</span>
                    <span className="associated-cluster">Clusters associated</span>
                </div>
                {!subscribed && <Button onClick={(clickElement) => openDialog(<SubscriptionFlowDialog
                    className="confirm subscription-flow-dialog"
                    provider={type}/>, clickElement.target)}
                                        disabled={associatedClusters === 0}>Subscribe</Button>}
                {subscribed && <div className="subscribed">
                    <Isvg src={require('../../../assets/svgs/success_circle.svg')}/>
                    <span>Subscribed</span>
                </div>}
            </div>
        </div>
    );
};

const LicenseTitle = ({title, children, styleClass}) => {
    return (
        <div className={`license-title ${styleClass}`}>{title}{children}</div>
    );
};

const useHandleOpenDialogOnInit = (clusters) => {
    const open = useQueryParam('open')

    useRunOnceWhenTruthy(() => {
        const [licenseType, payload] = open.split(':')

        switch (licenseType) {
            case 'byol':
                const cluster = clusters.find(({clusterId}) => clusterId === payload)
                openDialog(<ActivateByolDialog clusterInfo={cluster}/>, null);
                break;
            case 'paygo':
                openDialog(<SubscriptionFlowDialog
                    className="confirm subscription-flow-dialog"
                    provider={payload}/>)
                break;
            default:
                break;
        }
    }, open)
};

const LicensePage = ({clusters, licenseInfo}) => {
    const [filter, setFilter] = useState(null)
    const [mappedClusters, providersCount] = useMemo(() => {
        const providersCount = {
            AWS: 0,
            GCP: 0,
            AZURE: 0
        }
        const mappedCluster = _.map(clusters, cluster => {
            providersCount[cluster.licenseInfo?.provider] !== undefined && ++providersCount[cluster.licenseInfo?.provider]
            const status = _.get(cluster, 'status.status', null);
            const tieredCapacity = buildCapacityObject(_.get(cluster, 'tierInformation.currentSavings.capacity', 0));
            const tieredCapacityString = `${tieredCapacity.value} ${tieredCapacity.unit}`;
            const provider = _.get(cluster, 'licenseInfo.provider') || _.get(cluster, 'agentCloudProvider');
            const licenseType = provider ? PROVIDERS[provider.toUpperCase()] : BYOL;
            const dataAvailable = (status === CLUSTER_STATUS.ON || status === CLUSTER_STATUS.NO_TIERING);
            return {
                clusterName: _.get(cluster, 'name') || 'unknown',
                tieredCapacity: dataAvailable ? tieredCapacityString : '---',
                licenseInfo: dataAvailable ? getClusterLicenseInfo(cluster) : '---',
                licenseType: licenseType,
                shouldEnforce: checkShouldEnforce(cluster),
                clusterStatus: status,
                resourceType: dataAvailable ? DASHBOARD_FILTER_TYPE[cluster.resourceType] : '---',
                hasBYOL: cluster.licenseInfo?.hasBYOL,
                clusterId: cluster.clusterId,
                agentId: cluster.agentId,
                isEligibleForTiering: cluster.isEligibleForTiering,
                clusterNameTooltip: getClusterNameTooltip(cluster)
            };
        });
        return [mappedCluster, providersCount]
    }, [clusters])
    const showClusters = useMemo(() => {
        if (filter) {
            const sortedClusters = _.sortBy(mappedClusters, [filter.column]);
            return filter.ascending ? sortedClusters : _.reverse(sortedClusters);
        }
        return mappedClusters
    }, [mappedClusters, filter])

    useHandleOpenDialogOnInit(showClusters)

    const getTableHeader = () => {
        return [
            {
                name: 'clusterName',
                text: 'Cluster Name',
                sort: applySort
            },
            {
                name: 'resourceType',
                text: 'Cluster Type',
                sort: applySort
            },
            {
                name: 'tieredCapacity',
                text: 'Tiered Capacity',
                sort: applySort
            },
            {
                name: 'licenseInfo',
                text: 'License',
                sort: applySort,
                info: {
                    className: 'license-info',
                    text: LICENSE_PAGE_INFO.LICENSE,
                    width: '500px'
                }
            },
            {
                name: 'licenseType.name',
                text: 'Provider',
                sort: applySort
            },
            {
                name: 'action',
                text: '',
            }
        ];
    };

    const renderRow = (row) => {
        if (!row.clusterStatus) {
            return SkeletonLicenseRow
        }
        const unknownOrFailed = (row.clusterStatus === CLUSTER_STATUS.UNKNOWN || row.clusterStatus === CLUSTER_STATUS.FAILED);
        const isNotSGWS = row.licenseType?.name !== 'SGWS'

        return (<div className={classNames('table-rows', {'enforce-or-unknown': row.shouldEnforce || unknownOrFailed})}>
            <td>
                <span className='cluster-name'>
                    <span className='name'>{row.clusterName}</span>
                    {row.clusterNameTooltip && <ReactPopover popoverClass="cluster-error-info" trigger="mouseover"
                                                             container={<button className="error-info" type="button">
                                                                 <Isvg
                                                                     src={require('./../../../assets/svgs/license-page-red-info.svg')}/>
                                                             </button>}>
                        {row.clusterNameTooltip}
                    </ReactPopover>}
                </span>
            </td>
            <td>{row.resourceType}</td>
            <td>{row.tieredCapacity}</td>
            <td>{row.licenseInfo}</td>
            <td>{row.licenseType ? row.licenseType.licenseIcon : ''}</td>
            <td>
                {row.isEligibleForTiering && isNotSGWS && !unknownOrFailed && !row.hasBYOL ? (
                    <Button variant="secondary" styleClass="activate-byol-button" thin={true} onClick={(el) => {
                        openDialog(<ActivateByolDialog clusterInfo={row}/>, el.target);
                    }}>
                        Activate license (BYOL)
                    </Button>) : ''}</td>
        </div>);
    };

    const applySort = (column, ascending) => {
        setFilter({
            ascending,
            column
        })
    }
    if (!showClusters) {
        return <div/>;
    }
    const {subscriptions, daysLeftForTrial} = licenseInfo;
    const subscriptionsByProvider = _.groupBy(subscriptions, 'provider');

    const inFreeTrial = daysLeftForTrial > 0;
    const preFreeTrial = _.isNil(daysLeftForTrial);
    const freeTrialText = inFreeTrial ? "Your 30-day free trial began when you set up data tiering" : preFreeTrial ? "Your 30-day free trial will begin when you set up data tiering" : "";
    const emptyClusters = showClusters && showClusters.length === 0;

    // console.log(`%c LICENSE got new data licenseInfo ${JSON.stringify(licenseInfo)}`, 'font-weight:bold;color:blue');
    return (<div className="license-page" id="license">
            <div className="market-place center">
                <div className="pay-go container">
                    <LicenseTitle title="Pay As You Go"><ReactPopover popoverClass="pay-go-info" trigger="mouseover"
                                                                      contentStyle={{padding: "20px", width: "400px"}}
                                                                      container={<button className="pay-go-info"
                                                                                         type="button"><Isvg
                                                                          src={require('./../../../assets/svgs/license-page-red-info.svg')}/>
                                                                      </button>}>
                        {<div>
                            <div>Cloud Tiering offers capacity-based licensing in a pay-as-you-go model. You should
                                subscribe through your cloud provider's marketplace even if you have a free trial or if
                                you
                                Bring Your Own License (BYOL).<br/><br/></div>
                            <div>Subscribe to the Cloud Tiering service to ensure that there’s no disruption of service
                                after your free trial ends. When the trial ends, you’ll be charged hourly according to
                                the
                                amount of data that you tier.<br/><br/></div>
                            <div>{`We recommend that you set up a ${PAYGO} subscription, even if you purchase a BYOL license . If your license expires or if you tier more data than allowed by the license, then data tiering is never interrupted—it continues through your ${PAYGO} subscription. For example, if you have a 10 TB license, all capacity beyond the 10 TB is charged through the ${PAYGO} subscription.`}</div>
                        </div>}
                    </ReactPopover>
                        <div className="free-trial-text">{freeTrialText}</div>
                    </LicenseTitle>
                    <div className="subscriptions">
                        <LicenseCard type="AWS" subscribed={subscriptionsByProvider['AWS'][0].status === SUBSCRIBED}
                                     associatedClusters={providersCount['AWS']}/>
                        <LicenseCard type="Azure"
                                     subscribed={subscriptionsByProvider['AZURE'][0].status === SUBSCRIBED}
                                     associatedClusters={providersCount['AZURE']}/>
                        <LicenseCard type="GCP" subscribed={subscriptionsByProvider['GCP'][0].status === SUBSCRIBED}
                                     associatedClusters={providersCount['GCP']}/>
                    </div>
                </div>
            </div>
            <div className="license-content center">
                <LicenseTitle title="Clusters List" styleClass="no-margin"/>
                {!emptyClusters && <div className="cluster-table"><SearchableSelectableTable Table={{
                    renderRow,
                    getTableHeader
                }}
                                                                                             containerOptions={{
                                                                                                 singularTitle: 'Cluster',
                                                                                                 pluralTitle: 'Clusters',
                                                                                                 rowHeight: 55,
                                                                                             }}
                                                                                             data={showClusters}
                                                                                             searchKeys={['clusterName']}
                /></div>}
                {emptyClusters && <div className="no-clusters">
                    <div className="empty-icon"><Isvg src={require('../../../assets/svgs/no-clusters.svg')}/></div>
                    <div className="empty-upper"></div>
                    <div className="empty-lower">There are no clusters in the system</div>

                </div>}
            </div>
        </div>
    );
}

const LicenseTab = (props) => {
    const {accountId, agentId, workspaceId} = props
    const {loading: loadingLicenseInfo, error: errorLicenseInfo, data: dataLicenseInfo} = useQuery(GET_LICENSE_INFO_QUERY, {
        fetchPolicy: 'cache-only',
        variables: {accountId}
    })
    const {loading: loadingClusters, error: errorClusters, data: dataClusters} = useQuery(GET_LAZY_CLUSTERS_QUERY, {
        fetchPolicy: "cache-only",
        variables: {
            accountId,
            agentId,
            workspaceId
        }
    })


    if (loadingClusters || loadingLicenseInfo) {
        return (<LoadingIcon text={'License Info'}/>);
    }
    if (errorClusters || errorLicenseInfo) {
        props.dispatch({
            type: GENERAL_ERROR,
            code: ERROR_SERVER_ERROR,
            payload: errorClusters || errorLicenseInfo
        });
        routesObject.routes.push(`/`);
        return <div></div>;
    } //in case of an error, return to dashboard

    const clusters = !errorClusters && dataClusters.lazyClusters ? dataClusters.lazyClusters : null;
    const licenseInfo = !errorLicenseInfo && dataLicenseInfo.licenseInfo ? dataLicenseInfo.licenseInfo : null;
    return (<LicensePage licenseInfo={licenseInfo} clusters={clusters} {...props}/>);

};

const mapStateToProps = state => ({
    accountId: state.general.accountId,
    agentId: state.general.agentId,
    workspaceId: state.general.workspaceId
});
const mapDispatchToProps = dispatch => ({dispatch})

export default connect(mapStateToProps, mapDispatchToProps)(LicenseTab);
