import React, {useState, useEffect, useCallback} from 'react';
import {connect} from 'react-redux';
import {Link} from 'react-router-dom';
import mapStateToProps from '../wizard/wizardMapStateToProps';
import mapDispatchToProps from '../wizard/wizardMapDispatchToProps';
import Isvg from 'react-inlinesvg';
import {GET_CLUSTER_QUERY, GET_VOLUMES_QUERY} from '../../graphql/queries';
import SearchableSelectableTable, {HEADER_TYPES} from '../table/SearchableSelectableTable';
import _ from 'lodash';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import LoadingIcon from '../widgets/LoadingIcon';
import classNames from 'classnames';
import {
    buildCapacityObject,
    extractByType,
    extractError,
    extractONTAPVersion,
    getWizardRoute,
    sendMixPanelEvent
} from '../../utils/utils';
import InfoPopover from '../reactPopover/InfoPopover';
import {ERROR_SERVER_ERROR, GENERAL_ERROR, routesObject} from '../../actions';
import {openDialog} from '../dialog/Dialog';
import {
    MAX_VOLUMES_IN_PAGE,
    NOTIFICATION_TYPES,
    TIERING_POLICY,
    TIERING_POLICY_NONE,
    VOLUMES_STEP_INFO
} from '../../utils/consts';
import TieringPolicyDialog from './TieringPolicyDialog';
import EditVolumePanel from './EditVolumePanel';
import OutsideClickHandler from 'react-outside-click-handler';
import {reduxForm} from 'redux-form'
import CloseWizardDialog from '../wizard/CloseWizardDialog';
import {parse} from 'query-string';
import NotificationWidget from '../widgets/NotificationWidget';
import Button from "../widgets/Button";
import Breadcrumbs from "../widgets/Breadcrumbs";
import {useLazyQuery} from '@apollo/client'

require('./sourceVolumesStep.scss');

const buildVolumes = (volumes) => {
    const isVolumeDisabled = (volume) => !volume.isEligibleForTiering;

    const getAllTieredVolumes = (volumes) => {
        return _.compact(_.map(volumes, volume => {
            if (volume.isTiered) {
                return volume;
            }
            return null;
        }));
    }
    const getNotEligibleForTieringVolumes = (volumes) => {
        return _.compact(_.map(volumes, volume => {
            if (!volume.isEligibleForTiering) {
                return volume;
            }
            return null;
        }));

    }
    const getNotTieredVolumes = (volumes) => {
        return _.compact(_.map(volumes, volume => {
            if (!isVolumeDisabled(volume) && !volume.isTiered) {
                return volume;
            }
            return null;
        }));
    }
    const getStateWithVolumesAvailability = (showVolumes) => {
        const updatedVolumes = _.map(showVolumes, (volume) => {
            const {isDisabled, ...rest} = volume;
            return {
                isDisabled: isVolumeDisabled(volume),
                ...rest
            };
        });

        const tieredVolumes = getAllTieredVolumes(updatedVolumes);
        const notEligible = getNotEligibleForTieringVolumes(updatedVolumes);
        const notTiered = getNotTieredVolumes(updatedVolumes);

        const tierStatusFilters = [
            {
                name: 'available',
                text: 'Not Tiered Volumes',
                count: notTiered.length,
                checked: true
            },
            {
                name: 'tiered',
                text: 'Tiered Volumes',
                count: tieredVolumes.length,
                checked: true
            },
            {
                name: 'unavailable',
                text: 'Unavailable for Tiering',
                count: notEligible.length,
                checked: true
            }
        ];

        return {
            showVolumes: _.sortBy(updatedVolumes, ['isDisabled']),
            // allSelected: false,
            tiered: tieredVolumes,
            unavailable: notEligible,
            available: notTiered,
            tierStatusFilters
        };
    }
    const volumesAvailability = getStateWithVolumesAvailability(volumes);
    return {...volumesAvailability};
}


const VolumesTable = ({volumes, openEditVolume, editable}) => {
    const [page, setPage] = useState(0);
    const [state, setState] = useState(() => {
        return buildVolumes(volumes);
    });

    useEffect(() => {
        setState((prevState) => ({...prevState, ...buildVolumes(volumes)}))
    }, [volumes])

    const applyFilter = (filters, sort) => {
        const updatedTierStatusFilters = state.tierStatusFilters.map(value => {
            const filter = _.find(filters, {name: value.name});
            value.checked = filter.checked;
            return value;
        });
        const showVolumes = _.reduce(filters, (volumes, filter) => {
            if (filter.checked) {
                return [...volumes, ...state[filter.name]];
            } else {
                return volumes;
            }
        }, []);
        if (!sort) {
            setState(prevState => ({...prevState, tierStatusFilter: updatedTierStatusFilters, showVolumes}));
        } else {
            setState((prevState) => ({...prevState, tierStatusFilter: updatedTierStatusFilters}));
            applySort(sort.column, sort.ascending, showVolumes);
        }
    }
    const applySort = (column, ascending, volumes) => {
        const sortedVolumes = _.sortBy(volumes ? volumes : state.showVolumes, [column]);
        setState(prevState => ({...prevState, showVolumes: ascending ? sortedVolumes : _.reverse(sortedVolumes)}));
    }
    const getTableHeader = () => {
        return [
            {
                className: 'check'
            },
            {
                name: 'name',
                text: 'Volume Name',
                sort: applySort
            },
            {
                name: 'svmName',
                text: 'SVM Name',
                sort: applySort
            },
            {
                name: 'capacity',
                text: 'Volume Size',
                sort: applySort
            },
            {
                name: 'usedPerformanceAndTieredCapacity',
                text: 'Used Size',
                sort: applySort,
                info: {
                    text: VOLUMES_STEP_INFO.USED_SIZE,
                    width: '500px'
                }
            },
            {
                className: 'cold-data',
                name: 'inactiveCapacity',
                text: state.isColdDataEstimated ? 'Cold Data (Estimated)' : 'Cold Data',
                sort: applySort,
                info: {
                    text: state.isColdDataEstimated ? VOLUMES_STEP_INFO.COLD_DATA_ESTIMATED : VOLUMES_STEP_INFO.COLD_DATA,
                    width: '410px'
                }
            },
            {
                text: 'Tier Status',
                className: 'tier-status',
                type: HEADER_TYPES.FILTER,
                filterOptions: {
                    filterName: 'tierStatus',
                    filter: applyFilter,
                    filterValues: state.tierStatusFilters
                },
                sort: applySort //must give the sort method to allow sorting after the filter is applied
            },
            {
                text: 'Tiering Policy',
                className: 'policy',
                sort: applySort //must give the sort method to allow sorting after the filter is applied
            },
            {
                className: 'pencil',
                name: '',
                text: ''
            }
        ];
    }
    const renderRow = (volume) => {
        const showTiered = volume.isTiered;
        const showNotEligible = !volume.isEligibleForTiering;
        const policy = volume.tieringPolicy === 'none' ? TIERING_POLICY_NONE : TIERING_POLICY[volume.tieringPolicy];

        return (
            <div className="table-rows ">
                <td className="check"/>
                <td className="volume-name">
                    <div title={volume.name} className="no-wrap">{volume.name}</div>
                </td>
                <td className="svm-name no-wrap" title={volume.svmName}>{volume.svmName}</td>
                <td className="volume-size">{volume.capacityText}</td>
                <td className="used-size">{volume.usedPerformanceAndTieredCapacityText}</td>
                <td className="cold-data"><span className="no-wrap"
                                                title={volume.inactiveCapacityText}>{volume.inactiveCapacityText}</span><span>{volume.inactiveCapacityPercentText}</span>
                </td>
                {volume.isDisabled && <td className={classNames('info',
                    {
                        'not-eligible': !volume.isEligibleForTiering
                    })}>
                    <div>
                        {showNotEligible && <InfoPopover contentStyle={{width: '360px'}}
                                                         textElement={<span className="italic not-eligible">Unavailable for Tiering</span>}
                                                         popoverClassName="not-eligible" wrapperClassName="not-eligible"
                                                         popoverText={volume.tieringRestrictionReason}/>}
                    </div>
                </td>}
                {!volume.isDisabled && !showTiered && <td className="info">Not Tiered Volume</td>}
                {!volume.isDisabled && showTiered && <td className="info tiered">
                    <div>
                        {showTiered &&
                        <Isvg src={require('./../../assets/svgs/checkmark-fill-circle-icon.svg')}></Isvg>}
                        {showTiered ? 'Tiered Volume' : ''}
                    </div>

                </td>}
                <td className={classNames('policy', {disabled: volume.isDisabled})}>
                    {policy.description}
                </td>
                <td className="pencil"
                    onClick={() => volume.isDisabled || !editable ? _.noop : openEditVolume(volume)}>
                    {editable && !volume.isDisabled &&
                    <div className="pencil"><Isvg src={require('./../../assets/svgs/pencil.svg')}/></div>}
                </td>
            </div>
        );
    }
    return (
        <div className="volumes">
            <SearchableSelectableTable
                Table={{
                    getTableHeader,
                    renderRow
                }}
                data={state.showVolumes}
                searchKeys={['name', 'svmName', 'capacityText', 'usedPerformanceAndTieredCapacityText', 'inactiveCapacityText', 'inactiveCapacityPercentText']}
                containerOptions={{
                    singularTitle: 'Volume',
                    pluralTitle: 'Volumes',
                    maxElementsOnPage: MAX_VOLUMES_IN_PAGE,
                    keepPage: (page) => setPage(page),
                    startFromPage: page,
                    rowHeight: 55,
                    info: {
                        text: VOLUMES_STEP_INFO.DISPLAYED_VOLUMES,
                        width: '500px'
                    }
                }}>
            </SearchableSelectableTable>
        </div>
    )
}

const SourceVolumesStep = (props) => {
    const {up, match: {params}, accountId, agentId, workspaceId} = props;
    const clusterId = params.clusterId;
    const clusterName = params.clusterName;
    const origin = _.get(parse(window.location.search), 'origin');
    const [state, setState] = useState({hideClose: ['occm', 'dashboard', 'moreInfo'].includes(origin)});

    //handle GET_CLUSTER
    const [getCluster, {loading: getClusterLoading, error: getClustersError, data: getClusterData}] = useLazyQuery(GET_CLUSTER_QUERY);
    useEffect(() => {
        if (clusterId) {
            getCluster({
                variables: {
                    clusterId,
                    accountId,
                    agentId,
                    workspaceId
                },
                fetchPolicy: 'network-only'
            });
        }
    }, [clusterId, getCluster, accountId, workspaceId, agentId]);

    useEffect(() => {
        if (getClustersError) {
            setState((prevState) => ({...prevState, error: extractError(getClustersError, 'Source Volumes Step')}));
        }
        if (!state?.ontapVersion && getClusterData?.cluster) {
            const {cluster} = getClusterData;
            const wizardCluster = extractByType(cluster);
            if (wizardCluster) {
                const ontapVersion = extractONTAPVersion(wizardCluster.ontapClusterProperties.ontapVersion);
                const policies = wizardCluster.volumes.supportedPolicies;
                const editable = !!_.get(wizardCluster, 'tierInformation.bucketName') && wizardCluster.resourceType === 'ON_PREM';
                const policiesWithMoreInfo = _.compact(_.map(policies, (policy) => {
                    const {name} = policy;
                    return name === 'none' ? null : {
                        ...(TIERING_POLICY[name]),
                        ...policy
                    };
                }));
                const {capacityInformation} = wizardCluster;
                const isColdDataEstimated = _.get(capacityInformation, 'performanceCapacity.isEstimatedInactiveCapacity', false);
                setState((prevState) => ({
                    ...prevState,
                    editable,
                    ontapVersion,
                    isColdDataEstimated,
                    policies: _.sortBy(policiesWithMoreInfo, 'order'),
                    totalVolumes: wizardCluster.volumes.totalVolumes,
                    wizardCluster
                }));
            }
        }
    }, [getClusterLoading, getClustersError, getClusterData, state.error, state.ontapVersion])

    //handle GET_VOLUMES query
    const [getVolumes, {loading, error, data}] = useLazyQuery(GET_VOLUMES_QUERY);
    const queryCompleted = useCallback((data) => {
        if (!data) {
            return;
        }
        const volumes = _.map(data.volumes, volume => {
            const {supportedSettings, ...rest} = volume;
            const updatedPolicies = _.map(supportedSettings, (supportedSetting) => {
                const {tieringPolicy: policy} = supportedSetting;
                const uiPolicy = policy === 'none' ? TIERING_POLICY_NONE : TIERING_POLICY[policy];
                return {
                    ...uiPolicy
                };
            });
            const capacityObject = buildCapacityObject(_.get(volume, 'capacity', 0));
            const usedPerformanceAndTieredCapacityObject = buildCapacityObject(_.get(volume, 'usedPerformanceAndTieredCapacity', 0));
            const inactiveCapacityObject = buildCapacityObject(_.get(volume, 'inactiveCapacity', 0));
            return {
                capacityText: `${capacityObject.value} ${capacityObject.unit}`,
                usedPerformanceAndTieredCapacityText: `${usedPerformanceAndTieredCapacityObject.value} ${usedPerformanceAndTieredCapacityObject.unit}`,
                inactiveCapacityText: `${inactiveCapacityObject.value} ${inactiveCapacityObject.unit}`,
                inactiveCapacityPercentText: `${volume.inactiveCapacityPercent} %`,
                supportedPolicies: updatedPolicies,
                supportedSettings,
                ...rest
            };
        });
        setState((prevState) => ({...prevState, showVolumes: volumes}))
    }, [])

    useEffect(() => {
        if (clusterId) {
            getVolumes({
                variables: {
                    accountId,
                    agentId,
                    clusterId,
                    workspaceId
                },
                fetchPolicy: 'cache-first',
                notifyOnNetworkStatusChange: true
            })
        }
    }, [clusterId, agentId, accountId, getVolumes, workspaceId])

    useEffect(() => {
        if (!loading && !error && data) {
            queryCompleted(data)
        }
    }, [loading, error, data, queryCompleted])

    const next = () => {
        sendMixPanelEvent("VolumesStepContinue");
        props.history.push('/clusters');
    }

    const closeEditVolume = () => {
        setTimeout(() => setState((prevState) => ({...prevState, editVolumeOpen: false})));
    }

    const openEditVolume = (volume) => {
        setState(prevState => ({
            ...prevState,
            editVolumeOpen: true,
            editVolume: volume
        }));
    }

    const shouldOpenDiscardDialog = () => {
        const atLeastOneTiered = _.find(state.showVolumes, {isTiered: true});
        return !atLeastOneTiered;
    }

    const {error: queryError, editable, ontapVersion, policies, showVolumes, wizardCluster, hideClose} = state;
    let nextRoute = null;
    if (!editable && wizardCluster) {
        const {tierInformation, name, agentCloudProvider, ontapClusterProperties, agentId} = wizardCluster;
        const wizardRoute = getWizardRoute(tierInformation, clusterId, name, agentCloudProvider, ontapClusterProperties, agentId);
        nextRoute = `/${wizardRoute}?origin=dashboard`;
    }

    if (error) {
        props.dispatch({
            type: GENERAL_ERROR,
            code: ERROR_SERVER_ERROR,
            payload: error
        });
        routesObject.routes.push(`/`);
        return <div></div>;
    } //in case of an error, return to dashboard
    if (!ontapVersion || !policies || !showVolumes || loading) {
        return (<LoadingIcon text={'Volumes'} className="loading-volumes"/>);
    }
    return (
        <div className={classNames('volumes-step new-wizard-step-full-screen', {
            up,
            down: !up
        })}>
            <div className="step-title">
                <div className="left-side">
                    {origin === 'dashboard' && <Breadcrumbs levels={[{
                        text: 'Cluster Dashboard',
                        onClick: next
                    }, {text: `${clusterName} volumes`}]}/>}
                </div>
                <h1>{editable ? 'Tier' : 'View'} Volumes</h1>
                <div className="right-side">
                    <div className="learn-more-button"
                         onClick={(el) => {
                             openDialog(<TieringPolicyDialog className="tiering-policy-dialog"
                                                             policies={policies}
                                                             totalVolumes={state.totalVolumes}></TieringPolicyDialog>, el.target);
                         }}>{'Learn how much you can save with each Tiering Policy'}</div>
                </div>
            </div>
            <form className="volumes-form">
                <VolumesTable volumes={showVolumes} openEditVolume={openEditVolume} editable={editable}/>
                {state.editVolumeOpen && <div className='slider-mask show-mask'/>}
                <OutsideClickHandler
                    onOutsideClick={state.editVolumeOpen ? closeEditVolume : _.noop}>
                    <EditVolumePanel
                        isOpen={state.editVolumeOpen}
                        close={closeEditVolume}
                        editVolume={state.editVolume}
                        clusterId={clusterId}
                    />
                </OutsideClickHandler>
            </form>
            {!editable && <div className="notification-widget-container">
                <NotificationWidget variant={NOTIFICATION_TYPES.WARNING}>
                    {wizardCluster.resourceType === 'ON_PREM' &&
                    <div>This table enables you to view information about your volumes. To start tiering
                        data from the volumes, you'll need to finish your <Link to={nextRoute}>cluster
                            setup.</Link></div>}
                    {wizardCluster.resourceType === 'CVO' &&
                    <div>This table enables you to view information about your volumes. You can change
                        tiering policies from Cloud Manager.</div>}
                </NotificationWidget>
            </div>}
            {!hideClose && <div className="wizard-button">
                <ButtonsPanelWithError error={queryError}>
                    <Button onClick={(el) => {
                        if (shouldOpenDiscardDialog()) {
                            openDialog(<CloseWizardDialog className="close-wizard-dialog"
                                                          discardWizard={next} {...props}
                                                          currentStepName="Tier Volumes"/>, el.target);
                        } else {
                            next();
                        }

                    }}>Close</Button>
                </ButtonsPanelWithError>
            </div>}
        </div>
    );

}

const SourceVolumesStepForm = reduxForm({
    form: 'volumes'
})(SourceVolumesStep);


export default connect(mapStateToProps, mapDispatchToProps)(SourceVolumesStepForm);


