import React from 'react';
import ReactDOM from 'react-dom';
import {Provider} from 'react-redux';
import {ConnectedRouter, push} from 'connected-react-router';
import {bindActionCreators} from 'redux';
import {Route, Switch} from 'react-router-dom';
import {
    ApolloProvider,
    ApolloClient,
    createHttpLink,
    defaultDataIdFromObject,
    InMemoryCache,
    split
} from "@apollo/client"
import {WebSocketLink} from "@apollo/client/link/ws"
import {RetryLink} from "@apollo/client/link/retry"
import {setContext} from '@apollo/client/link/context';
import {onError} from '@apollo/client/link/error';
import {getMainDefinition} from '@apollo/client/utilities';
import possibleTypes from './graphql/fragmentTypes.json';
import * as serviceWorker from './serviceWorker';
import App from './App';
import {history, store} from './store';
import * as actions from './actions';
import numeral from 'numeral';
import _ from 'lodash';
import {getAuthToken} from './auth/authMiddleware';
import localStorage from './utils/localStorage';
import './index.scss';
import {NOTIFICATION_ERROR_CODE} from './utils/consts';
import {DialogContainer} from './components/dialog/Dialog';
import {IsInIframe} from "./utils/utils";

// Redirect to OCCM if accessed directly (not within Iframe)
if (process.env.REACT_APP_DEVELOPMENT_MODE !== 'true' || window.localStorage.getItem('redirectOnStandalone') === 'true') {
    if (!IsInIframe()) {
        window.location = process.env.REACT_APP_OCCM_REDIRECT_URL
    }
}

//handle auth issues
const Auth = require('./auth/Auth.js').default;
const auth0Options = {
    clientID: process.env.REACT_APP_AUTH_CLIENT,
    domain: process.env.REACT_APP_AUTH_DOMAIN,
    audience: process.env.REACT_APP_AUTH_AUDIENCE
};
console.log(`%c connecting to auth0 with ${JSON.stringify(auth0Options)}`, 'font-weight:bold;color:green');
window.auth = new Auth(auth0Options);

actions.routesObject.routes = {push: bindActionCreators(push, store.dispatch)};
store.dispatch({type: actions.AUTH_REFRESH_SSO}); //start login flow

Object.assign(numeral.locales['en'].abbreviations, {
    thousand: 'K',
    million: 'M',
    billion: 'B',
    trillion: 'T'
});

const cache = new InMemoryCache({
    possibleTypes,
    dataIdFromObject: object => {
        switch (object.__typename) {
            case 'Cluster':
                const id = `${object.__typename}:${object.clusterId}`;
                // console.log(`%c dataIdFromObject to Cluster ${id}`, 'font-weight:bold;color:brown');
                return id;
            case 'Volume':
                return `${object.__typename}:${object.name}:${object.svmName}`;
            case 'Region':
                return `${object.__typename}:${object.code}`;
            case 'Bucket':
                return `${object.__typename}:${object.name}`;
            case 'Aggregate':
                return `${object.__typename}:${object.name}`;
            default:
                return defaultDataIdFromObject(object); // fall back to default handling
        }
    }
});

//listen to url changes and send google analytics events
history.listen(location => {
    window.ga && window.ga('send', 'pageview', location.pathname);
    // sendMixPanelEvent('pageview', {path: location.pathname});
});

const noPort = localStorage.get('noPort');
//port 3000 is reserved for UI proxy
const host = noPort || document.location.port === '3000' ? `${document.location.hostname}` : `${document.location.hostname}:${document.location.port}`;
const CLOUD_TIERING_BASE_URL = `${document.location.protocol}//${host}/graphql`;
const CLOUD_TIERING_GRAPHQL_SUBSCRIPTION_URL = `${document.location.protocol === 'https:' ? 'wss' : 'ws'}://${host}/graphql`;

const httpLink = createHttpLink({
    uri: CLOUD_TIERING_BASE_URL
});

const authLink = setContext((_, {headers}) => {
    const contextHeaders = {
        ...headers
    };
    // get the authentication token from local storage if it exists
    const authorization = getAuthToken();
    if (authorization) {
        contextHeaders.authorization = authorization;
    }
    // return the contextHeaders to the context so httpLink can read them
    return {headers: contextHeaders};
});

const errorHandler = reduxStore => onError((error) => {
    const {graphQLErrors, networkError} = error;
    const htmlError = _.get(networkError, "bodyText", "").indexOf("!DOCTYPE html") >= 0;
    if (htmlError) {
        console.log('html error detected from ngnix..\n\n' + networkError.bodyText);
        reduxStore.dispatch({
            type: actions.GENERAL_ERROR,
            code: actions.ERROR_SERVER_ERROR,
            payload: "General Error, please contact support."
        });
    } else if (networkError) {
        const errorCode = _.get(networkError, 'result.errors[0].extensions.code', '');
        const errorMessage = _.get(networkError, 'result.errors[0].message', '');
        switch (errorCode) {
            case 400: {
                reduxStore.dispatch({
                    type: actions.GENERAL_ERROR,
                    code: actions.ERROR_BAD_REQUEST,
                    payload: errorMessage
                });
                break;
            }
            case 401: {
                console.log(`%c Failed authenticating user, error: ${errorMessage} logging out`, 'font-weight: bold; color: red');
                break;
            }
            default: {
                console.log('Internal error detected from server..\n\n' + networkError);
                reduxStore.dispatch({
                    type: actions.GENERAL_ERROR,
                    code: actions.ERROR_SERVER_ERROR,
                    payload: networkError.message
                });
                break;
            }
        }
    }
    if (graphQLErrors) {
        _.each(graphQLErrors, ({message, locations, extensions, path}) => {
            const code = _.get(extensions, 'code');
            if (path && path.indexOf('clusters') > -1 && !_.isEqual(code, NOTIFICATION_ERROR_CODE)) { //only clusters errors are relevant. all other errors will be shown under the button
                switch (extensions.code) {
                    case 400: {
                        reduxStore.dispatch({
                            type: actions.GENERAL_ERROR,
                            code: actions.ERROR_BAD_REQUEST,
                            payload: message
                        });
                        break;
                    }
                    case 401: {
                        reduxStore.dispatch({
                            type: actions.GENERAL_ERROR,
                            code: actions.ERROR_AUTHENTICATION,
                            payload: message
                        });
                        break;
                    }
                    case 403: {
                        reduxStore.dispatch({
                            type: actions.GENERAL_ERROR,
                            code: actions.ERROR_FORBIDDEN,
                            payload: message
                        });
                        break;
                    }
                    case 404: {
                        reduxStore.dispatch({
                            type: actions.GENERAL_ERROR,
                            code: actions.ERROR_NOT_FOUND,
                            payload: message
                        });
                        break;
                    }
                    default: {
                        console.log(`%c got GraphQL error ${message} in locations ${JSON.stringify(locations)} and path ${path} url ${JSON.stringify(window.location)}`, 'font-weight: bold; color: red');
                        reduxStore.dispatch({
                            type: actions.GENERAL_ERROR,
                            code: actions.ERROR_SERVER_ERROR,
                            payload: message
                        });
                    }
                }

            }
        });
    }
});

const regularLink = errorHandler(store)
    .concat(authLink)
    .concat(httpLink);

const winSocketLink = new WebSocketLink({
    uri: CLOUD_TIERING_GRAPHQL_SUBSCRIPTION_URL,
    options: {
        reconnect: true,
        lazy: true,
        connectionParams: () => {
            return {authorization: getAuthToken() || ''};
        },
        connectionCallback: (error) => {
            if (error) {
                if (error.message !== 'Invalid JWT token') {
                    console.log(`%c got error on connecting web socket ${JSON.stringify(error)}`, 'font-weight:bold;color:red');
                }
            }
        }
    }
});

const retryLink = new RetryLink({
    delay: {
        initial: 300,
        max: Infinity,
        jitter: true
    },
    attempts: {
        max: 5,
        retryIf: (error) => {
            if (error) {
                console.log(`%c got error ${JSON.stringify(error, null, 2)} retrying`, 'font-weight:bold; color:red');
            }
            return !!error
        }
    }
});

const retryableWinSocketLink = retryLink.concat(winSocketLink);

const client = new ApolloClient({
    link: split(// split based on operation type
        ({query}) => {
            const definition = getMainDefinition(query);
            return (
                definition.kind === 'OperationDefinition' &&
                definition.operation === 'subscription'
            );
        },
        retryableWinSocketLink,
        regularLink),
    cache,
    connectToDevTools: true
});

ReactDOM.render(
    <ApolloProvider client={client}>
        <Provider store={store}>
            <ConnectedRouter history={history}>
                <DialogContainer/>
                <Switch>
                    <Route path="/" component={App}/>
                </Switch>
            </ConnectedRouter>
        </Provider>
    </ApolloProvider>,
    document.getElementById('root'));

// If you want your app to work offline and load faster, you can change
// unregister() to register() below. Note this comes with some pitfalls.
// Learn more about service workers: http://bit.ly/CRA-PWA
serviceWorker.unregister();
