import * as utils from '../../src/utils/utils'
import { buildCapacityObject } from '../../src/utils/utils';
import { FREE_TRIAL, FREE_TRIAL_EXPIRED, SUBSCRIBED,} from '../../src/utils/consts';
import { getClusterLicenseInfo } from '../../src/utils/utils';
import { getStorageClassNameByProvider } from '../../src/utils/utils';

describe('defaultFilter', () => {
    test('should return false when item is function', () => {
        expect(utils.defaultFilter(() => {
        })).toEqual(false);
    });
    test('should return false for an empty value in object', () => {
        expect(utils.defaultFilter({empty: null}, "exists")).toEqual(undefined);
    });
    test('should return the found object if the filter string exists', () => {
        expect(utils.defaultFilter({find: "found element"}, "element")).toEqual("found element");
    });
    test('should find the filter string even inside a compound object', () => {
        expect(utils.defaultFilter({compound: {find: "inner element"}}, "element")).toEqual({find: "inner element"});
    });
});

describe('defaultFilterWithKeys', function () {
    const item = {
        key1: "value1",
        key2: "value2",
        key3: "value3"
    };
    test('should return false if none of the requested keys exists in the object', () => {
        expect(utils.defaultFilterWithKeys(item, "value3", ["key1", "key2"])).toEqual(undefined);
    });
    test('should return the found key when filter string exists and the key is requested', () => {
        expect(utils.defaultFilterWithKeys(item, "value3", ["key3"])).toEqual("key3");
    });
    test('should return undefined when filter string does not exist even if thekey exists', () => {
        expect(utils.defaultFilterWithKeys(item, "not exist", ["key1", "key2", "key3"])).toEqual(undefined);
    });
});

describe('calculateSavings', function () {
    let cluster = {};
    beforeEach(() => {
        cluster = {
            agentCloudProvider: "AWS",
            agentId: null,
            capacityInformation: {
                performanceCapacity: {
                    activeCapacity: 4947802324992,
                    inactiveCapacity: 11544872091648,
                    isEstimatedInactiveCapacity: true,
                    usedPerformanceCapacity: 16492674416640
                },
                savingOpportunity: {
                    cost: 0,
                    capacity: 11544872091648
                },
                usedPerformanceAndTieredCapacity: 20340965113856
            },
            clusterId: "eda35168-d036-4cce-9cc8-ca916ec36326",
            name: "AFF-1",
            ontapClusterProperties: {
                clusterAddress: "192.168.0.1",
                ontapVersion: "9.4"
            },
            status: {
                message: null,
                status: "ON"
            },
            tierInformation: {
                bucketName: "bucket",
                objectStoreName: "tiering_objectStore",
                providerType: "AWS",
                regionName: "US East (Ohio)",
                tierLevel: "Standard-Infrequent Access",
                currentSavings: {
                    cost: 16860,
                    capacity: 3848290697216
                }
            },
            volumes: {
                eligibleVolumes: 16,
                tieredVolumes: 1
            }
        };
    });
    test('should return no available data when cluster is failed', () => {
        cluster.status = {message: "status is off", status: "FAILED"};
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{"italicText": "No Available Data"}],
            "currentSavings": [{"italicText": "No Available Data"}]
        });
    });

    test('should return no available data when cluster is unkown', () => {
        cluster.status = {message: "status is off", status: "UNKNOWN"};
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{"italicText": "No Available Data"}],
            "currentSavings": [{"italicText": "No Available Data"}]
        });
    });
    test('should build the right object when tieredCapacity = 0', () => {
        cluster.tierInformation.currentSavings.capacity = 0;
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{
                "bytes": 11544872091648,
                "unit": "TB",
                "value": "10.5"
            }], "currentSavings": [{"italicText": "Cold data has not been tiered yet"}]
        });
    });
    test('should build the right object when tieredCapacity > 0', () => {
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{
                "bytes": 11544872091648,
                "unit": "TB",
                "value": "10.5"
            }], "currentSavings": [{"bytes": 3848290697216, "unit": "TB", "value": "3.5"}, {"value": "$ 16,860"}]
        });
    });
    test('should build the right object when performanceCapacity.inactiveCapacity = 0', () => {
        cluster.capacityInformation.savingOpportunity.capacity = 0;
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{"italicText": "No Available Data"}],
            "currentSavings": [{"bytes": 3848290697216, "unit": "TB", "value": "3.5"}, {"value": "$ 16,860"}]
        });
    });
    test('should build the right object when performanceCapacity.savingOpportunity.capacity > 0', () => {
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{
                "bytes": 11544872091648,
                "unit": "TB",
                "value": "10.5"
            }], "currentSavings": [{"bytes": 3848290697216, "unit": "TB", "value": "3.5"}, {"value": "$ 16,860"}]
        });
    });
    test('should build the right object when tieredVolumes === 0', () => {
        cluster.capacityInformation.savingOpportunity.capacity = 0;
        cluster.volumes.tieredVolumes = 0;
        expect(utils.calculateSavings(cluster).additionalSavings).toEqual([{"italicText": "Cold data has not been tiered yet"}]);
    });
    test('should build the right object when tieredVolumes === eligible volumes', () => {
        cluster.capacityInformation.savingOpportunity.capacity = 0;
        cluster.volumes.tieredVolumes = 3;
        cluster.volumes.eligibleVolumes = 3;
        expect(utils.calculateSavings(cluster).additionalSavings).toEqual([{"italicText": "Data tiering is enabled on all volumes"}]);
    });

});

describe ('buildCapacityObject', ()=> {
   test('should return null if the input is null', ()=>{
       expect(buildCapacityObject(null)).toEqual(null);
   });
    test('should return the right object with the right value and unit', ()=>{
       expect(buildCapacityObject(120000000)).toEqual({bytes: 120000000, unit: "MB", value: "114.44"});
    });
});

describe('getClusterLicenseInfo', function () {
    let licenseInfo = {};
    beforeEach(() => {
        licenseInfo = {byolCapacity: 0, currentCapacity: 0, freeTrialStatus: FREE_TRIAL_EXPIRED, subscriptions: {}, hasBYOL: false};
    });
    test('should return the right info for free trial', () => {
        licenseInfo.freeTrialStatus = FREE_TRIAL;
        expect(getClusterLicenseInfo(licenseInfo))
            .toEqual("Free trial");
    });
    // test('should return the right info for pre free trial', () => {
    //     licenseInfo.freeTrialStatus = PRE_FREE_TRIAL;
    //     expect(getClusterLicenseInfo(licenseInfo))
    //         .toEqual("");
    // });
    test('should return the right info for subscribed', () => {
        licenseInfo.hasBYOL = false;
        licenseInfo.subscriptionStatus = SUBSCRIBED;
        expect(getClusterLicenseInfo(licenseInfo))
            .toEqual("PAYGO");
    });
    test('should return the right info for not subscribed', () => {
        licenseInfo.hasBYOL = false;
        expect(getClusterLicenseInfo(licenseInfo))
            .toEqual("Free trial is over");
    });
    test('should return the right info for of license capacity exceeded and not subscribed', () => {
        licenseInfo.hasBYOL = true;
        licenseInfo.byolCapacity = 10000000000;
        licenseInfo.currentCapacity = 12000000000;
        expect(getClusterLicenseInfo(licenseInfo))
            .toEqual("9.31 GB of license capacity exceeded");
    });
    test('should return the right info for of license capacity exceeded and subscribed', () => {
        licenseInfo.hasBYOL = true;
        licenseInfo.subscriptionStatus = SUBSCRIBED;
        licenseInfo.byolCapacity = 10000000000;
        licenseInfo.currentCapacity = 12000000000;
        expect(getClusterLicenseInfo(licenseInfo))
            .toEqual("9.31 GB of license capacity + 1.86 GB pay-as-you-go");
    });

    test('should return the right info for of license capacity not exceeded', () => {
        licenseInfo.hasBYOL = true;
        licenseInfo.subscriptionStatus = SUBSCRIBED;
        licenseInfo.byolCapacity = 10000000000;
        licenseInfo.currentCapacity = 9000000000;
        expect(getClusterLicenseInfo(licenseInfo))
            .toEqual("9.31 GB of license capacity");
    });
});

describe('getStorageClassNameByProvider', function () {
    test('should return Standard for AWS and standard', () => {
        expect(getStorageClassNameByProvider('AWS', 'standard'))
            .toEqual("Standard");
    });
    test('should return Standard for AWS and standard_ia', () => {
        expect(getStorageClassNameByProvider('AWS', 'standard_ia'))
            .toEqual("Standard-Infrequent Access");
    });
    test('should return Standard for AZURE and hot', () => {
        expect(getStorageClassNameByProvider('AZURE', 'hot'))
            .toEqual("Hot");
    });
    test('should return Standard for AZURE and cool', () => {
        expect(getStorageClassNameByProvider('AZURE', 'cool'))
            .toEqual("Cool");
    });
    test('should return Standard for GCP and standard', () => {
        expect(getStorageClassNameByProvider('GCP', 'standard'))
            .toEqual("Standard");
    });
});
